@@ -311,7 +311,7 @@ enum SPAttributeEnum {
     /* Conditional */
     SP_PROP_SYSTEM_LANGUAGE,
     SP_PROP_REQUIRED_FEATURES,
-    SP_PROP_REQUIRED_EXTENSIONS,
+    SP_PROP_REQUIRED_EXTENSIONS
 };
 
 #endif
