@@ -14,11 +14,9 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include "forward.h"
-
-#include <glib.h>
-
+#include <gdk/gdktypes.h>
 
+struct SPColorSpace;
 
 /* Useful composition macros */
 
