@@ -49,6 +49,7 @@
 #endif
 
 #include <glibmm/i18n.h>
+#include <sigc++/functors/mem_fun.h>
 
 #include "macros.h"
 #include "inkscape-private.h"
@@ -260,7 +261,9 @@ SPDesktop::init (SPNamedView *nv, SPCanvas *aCanvas)
     _reconstruction_finish_connection =
         document->connectReconstructionFinish(sigc::bind(sigc::ptr_fun(_reconstruction_finish), this));
     _reconstruction_old_layer_id = NULL;
-
+    
+    _commit_connection = document->connectCommit(sigc::mem_fun(*this, &SPDesktop::updateNow));
+    
     // ?
     // sp_active_desktop_set (desktop);
     _inkscape = INKSCAPE;
@@ -1037,6 +1040,12 @@ SPDesktop::emitToolSubselectionChanged(gpointer data)
 	inkscape_subselection_changed (this);
 }
 
+void
+SPDesktop::updateNow()
+{
+  sp_canvas_update_now(canvas);
+}
+
 //----------------------------------------------------------------------
 // Callback implementations. The virtual ones are connected by the view.
 
@@ -1066,6 +1075,7 @@ SPDesktop::onRedrawRequested ()
 /**
  * Associate document with desktop.
  */
+/// \todo fixme: refactor SPDesktop::init to use setDocument
 void
 SPDesktop::setDocument (SPDocument *doc)
 {
@@ -1084,6 +1094,9 @@ SPDesktop::setDocument (SPDocument *doc)
     _layer_hierarchy->connectChanged(sigc::bind(sigc::ptr_fun(_layer_hierarchy_changed), this));
     _layer_hierarchy->setTop(SP_DOCUMENT_ROOT(doc));
 
+    _commit_connection.disconnect();
+    _commit_connection = doc->connectCommit(sigc::mem_fun(*this, &SPDesktop::updateNow));
+
     /// \todo fixme: This condition exists to make sure the code
     /// inside is called only once on initialization. But there
     /// are surely more safe methods to accomplish this.
