@@ -223,6 +223,7 @@ struct SPDesktop : public Inkscape::UI::View::View
     void setToolboxFocusTo (gchar const* label);
     void setToolboxAdjustmentValue (gchar const* id, double val);
     bool isToolboxButtonActive (gchar const *id);
+    void updateNow();
     
     void fullscreen();
 
@@ -264,6 +265,7 @@ private:
     sigc::connection _sel_changed_connection;
     sigc::connection _reconstruction_start_connection;
     sigc::connection _reconstruction_finish_connection;
+    sigc::connection _commit_connection;
     
     virtual void onPositionSet (double, double);
     virtual void onResized (double, double);
