@@ -25,6 +25,7 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtkexpander.h>
 
+#include <unistd.h>
 #include <sys/stat.h>
 #include <glibmm/i18n.h>
 #include <gtkmm/box.h>
@@ -894,7 +895,10 @@ FileOpenDialogImpl::~FileOpenDialogImpl()
 bool
 FileOpenDialogImpl::show()
 {
-    set_current_folder(get_current_folder()); //hack to force initial dir listing
+    Glib::ustring s = Glib::filename_to_utf8 (get_current_folder());
+    if (s.length() == 0) 
+        s = getcwd (NULL, 0);
+    set_current_folder(Glib::filename_from_utf8(s)); //hack to force initial dir listing
     set_modal (TRUE);                      //Window
     sp_transientize((GtkWidget *)gobj());  //Make transient
     gint b = run();                        //Dialog
@@ -1335,14 +1339,16 @@ FileSaveDialogImpl::~FileSaveDialogImpl()
 
 
 
-
 /**
  * Show this dialog modally.  Return true if user hits [OK]
  */
 bool
 FileSaveDialogImpl::show()
 {
-    set_current_folder(get_current_folder()); //hack to force initial dir listing
+    Glib::ustring s = Glib::filename_to_utf8 (get_current_folder());
+    if (s.length() == 0) 
+        s = getcwd (NULL, 0);
+    set_current_folder(Glib::filename_from_utf8(s)); //hack to force initial dir listing
     set_modal (TRUE);                      //Window
     sp_transientize((GtkWidget *)gobj());  //Make transient
     gint b = run();                        //Dialog
