@@ -5,6 +5,7 @@
  *    and enums FileDialogType, FileDialogSelectionType. */
 
 #include <glib/gtypes.h>
+#include <glibmm/slisthandle.h>
 
 namespace Inkscape {
 
@@ -163,9 +164,9 @@ public:
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
