@@ -19,46 +19,22 @@
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
-#include <string.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkmisc.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtktooltips.h>
 
-#include <libnr/nr-values.h>
-#include <libnr/nr-matrix-fns.h>
-#include <libnr/nr-path.h>
 
-#include <glibmm/i18n.h>
-#include <helper/window.h>
-#include <svg/svg.h>
 #include <widgets/sp-widget.h>
-#include <sp-gradient.h>
 #include <sp-linear-gradient.h>
 #include <sp-pattern.h>
 #include <sp-radial-gradient.h>
-#include <sp-use.h>
 #include <widgets/paint-selector.h>
 #include <style.h>
 #include <gradient-chemistry.h>
-#include <document.h>
 #include <desktop-style.h>
 #include <desktop-handles.h>
 #include <selection.h>
-#include <sp-item.h>
 #include <inkscape.h>
 #include <document-private.h>
-#include <file.h>
 #include <xml/repr.h>
 
-#include "fill-style.h"
 
 // These can be deleted once we sort out the libart dependence.
 
@@ -154,7 +130,7 @@ sp_fill_style_widget_construct ( SPWidget *spw, SPPaintSelector *psel )
 
         sp_fill_style_widget_update (spw);
 
-    } 
+    }
 
 } // end of sp_fill_style_widget_construct()
 
@@ -173,7 +149,7 @@ sp_fill_style_widget_modify_selection ( SPWidget *spw,
 }
 
 static void
-sp_fill_style_widget_change_subselection ( Inkscape::Application *inkscape, 
+sp_fill_style_widget_change_subselection ( Inkscape::Application *inkscape,
                                         SPDesktop *desktop,
                                         SPWidget *spw )
 {
@@ -348,7 +324,7 @@ sp_fill_style_widget_paint_dragged (SPPaintSelector *psel, SPWidget *spw)
         case SP_PAINT_SELECTOR_MODE_COLOR_CMYK:
         {
             sp_paint_selector_set_flat_color (psel, SP_ACTIVE_DESKTOP, "fill", "fill-opacity");
-            sp_document_maybe_done (SP_DT_DOCUMENT(SP_ACTIVE_DESKTOP), undo_label);
+            sp_document_maybe_done (sp_desktop_document(SP_ACTIVE_DESKTOP), undo_label);
             g_object_set_data (G_OBJECT (spw), "local", GINT_TO_POINTER (TRUE)); // local change, do not update from selection
             break;
         }
@@ -383,8 +359,8 @@ sp_fill_style_widget_paint_changed ( SPPaintSelector *psel,
     if (!desktop) {
         return;
     }
-    SPDocument *document = SP_DT_DOCUMENT (desktop);
-    Inkscape::Selection *selection = SP_DT_SELECTION (desktop);
+    SPDocument *document = sp_desktop_document (desktop);
+    Inkscape::Selection *selection = sp_desktop_selection (desktop);
 
     GSList const *items = selection->itemList();
 
@@ -417,12 +393,12 @@ sp_fill_style_widget_paint_changed ( SPPaintSelector *psel,
         case SP_PAINT_SELECTOR_MODE_COLOR_CMYK:
         {
             sp_paint_selector_set_flat_color (psel, desktop, "fill", "fill-opacity");
-            sp_document_maybe_done (SP_DT_DOCUMENT(desktop), undo_label);
+            sp_document_maybe_done (sp_desktop_document(desktop), undo_label);
 
             // on release, toggle undo_label so that the next drag will not be lumped with this one
             if (undo_label == undo_label_1)
                 undo_label = undo_label_2;
-            else 
+            else
                 undo_label = undo_label_1;
 
             break;
