@@ -16,15 +16,9 @@
 
 void sp_object_properties_dialog            (void);
 
-void sp_object_properties_stroke            (void);
 void sp_object_properties_fill              (void);
-void sp_object_properties_layout            (void);
-
-void sp_object_properties_reread_page       (void);
-void sp_object_properties_selection_changed (void);
-void sp_object_properties_close             (void);
-void sp_object_properties_apply             (void);
-
+void sp_object_properties_stroke            (void);
+void sp_object_properties_stroke_style   (void);
 
 #endif
 
@@ -32,9 +26,9 @@ void sp_object_properties_apply             (void);
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
