@@ -15,20 +15,15 @@
 #define noNR_ARENA_ITEM_VERBOSE
 #define noNR_ARENA_ITEM_DEBUG_CASCADE
 
-#include <string.h>
-#include <assert.h>
 
-#include <libnr/nr-macros.h>
-#include <libnr/nr-rect.h>
-#include <libnr/nr-matrix.h>
 #include <libnr/nr-blit.h>
 #include <libnr/nr-pixops.h>
 #include "nr-arena.h"
 #include "nr-arena-item.h"
 //#include "nr-arena-group.h"
+#include "gc-core.h"
 
-#include "../prefs-utils.h"
-#include <sys/time.h>
+namespace GC = Inkscape::GC;
 
 static void nr_arena_item_class_init (NRArenaItemClass *klass);
 static void nr_arena_item_init (NRArenaItem *item);
@@ -70,13 +65,6 @@ nr_arena_item_class_init (NRArenaItemClass *klass)
 	object_class->cpp_ctor = NRObject::invoke_ctor<NRArenaItem>;
 }
 
-NRArenaItem::NRArenaItem() {
-	// clear all reverse-pointing pointers before finalization
-	clearOnceInaccessible(&arena);
-	clearOnceInaccessible(&parent);
-	clearOnceInaccessible(&prev);
-}
-
 static void
 nr_arena_item_init (NRArenaItem *item)
 {
@@ -116,13 +104,8 @@ nr_arena_item_private_finalize (NRObject *object)
   remove_caches(item);
 #endif
 
-	if (item->px) {
-		nr_free (item->px);
-	}
-
-	if (item->transform) {
-		nr_free (item->transform);
-	}
+	item->px = NULL;
+	item->transform = NULL;
 
 	((NRObjectClass *) (parent_class))->finalize (object);
 }
@@ -249,7 +232,6 @@ nr_arena_item_invoke_update (NRArenaItem *item, NRRectL *area, NRGC *gc, unsigne
 
 	/* Reset image cache, if not to be kept */
 	if (!(item->state & NR_ARENA_ITEM_STATE_IMAGE) && (item->px)) {
-		nr_free (item->px);
 		item->px = NULL;
 	}
 #ifdef arena_item_tile_cache
@@ -350,10 +332,11 @@ unsigned int nr_arena_item_invoke_render(NRArenaItem *item, NRRectL const *area,
 		carea.y0 = item->bbox.y0;
 		carea.x1 = item->bbox.x1;
 		carea.y1 = item->bbox.y1;
-		item->px = nr_new (unsigned char, 4 * (carea.x1 - carea.x0) * (carea.y1 - carea.y0));
+		item->px = new (GC::ATOMIC) unsigned char[4 * (carea.x1 - carea.x0) * (carea.y1 - carea.y0)];
 		nr_pixblock_setup_extern (&cpb, NR_PIXBLOCK_MODE_R8G8B8A8P,
                               carea.x0, carea.y0, carea.x1, carea.y1,
                               item->px, 4 * (carea.x1 - carea.x0), TRUE, TRUE);
+        cpb.visible_area = pb->visible_area; 
 		dpb = &cpb;
 		// Set nocache flag for downstream rendering
 		flags |= NR_ARENA_ITEM_RENDER_NO_CACHE;
@@ -415,6 +398,7 @@ unsigned int nr_arena_item_invoke_render(NRArenaItem *item, NRRectL const *area,
 
     /* Setup and render item buffer */
     nr_pixblock_setup_fast (&ipb, NR_PIXBLOCK_MODE_R8G8B8A8P, carea.x0, carea.y0, carea.x1, carea.y1, TRUE);
+    ipb.visible_area = pb->visible_area; 
     state = NR_ARENA_ITEM_VIRTUAL (item, render) (item, &carea, &ipb, flags);
     if (state & NR_ARENA_ITEM_STATE_INVALID) {
       /* Clean up and return error */
@@ -428,6 +412,7 @@ unsigned int nr_arena_item_invoke_render(NRArenaItem *item, NRRectL const *area,
     if (item->clip || item->mask) {
       /* Setup mask pixblock */
       nr_pixblock_setup_fast (&mpb, NR_PIXBLOCK_MODE_A8, carea.x0, carea.y0, carea.x1, carea.y1, TRUE);
+      mpb.visible_area = pb->visible_area; 
       /* Do clip if needed */
       if (item->clip) {
         state = nr_arena_item_invoke_clip (item->clip, &carea, &mpb);
@@ -446,6 +431,7 @@ unsigned int nr_arena_item_invoke_render(NRArenaItem *item, NRRectL const *area,
         NRPixBlock tpb;
         /* Set up yet another temporary pixblock */
         nr_pixblock_setup_fast (&tpb, NR_PIXBLOCK_MODE_R8G8B8A8N, carea.x0, carea.y0, carea.x1, carea.y1, TRUE);
+        tpb.visible_area = pb->visible_area; 
         state = NR_ARENA_ITEM_VIRTUAL (item->mask, render) (item->mask, &carea, &tpb, flags);
         if (state & NR_ARENA_ITEM_STATE_INVALID) {
           /* Clean up and return error */
@@ -490,7 +476,7 @@ unsigned int nr_arena_item_invoke_render(NRArenaItem *item, NRRectL const *area,
         nr_pixblock_release (&tpb);
       }
       /* Multiply with opacity if needed */
-      if ((item->opacity != 255) && !item->render_opacity) {
+      if ((item->opacity != 255) && !item->render_opacity && item->arena->rendermode != RENDERMODE_OUTLINE) {
         int x, y;
         unsigned int a;
         a = item->opacity;
@@ -518,7 +504,7 @@ unsigned int nr_arena_item_invoke_render(NRArenaItem *item, NRRectL const *area,
       } else {
         nr_pixblock_release (&ipb);
       }
-    } else if ( ((item->opacity != 255) && !item->render_opacity) ) {
+    } else if ( ((item->opacity != 255) && !item->render_opacity && item->arena->rendermode != RENDERMODE_OUTLINE) ) {
       /* Opacity only */
       // gettimeofday(&end_time,NULL);
       g_get_current_time (&end_time);
@@ -551,11 +537,12 @@ unsigned int nr_arena_item_invoke_render(NRArenaItem *item, NRRectL const *area,
 #endif
   } else {
     /* Determine, whether we need temporary buffer */
-    if (item->clip || item->mask || ((item->opacity != 255) && !item->render_opacity)) {
+    if (item->clip || item->mask || ((item->opacity != 255) && !item->render_opacity && item->arena->rendermode != RENDERMODE_OUTLINE)) {
       NRPixBlock ipb, mpb;
 
       /* Setup and render item buffer */
       nr_pixblock_setup_fast (&ipb, NR_PIXBLOCK_MODE_R8G8B8A8P, carea.x0, carea.y0, carea.x1, carea.y1, TRUE);
+      ipb.visible_area = pb->visible_area; 
       state = NR_ARENA_ITEM_VIRTUAL (item, render) (item, &carea, &ipb, flags);
       if (state & NR_ARENA_ITEM_STATE_INVALID) {
         /* Clean up and return error */
@@ -569,6 +556,7 @@ unsigned int nr_arena_item_invoke_render(NRArenaItem *item, NRRectL const *area,
       if (item->clip || item->mask) {
         /* Setup mask pixblock */
         nr_pixblock_setup_fast (&mpb, NR_PIXBLOCK_MODE_A8, carea.x0, carea.y0, carea.x1, carea.y1, TRUE);
+        mpb.visible_area = pb->visible_area; 
         /* Do clip if needed */
         if (item->clip) {
           state = nr_arena_item_invoke_clip (item->clip, &carea, &mpb);
@@ -587,6 +575,7 @@ unsigned int nr_arena_item_invoke_render(NRArenaItem *item, NRRectL const *area,
           NRPixBlock tpb;
           /* Set up yet another temporary pixblock */
           nr_pixblock_setup_fast (&tpb, NR_PIXBLOCK_MODE_R8G8B8A8N, carea.x0, carea.y0, carea.x1, carea.y1, TRUE);
+          tpb.visible_area = pb->visible_area; 
           state = NR_ARENA_ITEM_VIRTUAL (item->mask, render) (item->mask, &carea, &tpb, flags);
           if (state & NR_ARENA_ITEM_STATE_INVALID) {
             /* Clean up and return error */
@@ -631,7 +620,7 @@ unsigned int nr_arena_item_invoke_render(NRArenaItem *item, NRRectL const *area,
           nr_pixblock_release (&tpb);
         }
         /* Multiply with opacity if needed */
-        if ((item->opacity != 255) && !item->render_opacity) {
+        if ((item->opacity != 255) && !item->render_opacity && item->arena->rendermode != RENDERMODE_OUTLINE) {
           int x, y;
           unsigned int a;
           a = item->opacity;
@@ -689,13 +678,14 @@ nr_arena_item_invoke_clip (NRArenaItem *item, NRRectL *area, NRPixBlock *pb)
 	nr_return_val_if_fail ((pb->area.y1 - pb->area.y0) >= (area->y1 - area->y0), NR_ARENA_ITEM_STATE_INVALID);
 
 #ifdef NR_ARENA_ITEM_VERBOSE
-	printf ("Invoke render %p: %d %d - %d %d\n", item, area->x0, area->y0, area->x1, area->y1);
+	printf ("Invoke clip by %p: %d %d - %d %d, item bbox %d %d - %d %d\n", item, area->x0, area->y0, area->x1, area->y1, (&item->bbox)->x0, (&item->bbox)->y0, (&item->bbox)->x1, (&item->bbox)->y1);
 #endif
 
 	if (item->visible && nr_rect_l_test_intersect (area, &item->bbox)) {
 		/* Need render that item */
-		if (((NRArenaItemClass *) NR_OBJECT_GET_CLASS (item))->clip)
+		if (((NRArenaItemClass *) NR_OBJECT_GET_CLASS (item))->clip) {
 			return ((NRArenaItemClass *) NR_OBJECT_GET_CLASS(item))->clip (item, area, pb);
+		}
 	}
 
 	return item->state;
@@ -812,10 +802,9 @@ nr_arena_item_set_transform(NRArenaItem *item, NRMatrix const *transform)
 		nr_arena_item_request_render (item);
 		if (!transform || nr_matrix_test_identity (transform, NR_EPSILON)) {
 			/* Set to identity affine */
-			if (item->transform) nr_free (item->transform);
 			item->transform = NULL;
 		} else {
-			if (!item->transform) item->transform = nr_new (NRMatrix, 1);
+			if (!item->transform) item->transform = new (GC::ATOMIC) NRMatrix();
 			*item->transform = *transform;
 		}
 		nr_arena_item_request_update (item, NR_ARENA_ITEM_STATE_ALL, TRUE);
@@ -864,8 +853,8 @@ nr_arena_item_set_clip (NRArenaItem *item, NRArenaItem *clip)
 
 	if (clip != item->clip) {
 		nr_arena_item_request_render (item);
-		if (item->clip) item->clip = nr_arena_item_detach_unref (item, item->clip);
-		if (clip) item->clip = nr_arena_item_attach_ref (item, clip, NULL, NULL);
+		if (item->clip) item->clip = nr_arena_item_detach (item, item->clip);
+		if (clip) item->clip = nr_arena_item_attach (item, clip, NULL, NULL);
 		nr_arena_item_request_update (item, NR_ARENA_ITEM_STATE_ALL, TRUE);
 	}
 }
@@ -879,8 +868,8 @@ nr_arena_item_set_mask (NRArenaItem *item, NRArenaItem *mask)
 
 	if (mask != item->mask) {
 		nr_arena_item_request_render (item);
-		if (item->mask) item->mask = nr_arena_item_detach_unref (item, item->mask);
-		if (mask) item->mask = nr_arena_item_attach_ref (item, mask, NULL, NULL);
+		if (item->mask) item->mask = nr_arena_item_detach (item, item->mask);
+		if (mask) item->mask = nr_arena_item_attach (item, mask, NULL, NULL);
 		nr_arena_item_request_update (item, NR_ARENA_ITEM_STATE_ALL, TRUE);
 	}
 }
@@ -914,7 +903,7 @@ nr_arena_item_set_order (NRArenaItem *item, int order)
 /* Helpers */
 
 NRArenaItem *
-nr_arena_item_attach_ref (NRArenaItem *parent, NRArenaItem *child, NRArenaItem *prev, NRArenaItem *next)
+nr_arena_item_attach (NRArenaItem *parent, NRArenaItem *child, NRArenaItem *prev, NRArenaItem *next)
 {
 	nr_return_val_if_fail (parent != NULL, NULL);
 	nr_return_val_if_fail (NR_IS_ARENA_ITEM (parent), NULL);
@@ -941,7 +930,7 @@ nr_arena_item_attach_ref (NRArenaItem *parent, NRArenaItem *child, NRArenaItem *
 }
 
 NRArenaItem *
-nr_arena_item_detach_unref (NRArenaItem *parent, NRArenaItem *child)
+nr_arena_item_detach (NRArenaItem *parent, NRArenaItem *child)
 {
 	NRArenaItem *prev, *next;
 
