@@ -17,8 +17,6 @@
 #define NR_ARENA_SHAPE(obj) (NR_CHECK_INSTANCE_CAST ((obj), NR_TYPE_ARENA_SHAPE, NRArenaShape))
 #define NR_IS_ARENA_SHAPE(obj) (NR_CHECK_INSTANCE_TYPE ((obj), NR_TYPE_ARENA_SHAPE))
 
-//#include <libnr/nr-svp.h>
-
 #include "display/curve.h"
 #include "display/canvas-bpath.h"
 #include "forward.h"
@@ -140,6 +138,10 @@ struct NRArenaShape : public NRArenaItem {
 	// skewing
 	NR::Matrix cached_fctm;
 	NR::Matrix cached_sctm;
+    NRRectL    cached_farea;
+    NRRectL    cached_sarea;
+    bool       cached_fpartialy;
+    bool       cached_spartialy;
 	
 	Shape    *cached_fill;
 	Shape    *cached_stroke;
@@ -165,6 +167,8 @@ struct NRArenaShape : public NRArenaItem {
 	void setLineJoin(JoinType join);
 	void setMitreLimit(double limit);
 
+        void setPaintBox(NR::Rect const &pbox);
+
 	void _invalidateCachedFill() {
 		if (cached_fill) {
 			delete cached_fill;
