@@ -14,7 +14,6 @@
 
 #include "nr-arena-item.h"
 #include "nr-arena.h"
-#include <libnr/nr-rect.h>
 #include <libnr/nr-blit.h>
 
 static void nr_arena_class_init (NRArenaClass *klass);
@@ -52,6 +51,9 @@ nr_arena_class_init (NRArenaClass *klass)
 static void
 nr_arena_init (NRArena *arena)
 {
+	arena->delta = 0; // to be set by desktop from prefs
+	arena->rendermode = RENDERMODE_NORMAL; // default is normal render
+	arena->outlinecolor = 0xff; // black; to be set by desktop from bg color
 }
 
 static void
@@ -107,6 +109,8 @@ nr_arena_render_paintserver_fill (NRPixBlock *pb, NRRectL *area, SPPainter *pain
 	NRPixBlock cb, cb_opa;
 	nr_pixblock_setup_fast (&cb, NR_PIXBLOCK_MODE_R8G8B8A8N, area->x0, area->y0, area->x1, area->y1, TRUE);
 	nr_pixblock_setup_fast (&cb_opa, NR_PIXBLOCK_MODE_R8G8B8A8N, area->x0, area->y0, area->x1, area->y1, TRUE);
+    cb.visible_area = pb->visible_area; 
+    cb_opa.visible_area = pb->visible_area; 
 
 	/* Need separate gradient buffer (lauris)*/
 	// do the filling
