@@ -19,8 +19,6 @@
 # include "config.h"
 #endif
 
-#include <libnr/nr-values.h>
-#include <libnr/nr-macros.h>
 #include <libnr/nr-pixblock.h>
 
 #include <gtk/gtkmain.h>
@@ -28,10 +26,17 @@
 
 #include <helper/sp-marshal.h>
 #include <display/sp-canvas.h>
+#include "display-forward.h"
 #include <libnr/nr-matrix-fns.h>
 #include <libnr/nr-matrix-ops.h>
 #include <libnr/nr-convex-hull.h>
 
+enum {
+	RENDERMODE_NORMAL,
+	RENDERMODE_NOAA,
+	RENDERMODE_OUTLINE
+};
+
 const gint sp_canvas_update_priority = G_PRIORITY_HIGH_IDLE;
 
 #define SP_CANVAS_WINDOW(c) (((GtkWidget *) (c))->window)
@@ -293,10 +298,11 @@ sp_canvas_item_affine_absolute (SPCanvasItem *item, NR::Matrix const& affine)
 
     if (!(item->flags & SP_CANVAS_ITEM_NEED_AFFINE)) {
         item->flags |= SP_CANVAS_ITEM_NEED_AFFINE;
-        if (item->parent != NULL)
+        if (item->parent != NULL) {
             sp_canvas_item_request_update (item->parent);
-        else
+        } else {
             sp_canvas_request_update (item->canvas);
+        }
     }
 
     item->canvas->need_repick = TRUE;
@@ -489,7 +495,7 @@ sp_canvas_item_grab (SPCanvasItem *item, guint event_mask, GdkCursor *cursor, gu
     /* fixme: But Canvas actualle does get key events, so all we need is routing these here */
     gdk_pointer_grab (SP_CANVAS_WINDOW (item->canvas), FALSE,
                       (GdkEventMask)(event_mask & (~(GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK))),
-                      NULL, cursor, etime);   
+                      NULL, cursor, etime);
 
     item->canvas->grabbed_item = item;
     item->canvas->grabbed_event_mask = event_mask;
@@ -581,7 +587,7 @@ sp_canvas_item_grab_focus (SPCanvasItem *item)
         ev.focus_change.window = SP_CANVAS_WINDOW (item->canvas);
         ev.focus_change.send_event = FALSE;
         ev.focus_change.in = TRUE;
-		
+
         emit_event (item->canvas, &ev);
     }
 }
@@ -954,6 +960,9 @@ sp_canvas_init (SPCanvas *canvas)
 
     canvas->need_repick = TRUE;
 
+    // See comment at in sp-canvas.h.
+    canvas->gen_all_enter_events = false;
+
     canvas->tiles=NULL;
     canvas->tLeft=canvas->tTop=canvas->tRight=canvas->tBottom=0;
     canvas->tileH=canvas->tileV=0;
@@ -985,11 +994,11 @@ shutdown_transients (SPCanvas *canvas)
     if (canvas->need_redraw) {
         canvas->need_redraw = FALSE;
     }
-    if ( canvas->tiles ) free(canvas->tiles);
+    if ( canvas->tiles ) g_free(canvas->tiles);
     canvas->tiles=NULL;
     canvas->tLeft=canvas->tTop=canvas->tRight=canvas->tBottom=0;
     canvas->tileH=canvas->tileV=0;
-  
+
     if (canvas->grabbed_item) {
         canvas->grabbed_item = NULL;
         gdk_pointer_ungrab (GDK_CURRENT_TIME);
@@ -1245,10 +1254,22 @@ emit_event (SPCanvas *canvas, GdkEvent *event)
 static int
 pick_current_item (SPCanvas *canvas, GdkEvent *event)
 {
+    int button_down = 0;
     double x, y;
 
     int retval = FALSE;
 
+    if (canvas->gen_all_enter_events == false) {
+        // If a button is down, we'll perform enter and leave events on the
+        // current item, but not enter on any other item.  This is more or
+        // less like X pointer grabbing for canvas items.
+        //
+        button_down = canvas->state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK |
+                GDK_BUTTON3_MASK | GDK_BUTTON4_MASK | GDK_BUTTON5_MASK);
+
+        if (!button_down) canvas->left_grabbed_item = FALSE;
+    }
+
     /* Save the event in the canvas.  This is used to synthesize enter and
      * leave events in case the current item changes.  It is also used to
      * re-pick the current item if the current one gets deleted.  Also,
@@ -1336,6 +1357,15 @@ pick_current_item (SPCanvas *canvas, GdkEvent *event)
         canvas->in_repick = FALSE;
     }
 
+    if (canvas->gen_all_enter_events == false) {
+        // new_current_item may have been set to NULL during the call to
+        // emit_event() above
+        if ((canvas->new_current_item != canvas->current_item) && button_down) {
+            canvas->left_grabbed_item = TRUE;
+            return retval;
+        }
+    }
+
     /* Handle the rest of cases */
 
     canvas->left_grabbed_item = FALSE;
@@ -1458,10 +1488,6 @@ sp_canvas_motion (GtkWidget *widget, GdkEventMotion *event)
     return emit_event (canvas, (GdkEvent *) event);
 }
 
-/* We have to fit into pixelstore 64K */
-#define IMAGE_WIDTH_AA 341
-#define IMAGE_HEIGHT_AA 64
-
 /**
  * Helper that draws a specific rectangular part of the canvas.
  */
@@ -1483,22 +1509,42 @@ sp_canvas_paint_rect (SPCanvas *canvas, int xx0, int yy0, int xx1, int yy1)
         return;
 
     int sw, sh;
-    /* 65536 is max cached buffer and we need 3 channels */
-    if (bw * bh < 21845) {
-        // We can go with single buffer 
-        sw = bw;
-        sh = bh;
-    } else if (bw <= (16 * IMAGE_WIDTH_AA)) {
-        // Go with row buffer 
-        sw = bw;
-        sh = 21845 / bw;
-    } else if (bh <= (16 * IMAGE_HEIGHT_AA)) {
-        // Go with column buffer 
-        sw = 21845 / bh;
-        sh = bh;
-    } else {
-        sw = IMAGE_WIDTH_AA;
-        sh = IMAGE_HEIGHT_AA;
+    if (canvas->rendermode != RENDERMODE_OUTLINE) { // use 256K as a compromise to not slow down gradients
+        /* 256K is the cached buffer and we need 3 channels */
+        if (bw * bh <  87381) { // 256K/3
+            // We can go with single buffer 
+            sw = bw;
+            sh = bh;
+        } else if (bw <= (16 * 341)) {
+            // Go with row buffer 
+            sw = bw;
+            sh =  87381 / bw;
+        } else if (bh <= (16 * 256)) {
+            // Go with column buffer 
+            sw = 87381 / bh;
+            sh = bh;
+        } else {
+            sw = 341;
+            sh = 256;
+        }
+    } else {  // paths only, so 1M works faster
+        /* 1M is the cached buffer and we need 3 channels */
+        if (bw * bh <  349525) { // 1M/3
+            // We can go with single buffer 
+            sw = bw;
+            sh = bh;
+        } else if (bw <= (16 * 682)) {
+            // Go with row buffer 
+            sw = bw;
+            sh =  349525 / bw;
+        } else if (bh <= (16 * 512)) {
+            // Go with column buffer 
+            sw = 349525 / bh;
+            sh = bh;
+        } else {
+            sw = 682;
+            sh = 512;
+        }
     }
 
     // As we can come from expose, we have to tile here 
@@ -1508,24 +1554,32 @@ sp_canvas_paint_rect (SPCanvas *canvas, int xx0, int yy0, int xx1, int yy1)
             int x1 = MIN (x0 + sw, draw_x2);
 
             SPCanvasBuf buf;
-            buf.buf = nr_pixelstore_64K_new (0, 0);
+	if (canvas->rendermode != RENDERMODE_OUTLINE) {
+            buf.buf = nr_pixelstore_256K_new (FALSE, 0);
+  } else {
+            buf.buf = nr_pixelstore_1M_new (FALSE, 0);
+  }
+
             buf.buf_rowstride = sw * 3;
             buf.rect.x0 = x0;
             buf.rect.y0 = y0;
             buf.rect.x1 = x1;
             buf.rect.y1 = y1;
+            buf.visible_rect.x0 = draw_x1;
+            buf.visible_rect.y0 = draw_y1;
+            buf.visible_rect.x1 = draw_x2;
+            buf.visible_rect.y1 = draw_y2;
             GdkColor *color = &widget->style->bg[GTK_STATE_NORMAL];
             buf.bg_color = (((color->red & 0xff00) << 8)
                             | (color->green & 0xff00)
                             | (color->blue >> 8));
-            buf.is_bg = 1;
-            buf.is_buf = 0;
+            buf.is_empty = true;
       
             if (canvas->root->flags & SP_CANVAS_ITEM_VISIBLE) {
                 SP_CANVAS_ITEM_GET_CLASS (canvas->root)->render (canvas->root, &buf);
             }
       
-            if (buf.is_bg) {
+            if (buf.is_empty) {
                 gdk_rgb_gc_set_foreground (canvas->pixmap_gc, buf.bg_color);
                 gdk_draw_rectangle (SP_CANVAS_WINDOW (canvas),
                                     canvas->pixmap_gc,
@@ -1542,7 +1596,13 @@ sp_canvas_paint_rect (SPCanvas *canvas, int xx0, int yy0, int xx1, int yy1)
                                               sw * 3,
                                               x0 - canvas->x0, y0 - canvas->y0);
             }
-            nr_pixelstore_64K_free (buf.buf);
+
+	if (canvas->rendermode != RENDERMODE_OUTLINE) {
+            nr_pixelstore_256K_free (buf.buf);
+  } else {
+            nr_pixelstore_1M_free (buf.buf);
+  }
+
         }
     }
 }
@@ -1660,6 +1720,9 @@ paint (SPCanvas *canvas)
     int const canvas_x1 = canvas->x0 + widget->allocation.width;
     int const canvas_y1 = canvas->y0 + widget->allocation.height;
 
+    NRRectL topaint;
+    topaint.x0 = topaint.y0 = topaint.x1 = topaint.y1 = 0;
+
     for (int j=canvas->tTop&(~3);j<canvas->tBottom;j+=4) {
         for (int i=canvas->tLeft&(~3);i<canvas->tRight;i+=4) {
             int  mode=0;
@@ -1679,16 +1742,21 @@ paint (SPCanvas *canvas)
             }
       
             if ( mode ) {
-                int x0 = 0,y0 = 0,x1 = 0,y1 = 0;
-                x0 = MAX (pl*32, canvas->x0);
-                y0 = MAX (pt*32, canvas->y0);
-                x1 = MIN (pr*32, canvas_x1);
-                y1 = MIN (pb*32, canvas_y1);
-                if ((x0 < x1) && (y0 < y1)) sp_canvas_paint_rect (canvas, x0, y0, x1, y1);
+                NRRectL tile;
+                tile.x0 = MAX (pl*32, canvas->x0);
+                tile.y0 = MAX (pt*32, canvas->y0);
+                tile.x1 = MIN (pr*32, canvas_x1);
+                tile.y1 = MIN (pb*32, canvas_y1);
+                if ((tile.x0 < tile.x1) && (tile.y0 < tile.y1)) {
+                    nr_rect_l_union (&topaint, &topaint, &tile);
+                }
+
             }
         }
     }
-  
+
+    sp_canvas_paint_rect (canvas, topaint.x0, topaint.y0, topaint.x1, topaint.y1);
+
     canvas->need_redraw = FALSE;
     return TRUE;
 }
@@ -1707,7 +1775,7 @@ do_update (SPCanvas *canvas)
 
     /* Paint if able to */
     if (GTK_WIDGET_DRAWABLE (canvas)) {
-        return paint (canvas);
+            return paint (canvas);
     }
 
     /* Pick new current item */
@@ -1785,8 +1853,9 @@ sp_canvas_scroll_to (SPCanvas *canvas, double cx, double cy, unsigned int clear)
     canvas->y0 = iy;
 
     sp_canvas_resize_tiles(canvas,canvas->x0,canvas->y0,canvas->x0+canvas->widget.allocation.width,canvas->y0+canvas->widget.allocation.height);
-  
+
     if (!clear) {
+        // scrolling without zoom; redraw only the newly exposed areas
         if ((dx != 0) || (dy != 0)) {
             int width, height;
             width = canvas->widget.allocation.width;
@@ -1807,7 +1876,7 @@ sp_canvas_scroll_to (SPCanvas *canvas, double cx, double cy, unsigned int clear)
             }
         }
     } else {
-        gtk_widget_queue_draw (GTK_WIDGET (canvas));
+        // scrolling as part of zoom; do nothing here - the next do_update will perform full redraw
     }
 }
 
@@ -1820,7 +1889,7 @@ sp_canvas_update_now (SPCanvas *canvas)
     g_return_if_fail (canvas != NULL);
     g_return_if_fail (SP_IS_CANVAS (canvas));
 
-    if (!(canvas->need_update || 
+    if (!(canvas->need_update ||
           canvas->need_redraw))
         return;
 
@@ -1936,18 +2005,12 @@ bool sp_canvas_world_pt_inside_window(SPCanvas const *canvas, NR::Point const &w
 /**
  * Return canvas window coordinates as NRRect.
  */
-NRRect *sp_canvas_get_viewbox(SPCanvas const *canvas, NRRect *viewbox)
+NR::Rect SPCanvas::getViewbox() const
 {
-    g_return_val_if_fail (canvas != NULL, NULL);
-    g_return_val_if_fail (SP_IS_CANVAS (canvas), NULL);
-    g_return_val_if_fail (viewbox != NULL, NULL);
+    GtkWidget const *w = GTK_WIDGET(this);
 
-    viewbox->x0 = canvas->dx0;
-    viewbox->y0 = canvas->dy0;
-    viewbox->x1 = viewbox->x0 + GTK_WIDGET (canvas)->allocation.width;
-    viewbox->y1 = viewbox->y0 + GTK_WIDGET (canvas)->allocation.height;
-
-    return viewbox;
+    return NR::Rect(NR::Point(dx0, dy0),
+                    NR::Point(dx0 + w->allocation.width, dy0 + w->allocation.height));
 }
 
 inline int sp_canvas_tile_floor(int x)
@@ -1966,7 +2029,7 @@ inline int sp_canvas_tile_ceil(int x)
 void sp_canvas_resize_tiles(SPCanvas* canvas,int nl,int nt,int nr,int nb)
 {
     if ( nl >= nr || nt >= nb ) {
-        if ( canvas->tiles ) free(canvas->tiles);
+        if ( canvas->tiles ) g_free(canvas->tiles);
         canvas->tLeft=canvas->tTop=canvas->tRight=canvas->tBottom=0;
         canvas->tileH=canvas->tileV=0;
         canvas->tiles=NULL;
@@ -1978,7 +2041,7 @@ void sp_canvas_resize_tiles(SPCanvas* canvas,int nl,int nt,int nr,int nb)
     int tb=sp_canvas_tile_ceil(nb);
 
     int nh=tr-tl,nv=tb-tt;
-    uint8_t* ntiles=(uint8_t*)malloc(nh*nv*sizeof(uint8_t));
+    uint8_t* ntiles=(uint8_t*)g_malloc(nh*nv*sizeof(uint8_t));
     for (int i=tl;i<tr;i++) {
         for (int j=tt;j<tb;j++) {
             int ind=(i-tl)+(j-tt)*nh;
@@ -1989,7 +2052,7 @@ void sp_canvas_resize_tiles(SPCanvas* canvas,int nl,int nt,int nr,int nb)
             }
         }
     }
-    if ( canvas->tiles ) free(canvas->tiles);
+    if ( canvas->tiles ) g_free(canvas->tiles);
     canvas->tiles=ntiles;
     canvas->tLeft=tl;
     canvas->tTop=tt;
@@ -2016,9 +2079,9 @@ void sp_canvas_dirty_rect(SPCanvas* canvas,int nl,int nt,int nr,int nb)
     if ( tr > canvas->tRight ) tr=canvas->tRight;
     if ( tt < canvas->tTop ) tt=canvas->tTop;
     if ( tb > canvas->tBottom ) tb=canvas->tBottom;
-  
+
     canvas->need_redraw = TRUE;
-  
+
     for (int i=tl;i<tr;i++) {
         for (int j=tt;j<tb;j++) {
             canvas->tiles[(i-canvas->tLeft)+(j-canvas->tTop)*canvas->tileH]=1;
