@@ -12,7 +12,7 @@
  * Authors:
  *   Bob Jamison
  *
- * Copyright (C) 2005 Bob Jamison
+ * Copyright (C) 2006 Bob Jamison
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -29,13 +29,19 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <string>
 #include <vector>
 
-#define OWN_STRING
+//# include this before the #ifdefs below
+#include "domconfig.h"
 
-#ifdef OWN_STRING
+#ifdef DOM_STRING_OWN
 #include "domstring.h"
+#else
+#ifdef DOM_STRING_GLIBMM
+#include <glibmm.h>
+#else
+#include <string>
+#endif
 #endif
 
 #define XMLNSNAME "http://www.w3.org/2000/xmlns/"
@@ -49,10 +55,17 @@ namespace dom
 
 
 
-#ifndef OWN_STRING
-typedef unsigned short XMLCh;
+#ifdef DOM_STRING_OWN
+#else
+#ifdef DOM_STRING_GLIBMM
+typedef Glib::ustring DOMString;
+typedef gunichar XMLCh;
+#else
 typedef std::string DOMString;
+typedef unsigned short XMLCh;
 #endif
+#endif
+
 
 /**
  *
@@ -246,6 +259,15 @@ public:
         strings = other.strings;
         }
 
+    /**
+     *
+     */
+    DOMStringList &operator=(const DOMStringList &other)
+        {
+        strings = other.strings;
+        return *this;
+        }
+
     /**
      *
      */
@@ -284,6 +306,12 @@ public:
         namespaceURI = other.namespaceURI;
         name         = other.name;
         }
+    NamePair &operator=(const NamePair &other)
+        {
+        namespaceURI = other.namespaceURI;
+        name         = other.name;
+        return *this;
+        }
     virtual ~NamePair() {}
 
     DOMString namespaceURI;
@@ -369,6 +397,15 @@ public:
         namePairs = other.namePairs;
         }
 
+    /**
+     *
+     */
+    NameList &operator=(const NameList &other)
+        {
+        namePairs = other.namePairs;
+        return *this;
+        }
+
     /**
      *
      */
@@ -426,6 +463,15 @@ public:
         implementations = other.implementations;
         }
 
+    /**
+     *
+     */
+    DOMImplementationList &operator=(const DOMImplementationList &other)
+        {
+        implementations = other.implementations;
+        return *this;
+        }
+
     /**
      *
      */
@@ -822,11 +868,28 @@ public:
         nodes = other.nodes;
         }
 
+    /**
+     *
+     */
+    NodeList &operator=(const NodeList &other)
+        {
+        nodes = other.nodes;
+        return *this;
+        }
+
     /**
      *
      */
     virtual ~NodeList() {}
 
+    /**
+     *
+     */
+    virtual void clear()
+        {
+        nodes.clear();
+        }
+
 protected:
 
 friend class NodeImpl;
@@ -866,13 +929,22 @@ public:
         }
     NamedNodeMapEntry(const NamedNodeMapEntry &other)
         {
-        namespaceURI = other.namespaceURI;
-        name         = other.name;
-        node         = other.node;
+        assign(other);
+        }
+    NamedNodeMapEntry &operator=(const NamedNodeMapEntry &other)
+        {
+        assign(other);
+        return *this;
         }
     virtual ~NamedNodeMapEntry()
         {
         }
+    void assign(const NamedNodeMapEntry &other)
+        {
+        namespaceURI = other.namespaceURI;
+        name         = other.name;
+        node         = other.node;
+        }
     DOMString namespaceURI;
     DOMString name;
     Node      *node;
@@ -1043,6 +1115,15 @@ public:
         entries = other.entries;
         }
 
+    /**
+     *
+     */
+    NamedNodeMap &operator=(const NamedNodeMap &other)
+        {
+        entries = other.entries;
+        return *this;
+        }
+
 
     /**
      *
@@ -1528,9 +1609,9 @@ public:
 
     typedef enum
         {
-        SEVERITY_WARNING     = 1,
-        SEVERITY_ERROR       = 2,
-        SEVERITY_FATAL_ERROR = 3
+        DOMERROR_SEVERITY_WARNING     = 1,
+        DOMERROR_SEVERITY_ERROR       = 2,
+        DOMERROR_SEVERITY_FATAL_ERROR = 3
         } ErrorSeverity;
 
 
