@@ -10,7 +10,7 @@
  * Authors:
  *   Bob Jamison
  *
- * Copyright (C) 2005 Bob Jamison
+ * Copyright (C) 2006 Bob Jamison
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -28,8 +28,9 @@
  */
 
 
-#include "xpathparser.h"
 #include "charclass.h"
+#include "xpathparser.h"
+
 
 namespace org
 {
@@ -45,6 +46,8 @@ namespace xpath
 //# M E S S A G E S
 //#########################################################################
 
+
+
 void XPathParser::trace(const char *fmt, ...)
 {
     if (!debug)
@@ -60,6 +63,8 @@ void XPathParser::trace(const char *fmt, ...)
     va_end(args);
 }
 
+
+
 void XPathParser::error(const char *fmt, ...)
 {
     FILE *f = stdout;
@@ -77,10 +82,13 @@ void XPathParser::error(const char *fmt, ...)
     fprintf(f, "^\n");
 }
 
+
+
 void XPathParser::traceStack(const char *name, int pos, int depth)
 {
     if (!debug)
         return;
+    return;
     int indent = depth;
 
     for (int i=0 ; i<indent ; i++)
@@ -93,6 +101,7 @@ void XPathParser::traceStack(const char *name, int pos, int depth)
 //#########################################################################
 //# L E X I C A L    S C A N N I N G
 //#########################################################################
+
 void XPathParser::lexTokAdd(int type, int loc)
 {
     LexTok tok(type, loc);
@@ -120,7 +129,7 @@ void XPathParser::lexTokAdd(int type, int loc, long   val)
 void XPathParser::lexicalTokenDump()
 {
     printf("####### LEXICAL TOKENS #######\n");
-    for (int i=0 ; i<(int)lexicalTokens.size() ; i++)
+    for (unsigned int i=0 ; i<lexicalTokens.size() ; i++)
         {
         printf("%d : ", i);
         lexicalTokens[i].print();
@@ -134,7 +143,7 @@ LexTok XPathParser::lexTok(int p)
 {
     if (p < 0 || p>=(int)lexicalTokens.size())
         {
-		LexTok tok;
+	LexTok tok;
         return tok;
         }
     return lexicalTokens[p];
@@ -142,7 +151,7 @@ LexTok XPathParser::lexTok(int p)
 
 int XPathParser::lexTokType(int p)
 {
-    if (p < 0 || p>(int)lexicalTokens.size())
+    if (p < 0 || p>=(int)lexicalTokens.size())
         return -1;
     return lexicalTokens[p].getType();
 }
@@ -178,7 +187,7 @@ int XPathParser::skipwhite(int p0)
     while (p < parselen)
         {
         int ch = peek(p);
-        if (!isspace(ch))
+        if (!isWhitespace(ch))
             break;
         ch = get(p++);
         }
@@ -191,10 +200,10 @@ int XPathParser::getword(int p0, DOMString &str)
     while (p < parselen)
         {
         int ch = peek(p);
-        if (!isalnum(ch))
+        if (!isLetterOrDigit(ch))
             break;
         ch = get(p++);
-        str.push_back(ch);
+        str.push_back((XMLCh)ch);
         }
     return p;
 }
@@ -263,7 +272,7 @@ int XPathParser::getNumber(int p0, double &dresult)
             }
         else if (!isDigit(ch))
             break;
-        num.push_back(ch);
+        num.push_back((XMLCh)ch);
         i++;
         }
 
@@ -295,6 +304,8 @@ int XPathParser::getNumber(int p0, double &dresult)
     return p;
 }
 
+
+
 int XPathParser::getLiteral(int p0, DOMString &result)
 {
     int p = p0;
@@ -317,7 +328,7 @@ int XPathParser::getLiteral(int p0, DOMString &result)
         ch = peek(p);
         if (ch == quotechar)
             break;
-        result.push_back(ch);
+        result.push_back((XMLCh)ch);
         p++;
         }
     p++; //skip over closing "
@@ -325,6 +336,10 @@ int XPathParser::getLiteral(int p0, DOMString &result)
 }
 
 
+
+/**
+ * NCName is a 'non-colonized' name
+ */
 int XPathParser::getNCName(int p0, DOMString &result)
 {
     int p = p0;
@@ -332,7 +347,7 @@ int XPathParser::getNCName(int p0, DOMString &result)
     if (ch != '_' && !isLetter(ch))
         return p0;
 
-    result.push_back(ch);
+    result.push_back((XMLCh)ch);
     p++;
     while (p < parselen)
         {
@@ -342,7 +357,7 @@ int XPathParser::getNCName(int p0, DOMString &result)
                isExtender(ch)      ||
                ch == '.' || ch == '-' || ch == '_' )
            {
-           result.push_back(ch);
+           result.push_back((XMLCh)ch);
            p++;
            }
        else
@@ -351,13 +366,18 @@ int XPathParser::getNCName(int p0, DOMString &result)
     return p;
 }
 
+
+
+/**
+ * Name parsing with post-parsing
+ */
 int XPathParser::getNameTest(int p0, DOMString &result)
 {
     int p = p0;
     int ch = peek(p);
     if (ch == '*')
         {
-        result.push_back(ch);
+        result.push_back((XMLCh)ch);
         p++;
         return p;
         }
@@ -385,7 +405,7 @@ int XPathParser::getNameTest(int p0, DOMString &result)
     ch = peek(p);
     if (ch == '*')
         {
-        result.push_back(ch);
+        result.push_back((XMLCh)ch);
         p++;
         return p;
         }
@@ -598,6 +618,12 @@ int XPathParser::lexicalScan()
 //# X P A T H    G R A M M A R    P A R S I N G
 //#########################################################################
 
+
+void XPathParser::tokAdd(Token *tok)
+{
+    tokens.add(tok);
+}
+
 /**
  * [1]  LocationPath ::=
  *        RelativeLocationPath
@@ -612,11 +638,17 @@ int XPathParser::getLocationPath(int p0, int depth)
 
     int p2 = getAbsoluteLocationPath(p, depth+1);
     if (p2 > p)
+        {
+        tokens.add(new TokAbsolute());
         return p2;
+        }
 
     p2 = getRelativeLocationPath(p, depth+1);
     if (p2 > p)
+        {
+        tokens.add(new TokRelative());
         return p2;
+        }
 
     return p0;
 }
@@ -698,6 +730,8 @@ int XPathParser::getRelativeLocationPath(int p0, int depth)
         if (t.getType() == OPERATOR && t.getIntValue()==DOUBLE_SLASH)
             {
             p++;
+            // a '//' is an abbreviation for /descendant-or-self:node()/
+            tokAdd(new TokAxisDescendantOrSelf());
             p2 = getRelativeLocationPath(p, depth+1);
             if (p2 < 0)
                 {
@@ -735,7 +769,6 @@ int XPathParser::getStep(int p0, int depth)
         error("Axis specifier in step section");
         return -1;
         }
-
     p = p2;
     p2 = getNodeTest(p, depth+1);
     if (p2 < 0)
@@ -786,6 +819,8 @@ int XPathParser::getAxisSpecifier(int p0, int depth)
     int p = p0;
     if (lexTokType(p) == AXIS_NAME)
         {
+        LexTok t = lexTok(p);
+        int axisType = t.getIntValue();
         p++;
         if (lexTokType(p) != DOUBLE_COLON)
             {
@@ -793,6 +828,40 @@ int XPathParser::getAxisSpecifier(int p0, int depth)
             return -1;
             }
         p++;
+        switch (axisType)
+            {
+            case ANCESTOR_OR_SELF:
+                tokAdd(new TokAxisAncestorOrSelf());
+            case ANCESTOR:
+                tokAdd(new TokAxisAncestor());
+            case ATTRIBUTE:
+                tokAdd(new TokAxisAttribute());
+            case CHILD:
+                tokAdd(new TokAxisChild());
+            case DESCENDANT_OR_SELF:
+                tokAdd(new TokAxisDescendantOrSelf());
+            case DESCENDANT:
+                tokAdd(new TokAxisDescendant());
+            case FOLLOWING_SIBLING:
+                tokAdd(new TokAxisFollowingSibling());
+            case FOLLOWING:
+                tokAdd(new TokAxisFollowing());
+            case NAMESPACE:
+                tokAdd(new TokAxisNamespace());
+            case PARENT:
+                tokAdd(new TokAxisParent());
+            case PRECEDING_SIBLING:
+                tokAdd(new TokAxisPrecedingSibling());
+            case PRECEDING:
+                tokAdd(new TokAxisPreceding());
+            case SELF:
+                tokAdd(new TokAxisSelf());
+            default:
+                {
+                error("unknown axis type %d", axisType);
+                return -1;
+                }
+            }
         return p;
         }
 
@@ -842,16 +911,15 @@ int XPathParser::getAxisName(int p0, int depth)
 int XPathParser::getNodeTest(int p0, int depth)
 {
     traceStack("getNodeTest", p0, depth);
-
     int p = p0;
 
     LexTok t = lexTok(p);
     if (t.getType() == NAME_TEST)
         {
         p++;
+        tokAdd(new TokNameTest(t.getStringValue()));
         return p;
         }
-
     if (t.getType() == NODE_TYPE)
         {
         if (t.getIntValue() == PROCESSING_INSTRUCTION)
@@ -1042,12 +1110,14 @@ int XPathParser::getPrimaryExpr(int p0, int depth)
 
     if (lexTokType(p) == LITERAL)
         {
+        tokens.add(new TokStr(lexTok(p).getStringValue()));
         p++;
         return p;
         }
 
     if (lexTokType(p) == NUMBER)
         {
+        tokens.add(new TokFloat(lexTok(p).getDoubleValue()));
         p++;
         return p;
         }
@@ -1080,12 +1150,16 @@ int XPathParser::getFunctionCall(int p0, int depth)
     if (lexTokType(p) != FUNCTION_NAME)
         return p0;
 
+    DOMString name = lexTok(p).getStringValue();
+
     p++;
 
     if (lexTokType(p) != LPAREN) //this makes a function
         return p0;
     p++;
 
+    int argCount = 0;
+
     int p2 = getArgument(p, depth+1);
     if (p2 < 0)
         {
@@ -1094,6 +1168,7 @@ int XPathParser::getFunctionCall(int p0, int depth)
         }
     if (p2 > p)
         {
+        argCount++;
         p = p2;
         while (lexTokType(p) == COMMA)
             {
@@ -1104,6 +1179,9 @@ int XPathParser::getFunctionCall(int p0, int depth)
                 error("Error in function argument");
                 return -1;
                 }
+            if (p2 > p)
+                argCount++;
+            //do we add a token here?  i dont think so
             p = p2;
             }
         }
@@ -1115,6 +1193,66 @@ int XPathParser::getFunctionCall(int p0, int depth)
         }
     p++;
 
+    // Function names from http://www.w3.org/TR/xpath#NT-FunctionName
+    if (name == "last")
+        tokens.add(new TokFuncLast());
+    else if (name == "position")
+        tokens.add(new TokFuncPosition());
+    else if (name == "count")
+        tokens.add(new TokFuncCount());
+    else if (name == "id")
+        tokens.add(new TokFuncId());
+    else if (name == "local-name")
+        tokens.add(new TokFuncLocalName());
+    else if (name == "namespace-uri")
+        tokens.add(new TokFuncNamespaceUri());
+    else if (name == "name")
+        tokens.add(new TokFuncName());
+    else if (name == "string")
+        tokens.add(new TokFuncString());
+    else if (name == "concat")
+        tokens.add(new TokFuncConcat());
+    else if (name == "starts-with")
+        tokens.add(new TokFuncStartsWith());
+    else if (name == "contains")
+        tokens.add(new TokFuncContains());
+    else if (name == "substring-before")
+        tokens.add(new TokFuncSubstringBefore());
+    else if (name == "substring-after")
+        tokens.add(new TokFuncSubstringAfter());
+    else if (name == "substring")
+        tokens.add(new TokFuncSubstring());
+    else if (name == "string-length")
+        tokens.add(new TokFuncStringLength());
+    else if (name == "normalize-space")
+        tokens.add(new TokFuncNormalizeSpace());
+    else if (name == "translate")
+        tokens.add(new TokFuncTranslate());
+    else if (name == "boolean")
+        tokens.add(new TokFuncBoolean());
+    else if (name == "not")
+        tokens.add(new TokFuncNot());
+    else if (name == "true")
+        tokens.add(new TokFuncTrue());
+    else if (name == "false")
+        tokens.add(new TokFuncFalse());
+    else if (name == "lang")
+        tokens.add(new TokFuncLang());
+    else if (name == "number")
+        tokens.add(new TokFuncNumber());
+    else if (name == "sum")
+        tokens.add(new TokFuncSum());
+    else if (name == "floor")
+        tokens.add(new TokFuncFloor());
+    else if (name == "ceiling")
+        tokens.add(new TokFuncCeiling());
+    else if (name == "round")
+        tokens.add(new TokFuncRound());
+    else
+        {
+        error("unknown function name:'%s'", name.c_str());
+        return -1;
+        }
     return p;
 }
 
@@ -1164,6 +1302,7 @@ int XPathParser::getUnionExpr(int p0, int depth)
             error("OR (|) requires union expression on the left");
             return -1;
             }
+        tokens.add(new TokUnion());
         p = p2;
         }
     return p;
@@ -1307,6 +1446,7 @@ int XPathParser::getOrExpr(int p0, int depth)
             p = p2;
             return p;
             }
+        tokens.add(new TokOr());
         return p;
         }
 
@@ -1345,6 +1485,7 @@ int XPathParser::getAndExpr(int p0, int depth)
             p = p2;
             return p;
             }
+        tokens.add(new TokAnd());
         return p;
         }
 
@@ -1381,6 +1522,7 @@ int XPathParser::getEqualityExpr(int p0, int depth)
                 error("Equality expression expected after ==");
                 return -1;
                 }
+            tokens.add(new TokEquals());
             p = p2;
             return p;
             }
@@ -1394,6 +1536,7 @@ int XPathParser::getEqualityExpr(int p0, int depth)
                 error("Equality expression expected after !=");
                 return -1;
                 }
+            tokens.add(new TokNotEquals());
             p = p2;
             return p;
             }
@@ -1437,6 +1580,7 @@ int XPathParser::getRelationalExpr(int p0, int depth)
                 error("Relational expression after '>'");
                 return -1;
                 }
+            tokens.add(new TokGreaterThan());
             p = p2;
             return p;
             }
@@ -1449,6 +1593,7 @@ int XPathParser::getRelationalExpr(int p0, int depth)
                 error("Relational expression after '<'");
                 return -1;
                 }
+            tokens.add(new TokLessThan());
             p = p2;
             return p;
             }
@@ -1461,6 +1606,7 @@ int XPathParser::getRelationalExpr(int p0, int depth)
                 error("Relational expression after '>='");
                 return -1;
                 }
+            tokens.add(new TokGreaterThanEquals());
             p = p2;
             return p;
             }
@@ -1473,6 +1619,7 @@ int XPathParser::getRelationalExpr(int p0, int depth)
                 error("Relational expression after '<='");
                 return -1;
                 }
+            tokens.add(new TokLessThanEquals());
             p = p2;
             return p;
             }
@@ -1515,6 +1662,7 @@ int XPathParser::getAdditiveExpr(int p0, int depth)
                 error("Additive expression after '+'");
                 return -1;
                 }
+            tokens.add(new TokPlus());
             p = p2;
             return p;
             }
@@ -1527,6 +1675,7 @@ int XPathParser::getAdditiveExpr(int p0, int depth)
                 error("Additive expression after '-'");
                 return -1;
                 }
+            tokens.add(new TokMinus());
             p = p2;
             return p;
             }
@@ -1570,6 +1719,7 @@ int XPathParser::getMultiplicativeExpr(int p0, int depth)
                 error("Multiplicative expression after '*'");
                 return -1;
                 }
+            tokens.add(new TokMul());
             p = p2;
             return p;
             }
@@ -1583,6 +1733,7 @@ int XPathParser::getMultiplicativeExpr(int p0, int depth)
                 error("Multiplicative expression after 'div'");
                 return -1;
                 }
+            tokens.add(new TokDiv());
             p = p2;
             return p;
             }
@@ -1596,6 +1747,7 @@ int XPathParser::getMultiplicativeExpr(int p0, int depth)
                 error("Multiplicative expression after 'mod'");
                 return -1;
                 }
+            tokens.add(new TokMod());
             p = p2;
             return p;
             }
@@ -1638,6 +1790,7 @@ int XPathParser::getUnaryExpr(int p0, int depth)
             error("Unary expression after '-'");
             return -1;
             }
+        tokens.add(new TokNeg());
         p = p2;
         return p;
         }
@@ -1817,7 +1970,7 @@ bool XPathParser::parse(const DOMString &xpathString)
     int p0 = 0;
 
     DOMString str = xpathString;
-    
+
     parsebuf = (char *)str.c_str();
     parselen = (int)   str.size();
     position = 0;
@@ -1828,7 +1981,7 @@ bool XPathParser::parse(const DOMString &xpathString)
     lexicalTokenDump();
 
     tokens.clear();//Get ready to store new tokens
-    
+
     int p = getLocationPath(p0, 0);
 
     parsebuf = NULL;
@@ -1850,27 +2003,6 @@ bool XPathParser::parse(const DOMString &xpathString)
 //# E V A L U A T E
 //#########################################################################
 
-/**
- *  This method "executes" a list of Tokens in the context of a DOM root
- *  Node, returning a list of Nodes that match the xpath expression.
- */ 
-NodeList XPathParser::execute(const Node *root, 
-                              std::vector<Token> &toks)
-{
-    NodeList list;
-
-    if (!root)
-        return list;
-        
-    //### Execute the token list
-    std::vector<Token>::iterator iter;
-    for (iter = toks.begin() ; iter != toks.end() ; iter++)
-        {
-        }
-        
-    return list;
-}
-
 
 
 
@@ -1878,20 +2010,25 @@ NodeList XPathParser::execute(const Node *root,
  * This wraps the two-step call to parse(), then execute() to get a NodeList
  * of matching DOM nodes
  */
-NodeList XPathParser::evaluate(const Node *root, const DOMString &xpathString)
+NodeList XPathParser::evaluate(const Node *root,
+                               const DOMString &xpathString)
 {
     NodeList list;
 
     //### Maybe do caching for speed here
-    
+
     //### Parse and execute
     //### Error message can be generated as a side effect
     if (!parse(xpathString))
         return list;
 
+    if (debug)
+        tokens.dump();
+
     //### Execute the token list
-    list = execute(root, tokens);
-        
+    TokenExecutor executor;
+    list = executor.execute(tokens, root);
+
     return list;
 }
 
