@@ -7,12 +7,13 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <glibmm/i18n.h>
-
 #include "config.h"
 #include "path-prefix.h"
-
 #include "dependency.h"
 #include "db.h"
 
