@@ -17,23 +17,15 @@
 #endif
 #include "path-prefix.h"
 
-#include <inttypes.h>
-#include <dirent.h>
-#include <glib.h>
-#include <string.h>
 
 #include "inkscape.h"
-#include "xml/repr.h"
-#include <glibmm/fileutils.h>
 #include <glibmm/i18n.h>
 
-#include "extension/extension.h"
 #include "system.h"
 #include "db.h"
-#include "implementation/script.h"
-#include "internal/svg.h"
 #include "internal/svgz.h"
 #include "internal/ps.h"
+#include "internal/pdf.h"
 #ifdef WITH_GNOME_PRINT
 # include "internal/gnome.h"
 #endif
@@ -41,7 +33,9 @@
 # include "internal/win32.h"
 #endif
 #include "internal/ps-out.h"
+#include "internal/pdf-out.h"
 #include "internal/pov-out.h"
+#include "internal/odf.h"
 #include "internal/latex-pstricks-out.h"
 #include "internal/latex-pstricks.h"
 #include "internal/eps-out.h"
@@ -50,7 +44,6 @@
 #include "internal/gimpgrad.h"
 #include "internal/grid.h"
 #include "prefs-utils.h"
-#include "error-file.h"
 #include "io/sys.h"
 
 extern gboolean inkscape_app_use_gui( Inkscape::Application const *app );
@@ -112,6 +105,8 @@ init()
     Internal::PsOutput::init();
     Internal::EpsOutput::init();
     Internal::PrintPS::init();
+    Internal::PdfOutput::init();
+    Internal::PrintPDF::init();
 #ifdef WITH_GNOME_PRINT
     Internal::PrintGNOME::init();
 #endif
@@ -119,6 +114,7 @@ init()
     Internal::PrintWin32::init();
 #endif
     Internal::PovOutput::init();
+    Internal::OdfOutput::init();
     Internal::PrintLatex::init();
     Internal::LatexOutput::init();
 
@@ -126,7 +122,7 @@ init()
     Internal::BlurEdge::init();
     Internal::GimpGrad::init();
     Internal::Grid::init();
-    
+
     /* Load search path for extensions */
     if (Inkscape::Extension::Extension::search_path.size() == 0)
     {
@@ -179,7 +175,7 @@ build_module_from_dir(gchar const *dirname)
     }
 
     //# Hopefully doing this the Glib way is portable
-        
+
     GError *err;
     GDir *directory = g_dir_open(dirname, 0, &err);
     if (!directory) {
@@ -214,6 +210,7 @@ check_extensions_internal(Extension *in_plug, gpointer in_data)
 {
     int *count = (int *)in_data;
 
+    if (in_plug == NULL) return;
     if (!in_plug->deactivated() && !in_plug->check()) {
          in_plug->deactivate();
         (*count)++;
