@@ -533,6 +533,8 @@ PrintPDF::print_fill_alpha(SVGOStringStream &/*os*/, SPStyle const *const style,
         if (SP_IS_LINEARGRADIENT (SP_STYLE_FILL_SERVER (style))) {
             
             SPLinearGradient *lg=SP_LINEARGRADIENT(SP_STYLE_FILL_SERVER (style));
+            sp_gradient_ensure_vector(SP_GRADIENT(lg)); // when exporting from commandline, vector is not built
+
             NR::Point p1 (lg->x1.computed, lg->y1.computed);
             NR::Point p2 (lg->x2.computed, lg->y2.computed);
             if (pbox && SP_GRADIENT(lg)->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
@@ -571,7 +573,6 @@ PrintPDF::print_fill_alpha(SVGOStringStream &/*os*/, SPStyle const *const style,
                     *pdf_alpha << "/Domain [0 1]\n";
                     *pdf_alpha << "/Function <<\n/FunctionType 3\n/Functions\n[\n";
 
-                    sp_gradient_ensure_vector(SP_GRADIENT(lg)); // when exporting from commandline, vector is not built
                     for (gint i = 0; unsigned(i) < lg->vector.stops.size() - 1; i++) {
                         *pdf_alpha << "<<\n/FunctionType 2\n/Domain [0 1]\n";
                         *pdf_alpha << "/C0 [" << lg->vector.stops[i].opacity << "]\n";
@@ -658,6 +659,8 @@ PrintPDF::print_fill_alpha(SVGOStringStream &/*os*/, SPStyle const *const style,
         } else if (SP_IS_RADIALGRADIENT (SP_STYLE_FILL_SERVER (style))) {
 
             SPRadialGradient *rg=SP_RADIALGRADIENT(SP_STYLE_FILL_SERVER (style));
+            sp_gradient_ensure_vector(SP_GRADIENT(rg)); // when exporting from commandline, vector is not built
+
             NR::Point c (rg->cx.computed, rg->cy.computed);
             NR::Point f (rg->fx.computed, rg->fy.computed);
             double r = rg->r.computed;
@@ -698,7 +701,6 @@ PrintPDF::print_fill_alpha(SVGOStringStream &/*os*/, SPStyle const *const style,
                     *pdf_alpha << "/Domain [0 1]\n";
                     *pdf_alpha << "/Function <<\n/FunctionType 3\n/Functions\n[\n";
 
-                    sp_gradient_ensure_vector(SP_GRADIENT(rg)); // when exporting from commandline, vector is not built
                     for (gint i = 0; unsigned(i) < rg->vector.stops.size() - 1; i++) {
                         *pdf_alpha << "<<\n/FunctionType 2\n/Domain [0 1]\n";
                         *pdf_alpha << "/C0 [" << rg->vector.stops[i].opacity << "]\n";
@@ -806,6 +808,8 @@ PrintPDF::print_fill_style(SVGOStringStream &os, SPStyle const *const style, NRR
         if (SP_IS_LINEARGRADIENT (SP_STYLE_FILL_SERVER (style))) {
 
             SPLinearGradient *lg=SP_LINEARGRADIENT(SP_STYLE_FILL_SERVER (style));
+            sp_gradient_ensure_vector(SP_GRADIENT(lg)); // when exporting from commandline, vector is not built
+
             NR::Point p1 (lg->x1.computed, lg->y1.computed);
             NR::Point p2 (lg->x2.computed, lg->y2.computed);
             if (pbox && SP_GRADIENT(lg)->units == SP_GRADIENT_UNITS_OBJECTBOUNDINGBOX) {
@@ -823,7 +827,6 @@ PrintPDF::print_fill_style(SVGOStringStream &os, SPStyle const *const style, NRR
             *pdf_shade << "/Domain [0 1]\n";
             *pdf_shade << "/Function <<\n/FunctionType 3\n/Functions\n[\n";
 
-            sp_gradient_ensure_vector(SP_GRADIENT(lg)); // when exporting from commandline, vector is not built
             for (gint i = 0; unsigned(i) < lg->vector.stops.size() - 1; i++) {
                 float rgb[3];
                 sp_color_get_rgb_floatv(&lg->vector.stops[i].color, rgb);
@@ -851,6 +854,8 @@ PrintPDF::print_fill_style(SVGOStringStream &os, SPStyle const *const style, NRR
         } else if (SP_IS_RADIALGRADIENT (SP_STYLE_FILL_SERVER (style))) {
 
             SPRadialGradient *rg=SP_RADIALGRADIENT(SP_STYLE_FILL_SERVER (style));
+            sp_gradient_ensure_vector(SP_GRADIENT(rg)); // when exporting from commandline, vector is not built
+
             NR::Point c (rg->cx.computed, rg->cy.computed);
             NR::Point f (rg->fx.computed, rg->fy.computed);
             double r = rg->r.computed;
@@ -870,7 +875,6 @@ PrintPDF::print_fill_style(SVGOStringStream &os, SPStyle const *const style, NRR
             *pdf_shade << "/Domain [0 1]\n";
             *pdf_shade << "/Function <<\n/FunctionType 3\n/Functions\n[\n";
 
-            sp_gradient_ensure_vector(SP_GRADIENT(rg)); // when exporting from commandline, vector is not built
             for (gint i = 0; unsigned(i) < rg->vector.stops.size() - 1; i++) {
                 float rgb[3];
                 sp_color_get_rgb_floatv(&rg->vector.stops[i].color, rgb);
@@ -933,8 +937,7 @@ PrintPDF::print_stroke_style(SVGOStringStream &os, SPStyle const *style)
     // invalid PS-lines such as "[0.0000000 0.0000000] 0.0000000 setdash", which should be "[] 0 setdash",
     // we first check if all components of stroke_dash.dash are 0.
     bool LineSolid = true;
-    if (style->stroke_dasharray_set &&
-        style->stroke_dash.n_dash   &&
+    if (style->stroke_dash.n_dash   &&
         style->stroke_dash.dash       )
     {
         int i = 0;
@@ -962,9 +965,9 @@ PrintPDF::print_stroke_style(SVGOStringStream &os, SPStyle const *style)
     *page_stream << style->stroke_width.computed << " w\n";
     *page_stream << style->stroke_linejoin.computed << " j\n";
     *page_stream << style->stroke_linecap.computed << " J\n";
-    if (style->stroke_miterlimit.set) {
-        *page_stream << style->stroke_miterlimit.value << " M\n";
-    }
+    *page_stream <<
+        ( style->stroke_miterlimit.value > 1 ?
+          style->stroke_miterlimit.value : 1 ) << " M\n";
 }
 
 
