@@ -24,32 +24,27 @@
 #include "inkscape.h"
 #include "sp-path.h"
 #include <style.h>
-#include <color.h>
 #include "display/curve.h"
 #include "libnr/n-art-bpath.h"
-#include "libnr/nr-point-matrix-ops.h"
 #include "extension/system.h"
-#include "extension/db.h"
 
-#include <stdio.h>
-#include <string.h>
-#include <time.h>
 
-#include <vector>
-#include <string>
 
 #include "io/sys.h"
 
-namespace Inkscape {
-namespace Extension {
-namespace Internal {
+namespace Inkscape
+{
+namespace Extension
+{
+namespace Internal
+{
 
 
 
 static const char *
 dstr(gchar *sbuffer, double d)
 {
-    return (const char *)g_ascii_formatd(sbuffer, 
+    return (const char *)g_ascii_formatd(sbuffer,
 	         G_ASCII_DTOSTR_BUF_SIZE, "%.8g", (gdouble)d);
 
 }
@@ -155,13 +150,13 @@ PovOutput::save(Inkscape::Extension::Output *mod, SPDocument *doc, gchar const *
     gchar s6[G_ASCII_DTOSTR_BUF_SIZE + 1];
     gchar s7[G_ASCII_DTOSTR_BUF_SIZE + 1];
     gchar s8[G_ASCII_DTOSTR_BUF_SIZE + 1];
-	
+
     double bignum = 1000000.0;
     double minx  =  bignum;
     double maxx  = -bignum;
     double miny  =  bignum;
     double maxy  = -bignum;
-    
+
 
 
     unsigned indx;
@@ -222,12 +217,12 @@ PovOutput::save(Inkscape::Extension::Output *mod, SPDocument *doc, gchar const *
 
         //Count the NR_CURVETOs/LINETOs
         int segmentCount=0;
-        NArtBpath *bp = curve->bpath;
+        NArtBpath *bp = SP_CURVE_BPATH(curve);
         for (curveNr=0 ; curveNr<curve->length ; curveNr++, bp++)
             if (bp->code == NR_CURVETO || bp->code == NR_LINETO)
                 segmentCount++;
 
-        bp = curve->bpath;
+        bp = SP_CURVE_BPATH(curve);
         double cminx  =  bignum;
         double cmaxx  = -bignum;
         double cminy  =  bignum;
@@ -245,7 +240,7 @@ PovOutput::save(Inkscape::Extension::Output *mod, SPDocument *doc, gchar const *
         fprintf(f, "    0.0, //bottom\n");
         fprintf(f, "    %d, //nr points\n", segmentCount * 4);
         int segmentNr = 0;
-        for (bp = curve->bpath, curveNr=0 ; curveNr<curve->length ; curveNr++, bp++) {
+        for (bp = SP_CURVE_BPATH(curve), curveNr=0 ; curveNr<curve->length ; curveNr++, bp++) {
             using NR::X;
             using NR::Y;
             NR::Point const p1(bp->c(1) * tf);
@@ -447,6 +442,8 @@ PovOutput::save(Inkscape::Extension::Output *mod, SPDocument *doc, gchar const *
     fclose(f);
 }
 
+#include "clear-n_.h"
+
 /**
  * This is the definition of PovRay output.  This function just
  * calls the extension system with the memory allocated XML that
@@ -457,13 +454,13 @@ PovOutput::init()
 {
     Inkscape::Extension::build_from_mem(
         "<inkscape-extension>\n"
-            "<name>PovRay Output</name>\n"
+            "<name>" N_("PovRay Output") "</name>\n"
             "<id>org.inkscape.output.pov</id>\n"
             "<output>\n"
                 "<extension>.pov</extension>\n"
                 "<mimetype>text/x-povray-script</mimetype>\n"
-                "<filetypename>PovRay (*.pov) (export splines)</filetypename>\n"
-                "<filetypetooltip>PovRay Raytracer File</filetypetooltip>\n"
+                "<filetypename>" N_("PovRay (*.pov) (export splines)") "</filetypename>\n"
+                "<filetypetooltip>" N_("PovRay Raytracer File") "</filetypetooltip>\n"
             "</output>\n"
         "</inkscape-extension>",
         new PovOutput());
