@@ -385,6 +385,10 @@ PrintPS::begin(Inkscape::Extension::Print *mod, SPDocument *doc)
                << d.x0 << " "
                << (_height - d.y0) << " "
                << d.x1 << "\n";
+            os << "%%DocumentMedia: plain "
+               << (int) ceil(_height) << " "
+               << (int) ceil(_width) << " "
+               << "0 () ()\n";
         } else {
             os << "%%Orientation: Portrait\n";
             os << "%%BoundingBox: " << (int) d.x0 << " "
@@ -395,6 +399,10 @@ PrintPS::begin(Inkscape::Extension::Print *mod, SPDocument *doc)
                << d.y0 << " "
                << d.x1 << " "
                << d.y1 << "\n";
+            os << "%%DocumentMedia: plain "
+               << (int) ceil(_width) << " "
+               << (int) ceil(_height) << " "
+               << "0 () ()\n";
         }
 
         os << "%%EndComments\n";
@@ -656,8 +664,7 @@ PrintPS::print_stroke_style(SVGOStringStream &os, SPStyle const *style)
     // invalid PS-lines such as "[0.0000000 0.0000000] 0.0000000 setdash", which should be "[] 0 setdash",
     // we first check if all components of stroke_dash.dash are 0.
     bool LineSolid = true;
-    if (style->stroke_dasharray_set &&
-        style->stroke_dash.n_dash   &&
+    if (style->stroke_dash.n_dash   &&
         style->stroke_dash.dash       )
     {
         int i = 0;
@@ -890,10 +897,10 @@ PrintPS::text(Inkscape::Extension::Print *mod, char const *text, NR::Point p,
                   "definefont} def\n";      // create the new font and leave it on the stack, define the proc
             _newlatin1font_proc_defined = true;
         }
-        os << "/" << fn << "-ISOLatin1 /" << fn << " newlatin1font\n";
+        os << "/(" << fn << "-ISOLatin1) /(" << fn << ") newlatin1font\n";
         _latin1_encoded_fonts.insert(fn);
     } else
-        os << "/" << fn << "-ISOLatin1 findfont\n";
+        os << "/(" << fn << "-ISOLatin1) findfont\n";
     os << style->font_size.computed << " scalefont\n";
     os << "setfont\n";
     g_free((void *) fn);
