@@ -20,11 +20,17 @@
 namespace Inkscape {
 namespace Extension {
 
+/** \brief  A class to represent the preferences for an extension */
 class PrefDialog : public Gtk::Dialog {
-    Gtk::Socket * _socket;
+    /** \brief  Help string if it exists */
+    gchar const * _help;
+    /** \brief  Name of the extension */
+    Glib::ustring _name;
 
 public:
-    PrefDialog (Glib::ustring name, Gtk::Widget * controls);
+    PrefDialog (Glib::ustring name, gchar const * help, Gtk::Widget * controls);
+    int run (void);
+
 };
 
 
