@@ -1,7 +1,8 @@
-
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "InkscapeBinding.h"
-
 #include "help.h"
 
 namespace Inkscape {
