@@ -15,10 +15,6 @@
 
 #include <glib-object.h>
 
-#include <glib.h>
-
-
-
 /* Generic containers */
 
 namespace Inkscape {
@@ -174,9 +170,9 @@ namespace Inkscape {
 namespace UI {
 namespace View {
 class View;
-};
-};
-};
+}
+}
+}
 
 class SPViewWidget;
 class SPViewWidgetClass;
