@@ -1,5 +1,6 @@
-/*
- *  geom.h
+/**
+ *  \file geom.h
+ *  \brief Various geometrical calculations
  *
  *  Authors:
  *   Nathan Hurst <njh@mail.csse.monash.edu.au>
@@ -7,17 +8,16 @@
  * Copyright (C) 1999-2002 authors
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
- *
  */
 
 #include "libnr/nr-forward.h"
 
-typedef enum sp_intersector_kind{
-	intersects = 0,
-	parallel,
-	coincident,
-	no_intersection
-} sp_intersector_kind;
+enum IntersectorKind {
+    INTERSECTS = 0,
+    PARALLEL,
+    COINCIDENT,
+    NO_INTERSECTION
+};
 
 /* Define here various primatives, such as line, line segment, circle, bezier path etc. */
 
@@ -25,6 +25,6 @@ typedef enum sp_intersector_kind{
 
 /* intersectors */
 
-sp_intersector_kind sp_intersector_line_intersection(NR::Point const &n0, double const d0,
-						     NR::Point const &n1, double const d1,
-						     NR::Point &result);
+IntersectorKind intersector_line_intersection(NR::Point const &n0, double const d0,
+					      NR::Point const &n1, double const d1,
+					      NR::Point &result);
