@@ -9,9 +9,7 @@
  * This code is in the Public Domain
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include "nr-macros.h"
+#include <glib/gmem.h>
 #include "nr-pixblock.h"
 
 /// Size of buffer that needs no allocation (default 4).
@@ -53,23 +51,29 @@ nr_pixblock_setup_fast (NRPixBlock *pb, NR_PIXBLOCK_MODE mode, int x0, int y0, i
 	} else if (size <= 65536) {
 		pb->size = NR_PIXBLOCK_SIZE_64K;
 		pb->data.px = nr_pixelstore_64K_new (clear, 0x0);
+	} else if (size <= 262144) {
+		pb->size = NR_PIXBLOCK_SIZE_256K;
+		pb->data.px = nr_pixelstore_256K_new (clear, 0x0);
+	} else if (size <= 1048576) {
+		pb->size = NR_PIXBLOCK_SIZE_1M;
+		pb->data.px = nr_pixelstore_1M_new (clear, 0x0);
 	} else {
 		pb->size = NR_PIXBLOCK_SIZE_BIG;
-		pb->data.px = nr_new (unsigned char, size);
+		pb->data.px = g_new (unsigned char, size);
 		if (clear) memset (pb->data.px, 0x0, size);
 	}
 
 	pb->mode = mode;
 	pb->empty = 1;
-	pb->area.x0 = x0;
-	pb->area.y0 = y0;
-	pb->area.x1 = x1;
-	pb->area.y1 = y1;
+    pb->visible_area.x0 = pb->area.x0 = x0;
+    pb->visible_area.y0 = pb->area.y0 = y0;
+    pb->visible_area.x1 = pb->area.x1 = x1;
+    pb->visible_area.y1 = pb->area.y1 = y1;
 	pb->rs = bpp * w;
 }
 
 /**
- * Pixbuf initialisation using nr_new.
+ * Pixbuf initialisation using g_new.
  *
  * After allocating memory, the buffer is cleared if the clear flag is set.
  * \param pb Pointer to the pixbuf struct.
@@ -94,16 +98,16 @@ nr_pixblock_setup (NRPixBlock *pb, NR_PIXBLOCK_MODE mode, int x0, int y0, int x1
 		if (clear) memset (pb->data.p, 0x0, size);
 	} else {
 		pb->size = NR_PIXBLOCK_SIZE_BIG;
-		pb->data.px = nr_new (unsigned char, size);
+		pb->data.px = g_new (unsigned char, size);
 		if (clear) memset (pb->data.px, 0x0, size);
 	}
 
 	pb->mode = mode;
 	pb->empty = 1;
-	pb->area.x0 = x0;
-	pb->area.y0 = y0;
-	pb->area.x1 = x1;
-	pb->area.y1 = y1;
+    pb->visible_area.x0 = pb->area.x0 = x0;
+    pb->visible_area.y0 = pb->area.y0 = y0;
+    pb->visible_area.x1 = pb->area.x1 = x1;
+    pb->visible_area.y1 = pb->area.y1 = y1;
 	pb->rs = bpp * w;
 }
 
@@ -127,10 +131,10 @@ nr_pixblock_setup_extern (NRPixBlock *pb, NR_PIXBLOCK_MODE mode, int x0, int y0,
 	pb->size = NR_PIXBLOCK_SIZE_STATIC; 
 	pb->mode = mode;
 	pb->empty = empty;
-	pb->area.x0 = x0;
-	pb->area.y0 = y0;
-	pb->area.x1 = x1;
-	pb->area.y1 = y1;
+	pb->visible_area.x0 = pb->area.x0 = x0;
+	pb->visible_area.y0 = pb->area.y0 = y0;
+	pb->visible_area.x1 = pb->area.x1 = x1;
+	pb->visible_area.y1 = pb->area.y1 = y1;
 	pb->data.px = px;
 	pb->rs = rs;
 
@@ -173,8 +177,14 @@ nr_pixblock_release (NRPixBlock *pb)
 	case NR_PIXBLOCK_SIZE_64K:
 		nr_pixelstore_64K_free (pb->data.px);
 		break;
+	case NR_PIXBLOCK_SIZE_256K:
+		nr_pixelstore_256K_free (pb->data.px);
+		break;
+	case NR_PIXBLOCK_SIZE_1M:
+		nr_pixelstore_1M_free (pb->data.px);
+		break;
 	case NR_PIXBLOCK_SIZE_BIG:
-		nr_free (pb->data.px);
+		g_free (pb->data.px);
 		break;
 	case NR_PIXBLOCK_SIZE_STATIC:
 		break;
@@ -187,14 +197,14 @@ nr_pixblock_release (NRPixBlock *pb)
  * Allocates NRPixBlock and sets it up.
  *
  * \return Pointer to fresh pixblock.
- * Calls nr_new() and nr_pixblock_setup().
+ * Calls g_new() and nr_pixblock_setup().
  */
 NRPixBlock *
 nr_pixblock_new (NR_PIXBLOCK_MODE mode, int x0, int y0, int x1, int y1, bool clear)
 {
 	NRPixBlock *pb;
 
-	pb = nr_new (NRPixBlock, 1);
+	pb = g_new (NRPixBlock, 1);
 
 	nr_pixblock_setup (pb, mode, x0, y0, x1, y1, clear);
 
@@ -211,7 +221,7 @@ nr_pixblock_free (NRPixBlock *pb)
 {
 	nr_pixblock_release (pb);
 
-	nr_free (pb);
+	g_free (pb);
 
 	return NULL;
 }
@@ -232,7 +242,7 @@ nr_pixelstore_4K_new (bool clear, unsigned char val)
 		nr_4K_len -= 1;
 		px = nr_4K_px[nr_4K_len];
 	} else {
-		px = nr_new (unsigned char, 4096);
+		px = g_new (unsigned char, 4096);
 	}
 	
 	if (clear) memset (px, val, 4096);
@@ -245,7 +255,7 @@ nr_pixelstore_4K_free (unsigned char *px)
 {
 	if (nr_4K_len == nr_4K_size) {
 		nr_4K_size += NR_4K_BLOCK;
-		nr_4K_px = nr_renew (nr_4K_px, unsigned char *, nr_4K_size);
+		nr_4K_px = g_renew (unsigned char *, nr_4K_px, nr_4K_size);
 	}
 
 	nr_4K_px[nr_4K_len] = px;
@@ -266,7 +276,7 @@ nr_pixelstore_16K_new (bool clear, unsigned char val)
 		nr_16K_len -= 1;
 		px = nr_16K_px[nr_16K_len];
 	} else {
-		px = nr_new (unsigned char, 16384);
+		px = g_new (unsigned char, 16384);
 	}
 	
 	if (clear) memset (px, val, 16384);
@@ -279,7 +289,7 @@ nr_pixelstore_16K_free (unsigned char *px)
 {
 	if (nr_16K_len == nr_16K_size) {
 		nr_16K_size += NR_16K_BLOCK;
-		nr_16K_px = nr_renew (nr_16K_px, unsigned char *, nr_16K_size);
+		nr_16K_px = g_renew (unsigned char *, nr_16K_px, nr_16K_size);
 	}
 
 	nr_16K_px[nr_16K_len] = px;
@@ -300,7 +310,7 @@ nr_pixelstore_64K_new (bool clear, unsigned char val)
 		nr_64K_len -= 1;
 		px = nr_64K_px[nr_64K_len];
 	} else {
-		px = nr_new (unsigned char, 65536);
+		px = g_new (unsigned char, 65536);
 	}
 
 	if (clear) memset (px, val, 65536);
@@ -313,20 +323,90 @@ nr_pixelstore_64K_free (unsigned char *px)
 {
 	if (nr_64K_len == nr_64K_size) {
 		nr_64K_size += NR_64K_BLOCK;
-		nr_64K_px = nr_renew (nr_64K_px, unsigned char *, nr_64K_size);
+		nr_64K_px = g_renew (unsigned char *, nr_64K_px, nr_64K_size);
 	}
 
 	nr_64K_px[nr_64K_len] = px;
 	nr_64K_len += 1;
 }
 
+#define NR_256K_BLOCK 32
+#define NR_256K 262144
+static unsigned char **nr_256K_px = NULL;
+static unsigned int nr_256K_len = 0;
+static unsigned int nr_256K_size = 0;
+
+unsigned char *
+nr_pixelstore_256K_new (bool clear, unsigned char val)
+{
+	unsigned char *px;
+
+	if (nr_256K_len != 0) {
+		nr_256K_len -= 1;
+		px = nr_256K_px[nr_256K_len];
+	} else {
+           px = g_new (unsigned char, NR_256K);
+	}
+
+	if (clear) memset (px, val, NR_256K);
+
+	return px;
+}
+
+void
+nr_pixelstore_256K_free (unsigned char *px)
+{
+	if (nr_256K_len == nr_256K_size) {
+		nr_256K_size += NR_256K_BLOCK;
+		nr_256K_px = g_renew (unsigned char *, nr_256K_px, nr_256K_size);
+	}
+
+	nr_256K_px[nr_256K_len] = px;
+	nr_256K_len += 1;
+}
+
+#define NR_1M_BLOCK 32
+#define NR_1M 1048576
+static unsigned char **nr_1M_px = NULL;
+static unsigned int nr_1M_len = 0;
+static unsigned int nr_1M_size = 0;
+
+unsigned char *
+nr_pixelstore_1M_new (bool clear, unsigned char val)
+{
+	unsigned char *px;
+
+	if (nr_1M_len != 0) {
+		nr_1M_len -= 1;
+		px = nr_1M_px[nr_1M_len];
+	} else {
+           px = g_new (unsigned char, NR_1M);
+	}
+
+	if (clear) memset (px, val, NR_1M);
+
+	return px;
+}
+
+void
+nr_pixelstore_1M_free (unsigned char *px)
+{
+	if (nr_1M_len == nr_1M_size) {
+		nr_1M_size += NR_1M_BLOCK;
+		nr_1M_px = g_renew (unsigned char *, nr_1M_px, nr_1M_size);
+	}
+
+	nr_1M_px[nr_1M_len] = px;
+	nr_1M_len += 1;
+}
+
 /*
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
