@@ -7,7 +7,6 @@
  */
 
 #include "livarot/Shape.h"
-#include <libnr/nr-point.h>
 #include <libnr/nr-point-fns.h>
 #include "livarot/Path.h"
 #include "livarot/path-description.h"
@@ -36,8 +35,6 @@ Shape::ConvertToForme (Path * dest)
 {
   if (numberOfPoints() <= 1 || numberOfEdges() <= 1)
     return;
-  if (directedEulerian(this) == false)
-    return;
   
   // prepare
   dest->Reset ();
@@ -542,8 +539,11 @@ Shape::MakeOffset (Shape * a, double dec, JoinType join, double miter)
     if (numberOfPoints() > maxPt)
     {
       maxPt = numberOfPoints();
-      if (_has_points_data)
+      if (_has_points_data) {
         pData.resize(maxPt);
+        _point_data_initialised = false;
+        _bbox_up_to_date = false;
+        }
     }
     
     _aretes = a->_aretes;
