@@ -39,9 +39,21 @@ struct SPNodeContext {
 	gboolean leftctrl;
 	gboolean rightctrl;
 
+      /// If true, rubberband was cancelled by esc, so the next button release should not deselect.
+	bool rb_escaped;
+
 	sigc::connection sel_changed_connection;
 
 	Inkscape::MessageContext *_node_message_context;
+
+	double grab_t;
+	Inkscape::NodePath::Node * grab_node;
+	bool hit;
+	NR::Point curvepoint_event; // int coords from event
+	NR::Point curvepoint_doc; // same, in doc coords
+	bool cursor_drag;
+
+	bool added_node;
 };
 
 struct SPNodeContextClass {
