@@ -18,19 +18,15 @@
 # include "config.h"
 #endif
 
-#include <math.h>
-#include <string.h>
 
 #include "libnr/n-art-bpath.h"
-#include "libnr/nr-matrix.h"
+#include "libnr/nr-path.h"
 #include "libnr/nr-matrix-fns.h"
-#include "libnr/nr-point-matrix-ops.h"
 #include "svg/svg.h"
 #include "svg/stringstream.h"
 #include "xml/repr.h"
 #include "attributes.h"
 #include "style.h"
-#include "version.h"
 #include "display/curve.h"
 #include <glibmm/i18n.h>
 
@@ -127,10 +123,10 @@ static void sp_genericellipse_class_init(SPGenericEllipseClass *klass)
 static void
 sp_genericellipse_init(SPGenericEllipse *ellipse)
 {
-    sp_svg_length_unset(&ellipse->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
-    sp_svg_length_unset(&ellipse->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
-    sp_svg_length_unset(&ellipse->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
-    sp_svg_length_unset(&ellipse->ry, SP_SVG_UNIT_NONE, 0.0, 0.0);
+    ellipse->cx.unset();
+    ellipse->cy.unset();
+    ellipse->rx.unset();
+    ellipse->ry.unset();
 
     ellipse->start = 0.0;
     ellipse->end = SP_2PI;
@@ -146,10 +142,10 @@ sp_genericellipse_update(SPObject *object, SPCtx *ctx, guint flags)
         double const d = 1.0 / NR::expansion(((SPItemCtx const *) ctx)->i2vp);
         double const em = style->font_size.computed;
         double const ex = em * 0.5; // fixme: get from pango or libnrtype
-        sp_svg_length_update(&ellipse->cx, em, ex, d);
-        sp_svg_length_update(&ellipse->cy, em, ex, d);
-        sp_svg_length_update(&ellipse->rx, em, ex, d);
-        sp_svg_length_update(&ellipse->ry, em, ex, d);
+        ellipse->cx.update(em, ex, d);
+        ellipse->cy.update(em, ex, d);
+        ellipse->rx.update(em, ex, d);
+        ellipse->ry.update(em, ex, d);
         sp_shape_set_shape((SPShape *) object);
     }
 
@@ -403,26 +399,22 @@ sp_ellipse_set(SPObject *object, unsigned int key, gchar const *value)
 
     switch (key) {
         case SP_ATTR_CX:
-            if (!sp_svg_length_read(value, &ellipse->cx)) {
-                sp_svg_length_unset(&ellipse->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
-            }
+            ellipse->cx.readOrUnset(value);
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_CY:
-            if (!sp_svg_length_read(value, &ellipse->cy)) {
-                sp_svg_length_unset(&ellipse->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
-            }
+            ellipse->cy.readOrUnset(value);
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_RX:
-            if (!sp_svg_length_read(value, &ellipse->rx) || (ellipse->rx.value <= 0.0)) {
-                sp_svg_length_unset(&ellipse->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
+            if (!ellipse->rx.read(value) || (ellipse->rx.value <= 0.0)) {
+                ellipse->rx.unset();
             }
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_RY:
-            if (!sp_svg_length_read(value, &ellipse->ry) || (ellipse->ry.value <= 0.0)) {
-                sp_svg_length_unset(&ellipse->ry, SP_SVG_UNIT_NONE, 0.0, 0.0);
+            if (!ellipse->ry.read(value) || (ellipse->ry.value <= 0.0)) {
+                ellipse->ry.unset();
             }
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
@@ -553,20 +545,16 @@ sp_circle_set(SPObject *object, unsigned int key, gchar const *value)
 
     switch (key) {
         case SP_ATTR_CX:
-            if (!sp_svg_length_read(value, &ge->cx)) {
-                sp_svg_length_unset(&ge->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
-            }
+            ge->cx.readOrUnset(value);
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_CY:
-            if (!sp_svg_length_read(value, &ge->cy)) {
-                sp_svg_length_unset(&ge->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
-            }
+            ge->cy.readOrUnset(value);
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_R:
-            if (!sp_svg_length_read(value, &ge->rx) || (ge->rx.value <= 0.0)) {
-                sp_svg_length_unset(&ge->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
+            if (!ge->rx.read(value) || ge->rx.value <= 0.0) {
+                ge->rx.unset();
             }
             ge->ry = ge->rx;
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
@@ -649,32 +637,14 @@ sp_arc_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
 
     Inkscape::Version version = sp_object_get_sodipodi_version(object);
 
-    if (sp_version_inside_range(version, 0, 0, 0, 25)) {
-        /* Old spec violating arc attributes */
-        sp_object_read_attr(object, "cx");
-        sp_object_read_attr(object, "cy");
-        sp_object_read_attr(object, "rx");
-        sp_object_read_attr(object, "ry");
-    } else {
-        /* New attributes */
-        sp_object_read_attr(object, "sodipodi:cx");
-        sp_object_read_attr(object, "sodipodi:cy");
-        sp_object_read_attr(object, "sodipodi:rx");
-        sp_object_read_attr(object, "sodipodi:ry");
-    }
+    sp_object_read_attr(object, "sodipodi:cx");
+    sp_object_read_attr(object, "sodipodi:cy");
+    sp_object_read_attr(object, "sodipodi:rx");
+    sp_object_read_attr(object, "sodipodi:ry");
 
     sp_object_read_attr(object, "sodipodi:start");
     sp_object_read_attr(object, "sodipodi:end");
     sp_object_read_attr(object, "sodipodi:open");
-
-    if (sp_version_inside_range(version, 0, 0, 0, 25)) {
-        /* fixme: I am 99.9% sure we can do this here safely, but check nevertheless (Lauris) */
-        sp_arc_write(object, repr, SP_OBJECT_WRITE_EXT);
-        sp_repr_set_attr(repr, "cx", NULL);
-        sp_repr_set_attr(repr, "cy", NULL);
-        sp_repr_set_attr(repr, "rx", NULL);
-        sp_repr_set_attr(repr, "ry", NULL);
-    }
 }
 
 /*
@@ -723,7 +693,8 @@ sp_arc_set_elliptical_path_attribute(SPArc *arc, Inkscape::XML::Node *repr)
 
         }
     }
-    return sp_repr_set_attr(repr, "d", os.str().c_str());
+    repr->setAttribute("d", os.str().c_str());
+    return true;
 }
 
 static Inkscape::XML::Node *
@@ -737,7 +708,7 @@ sp_arc_write(SPObject *object, Inkscape::XML::Node *repr, guint flags)
     }
 
     if (flags & SP_OBJECT_WRITE_EXT) {
-        sp_repr_set_attr(repr, "sodipodi:type", "arc");
+        repr->setAttribute("sodipodi:type", "arc");
         sp_repr_set_svg_double(repr, "sodipodi:cx", ge->cx.computed);
         sp_repr_set_svg_double(repr, "sodipodi:cy", ge->cy.computed);
         sp_repr_set_svg_double(repr, "sodipodi:rx", ge->rx.computed);
@@ -749,11 +720,11 @@ sp_arc_write(SPObject *object, Inkscape::XML::Node *repr, guint flags)
         if (!(fabs(len) < 1e-8 || fabs(len - SP_2PI) < 1e-8)) {
             sp_repr_set_svg_double(repr, "sodipodi:start", ge->start);
             sp_repr_set_svg_double(repr, "sodipodi:end", ge->end);
-            sp_repr_set_attr(repr, "sodipodi:open", (!ge->closed) ? "true" : NULL);
+            repr->setAttribute("sodipodi:open", (!ge->closed) ? "true" : NULL);
         } else {
-            sp_repr_set_attr(repr, "sodipodi:end", NULL);
-            sp_repr_set_attr(repr, "sodipodi:start", NULL);
-            sp_repr_set_attr(repr, "sodipodi:open", NULL);
+            repr->setAttribute("sodipodi:end", NULL);
+            repr->setAttribute("sodipodi:start", NULL);
+            repr->setAttribute("sodipodi:open", NULL);
         }
     }
 
@@ -771,47 +742,24 @@ sp_arc_set(SPObject *object, unsigned int key, gchar const *value)
 {
     SPGenericEllipse *ge = SP_GENERICELLIPSE(object);
 
-    Inkscape::Version version = sp_object_get_sodipodi_version(object);
-
-    if (sp_version_inside_range(version, 0, 0, 0, 25)) {
-        switch (key) {
-            case SP_ATTR_CX:
-                key = SP_ATTR_SODIPODI_CX;
-                break;
-            case SP_ATTR_CY:
-                key = SP_ATTR_SODIPODI_CY;
-                break;
-            case SP_ATTR_RX:
-                key = SP_ATTR_SODIPODI_RX;
-                break;
-            case SP_ATTR_RY:
-                key = SP_ATTR_SODIPODI_RY;
-                break;
-        }
-    }
-
     switch (key) {
         case SP_ATTR_SODIPODI_CX:
-            if (!sp_svg_length_read(value, &ge->cx)) {
-                sp_svg_length_unset(&ge->cx, SP_SVG_UNIT_NONE, 0.0, 0.0);
-            }
+            ge->cx.readOrUnset(value);
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_SODIPODI_CY:
-            if (!sp_svg_length_read(value, &ge->cy)) {
-                sp_svg_length_unset(&ge->cy, SP_SVG_UNIT_NONE, 0.0, 0.0);
-            }
+            ge->cy.readOrUnset(value);
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_SODIPODI_RX:
-            if (!sp_svg_length_read(value, &ge->rx) || (ge->rx.computed <= 0.0)) {
-                sp_svg_length_unset(&ge->rx, SP_SVG_UNIT_NONE, 0.0, 0.0);
+            if (!ge->rx.read(value) || ge->rx.computed <= 0.0) {
+                ge->rx.unset();
             }
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_SODIPODI_RY:
-            if (!sp_svg_length_read(value, &ge->ry) || (ge->ry.computed <= 0.0)) {
-                sp_svg_length_unset(&ge->ry, SP_SVG_UNIT_NONE, 0.0, 0.0);
+            if (!ge->ry.read(value) || ge->ry.computed <= 0.0) {
+                ge->ry.unset();
             }
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
