@@ -4,12 +4,10 @@
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
-#include <string.h>
 #include <glibmm/i18n.h>
 
 #include "attributes.h"
 #include "xml/repr.h"
-#include "svg/svg.h"
 #include "style.h"
 #include "inkscape.h"
 #include "document.h"
@@ -17,34 +15,22 @@
 #include "desktop-handles.h"
 #include "desktop.h"
 #include "desktop-affine.h"
-#include "print.h"
 
-#include "libnr/nr-matrix.h"
-#include "libnr/nr-point.h"
 #include "xml/repr.h"
 
 #include "sp-flowdiv.h"
 #include "sp-flowregion.h"
 #include "sp-flowtext.h"
 #include "sp-string.h"
-#include "sp-text.h"
 #include "sp-use.h"
 #include "sp-rect.h"
+#include "text-tag-attributes.h"
 
-#include "libnr/nr-matrix.h"
-#include "libnr/nr-translate.h"
-#include "libnr/nr-scale.h"
-#include "libnr/nr-matrix-ops.h"
-#include "libnr/nr-translate-ops.h"
-#include "libnr/nr-scale-ops.h"
 
 #include "livarot/Shape.h"
 
-#include "display/nr-arena-item.h"
-#include "display/nr-arena-group.h"
 #include "display/nr-arena-glyphs.h"
 
-#include <pango/pango.h>
 
 static void sp_flowtext_class_init(SPFlowtextClass *klass);
 static void sp_flowtext_init(SPFlowtext *group);
@@ -117,6 +103,7 @@ sp_flowtext_class_init(SPFlowtextClass *klass)
 static void
 sp_flowtext_init(SPFlowtext *group)
 {
+    group->par_indent = 0;
     new (&group->layout) Inkscape::Text::Layout();
 }
 
@@ -185,7 +172,7 @@ sp_flowtext_update(SPObject *object, SPCtx *ctx, unsigned flags)
 
     group->rebuildLayout();
 
-    // pass the bbox of the flowtext object as paintbox (used for paintserver fills)	
+    // pass the bbox of the flowtext object as paintbox (used for paintserver fills)
     NRRect paintbox;
     sp_item_invoke_bbox(group, &paintbox, NR::identity(), TRUE);
     for (SPItemView *v = group->display; v != NULL; v = v->next) {
@@ -266,6 +253,7 @@ sp_flowtext_set(SPObject *object, unsigned key, gchar const *value)
                     }
                 }
             }
+            */
             {   // This would probably translate to padding-left, if SPStyle had it.
                 gchar const *val = sp_repr_css_property(opts, "par-indent", NULL);
                 if ( val == NULL ) {
@@ -274,7 +262,6 @@ sp_flowtext_set(SPObject *object, unsigned key, gchar const *value)
                     sp_repr_get_double((Inkscape::XML::Node*)opts, "par-indent", &group->par_indent);
                 }
             }
-            */
             sp_repr_css_attr_unref(opts);
             object->requestModified(SP_OBJECT_MODIFIED_FLAG);
             break;
@@ -297,12 +284,12 @@ sp_flowtext_write(SPObject *object, Inkscape::XML::Node *repr, guint flags)
             Inkscape::XML::Node *c_repr = NULL;
             if ( SP_IS_FLOWDIV(child) || SP_IS_FLOWPARA(child) || SP_IS_FLOWREGION(child) || SP_IS_FLOWREGIONEXCLUDE(child)) {
                 c_repr = child->updateRepr(NULL, flags);
-            } 
+            }
             if ( c_repr ) l = g_slist_prepend(l, c_repr);
         }
         while ( l ) {
-            sp_repr_add_child(repr, (Inkscape::XML::Node *) l->data, NULL);
-            sp_repr_unref((Inkscape::XML::Node *) l->data);
+            repr->addChild((Inkscape::XML::Node *) l->data, NULL);
+            Inkscape::GC::release((Inkscape::XML::Node *) l->data);
             l = g_slist_remove(l, l->data);
         }
     } else {
@@ -324,6 +311,21 @@ sp_flowtext_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform,
 {
     SPFlowtext *group = SP_FLOWTEXT(item);
     group->layout.getBoundingBox(bbox, transform);
+
+    // Add stroke width
+    SPStyle* style=SP_OBJECT_STYLE (item);
+    if (style->stroke.type != SP_PAINT_TYPE_NONE) {
+        double const scale = expansion(transform);
+        if ( fabs(style->stroke_width.computed * scale) > 0.01 ) { // sinon c'est 0=oon veut pas de bord
+            double const width = MAX(0.125, style->stroke_width.computed * scale);
+            if ( fabs(bbox->x1 - bbox->x0) > -0.00001 && fabs(bbox->y1 - bbox->y0) > -0.00001 ) {
+                bbox->x0-=0.5*width;
+                bbox->x1+=0.5*width;
+                bbox->y0-=0.5*width;
+                bbox->y1+=0.5*width;
+            }
+        }
+    }
 }
 
 static void
@@ -340,8 +342,7 @@ sp_flowtext_print(SPItem *item, SPPrintContext *ctx)
     dbox.y0 = 0.0;
     dbox.x1 = sp_document_width(SP_OBJECT_DOCUMENT(item));
     dbox.y1 = sp_document_height(SP_OBJECT_DOCUMENT(item));
-    NRMatrix ctm;
-    sp_item_i2d_affine(item, &ctm);
+    NR::Matrix const ctm = sp_item_i2d_affine(item);
 
     group->layout.print(ctx, &pbox, &dbox, &bbox, ctm);
 }
@@ -352,9 +353,9 @@ static gchar *sp_flowtext_description(SPItem *item)
     Inkscape::Text::Layout const &layout = SP_FLOWTEXT(item)->layout;
     int const nChars = layout.iteratorToCharIndex(layout.end());
     if (SP_FLOWTEXT(item)->has_internal_frame())
-        return g_strdup_printf(_("<b>Flowed text</b> (%d characters)"), nChars);
-    else 
-        return g_strdup_printf(_("<b>Linked flowed text</b> (%d characters)"), nChars);
+        return g_strdup_printf(ngettext("<b>Flowed text</b> (%d character)", "<b>Flowed text</b> (%d characters)", nChars), nChars);
+    else
+        return g_strdup_printf(ngettext("<b>Linked flowed text</b> (%d character)", "<b>Linked flowed text</b> (%d characters)", nChars), nChars);
 }
 
 static NRArenaItem *
@@ -364,7 +365,7 @@ sp_flowtext_show(SPItem *item, NRArena *arena, unsigned/* key*/, unsigned /*flag
     NRArenaGroup *flowed = NRArenaGroup::create(arena);
     nr_arena_group_set_transparent(flowed, FALSE);
 
-    // pass the bbox of the flowtext object as paintbox (used for paintserver fills)	
+    // pass the bbox of the flowtext object as paintbox (used for paintserver fills)
     NRRect paintbox;
     sp_item_invoke_bbox(item, &paintbox, NR::identity(), TRUE);
     group->layout.show(flowed, &paintbox);
@@ -386,11 +387,31 @@ sp_flowtext_hide(SPItem *item, unsigned int key)
 
 void SPFlowtext::_buildLayoutInput(SPObject *root, Shape const *exclusion_shape, std::list<Shape> *shapes, SPObject **pending_line_break_object)
 {
+    Inkscape::Text::Layout::OptionalTextTagAttrs pi;
+    bool with_indent = false;
+
+    if (SP_IS_FLOWPARA(root)) {
+        // emulate par-indent with the first char's kern
+        SPObject *t = root;
+        for ( ; t != NULL && !SP_IS_FLOWTEXT(t); t = SP_OBJECT_PARENT(t));
+        if (SP_IS_FLOWTEXT(t)) {
+            double indent = SP_FLOWTEXT(t)->par_indent;
+            if (indent != 0) {
+                with_indent = true;
+                SVGLength sl;
+                sl.value = sl.computed = indent;
+                sl._set = true;
+                pi.dx.push_back(sl);
+            }
+        }
+    }
+
     if (*pending_line_break_object) {
-        if (SP_IS_FLOWREGIONBREAK(*pending_line_break_object))
+        if (SP_IS_FLOWREGIONBREAK(*pending_line_break_object)) {
             layout.appendControlCode(Inkscape::Text::Layout::SHAPE_BREAK, *pending_line_break_object);
-        else
+        } else {
             layout.appendControlCode(Inkscape::Text::Layout::PARAGRAPH_BREAK, *pending_line_break_object);
+        }
         *pending_line_break_object = NULL;
     }
 
@@ -399,11 +420,15 @@ void SPFlowtext::_buildLayoutInput(SPObject *root, Shape const *exclusion_shape,
             if (*pending_line_break_object) {
                 if (SP_IS_FLOWREGIONBREAK(*pending_line_break_object))
                     layout.appendControlCode(Inkscape::Text::Layout::SHAPE_BREAK, *pending_line_break_object);
-                else
+                else {
                     layout.appendControlCode(Inkscape::Text::Layout::PARAGRAPH_BREAK, *pending_line_break_object);
+                }
                 *pending_line_break_object = NULL;
             }
-            layout.appendText(SP_STRING(child)->string, root->style, child);
+            if (with_indent)
+                layout.appendText(SP_STRING(child)->string, root->style, child, &pi);
+            else
+                layout.appendText(SP_STRING(child)->string, root->style, child);
         } else if (SP_IS_FLOWREGION(child)) {
             std::vector<Shape*> const &computed = SP_FLOWREGION(child)->computed;
             for (std::vector<Shape*>::const_iterator it = computed.begin() ; it != computed.end() ; it++) {
@@ -477,7 +502,7 @@ void SPFlowtext::_clearFlow(NRArenaGroup *in_arena)
 void SPFlowtext::convert_to_text()
 {
     SPDesktop *desktop = SP_ACTIVE_DESKTOP;
-    Inkscape::Selection *selection = SP_DT_SELECTION(desktop);
+    Inkscape::Selection *selection = sp_desktop_selection(desktop);
     SPItem *item = selection->singleItem();
     if (!SP_IS_FLOWTEXT(item)) return;
 
@@ -493,7 +518,7 @@ void SPFlowtext::convert_to_text()
     sp_repr_set_svg_double(repr, "y", anchor_point[NR::Y]);
 
     for (Inkscape::Text::Layout::iterator it = group->layout.begin() ; it != group->layout.end() ; ) {
-        
+
 	    Inkscape::XML::Node *line_tspan = sp_repr_new("svg:tspan");
         line_tspan->setAttribute("sodipodi:role", "line");
 
@@ -533,9 +558,11 @@ void SPFlowtext::convert_to_text()
             if (set_y)
                 sp_repr_set_svg_double(span_tspan, "y", anchor_point[NR::Y]);
 
-            SPObject *source_obj;
+            SPObject *source_obj = 0;
+            void *rawptr = 0;
             Glib::ustring::iterator span_text_start_iter;
-            group->layout.getSourceOfCharacter(it, (void**)&source_obj, &span_text_start_iter);
+            group->layout.getSourceOfCharacter(it, &rawptr, &span_text_start_iter);
+            source_obj = SP_OBJECT (rawptr);
             gchar *style_text = sp_style_write_difference((SP_IS_STRING(source_obj) ? source_obj->parent : source_obj)->style, group->style);
             if (style_text && *style_text) {
                 span_tspan->setAttribute("style", style_text);
@@ -544,9 +571,11 @@ void SPFlowtext::convert_to_text()
 
             if (SP_IS_STRING(source_obj)) {
                 Glib::ustring *string = &SP_STRING(source_obj)->string;
-                SPObject *span_end_obj;
+                SPObject *span_end_obj = 0;
+                void *rawptr = 0;
                 Glib::ustring::iterator span_text_end_iter;
-                group->layout.getSourceOfCharacter(it_span_end, (void**)&span_end_obj, &span_text_end_iter);
+                group->layout.getSourceOfCharacter(it_span_end, &rawptr, &span_text_end_iter);
+                span_end_obj = SP_OBJECT(rawptr);
                 if (span_end_obj != source_obj) {
                     if (it_span_end == group->layout.end()) {
                         span_text_end_iter = span_text_start_iter;
@@ -562,29 +591,29 @@ void SPFlowtext::convert_to_text()
                         new_string += *span_text_start_iter++;    // grr. no substr() with iterators
                     Inkscape::XML::Node *new_text = sp_repr_new_text(new_string.c_str());
                     span_tspan->appendChild(new_text);
-                    sp_repr_unref(new_text);
+                    Inkscape::GC::release(new_text);
                 }
             }
             it = it_span_end;
 
             line_tspan->appendChild(span_tspan);
-	        sp_repr_unref(span_tspan);
+	        Inkscape::GC::release(span_tspan);
         }
         repr->appendChild(line_tspan);
-	    sp_repr_unref(line_tspan);
+	    Inkscape::GC::release(line_tspan);
     }
 
     Inkscape::XML::Node *parent = SP_OBJECT_REPR(item)->parent();
     parent->appendChild(repr);
-    SPItem *new_item = (SPItem *) SP_DT_DOCUMENT(desktop)->getObjectByRepr(repr);
+    SPItem *new_item = (SPItem *) sp_desktop_document(desktop)->getObjectByRepr(repr);
     sp_item_write_transform(new_item, repr, item->transform);
     SP_OBJECT(new_item)->updateRepr();
 
-    sp_repr_unref(repr);
+    Inkscape::GC::release(repr);
     selection->set(new_item);
     item->deleteObject();
 
-    sp_document_done(SP_DT_DOCUMENT(desktop));
+    sp_document_done(sp_desktop_document(desktop));
 }
 
 SPItem *SPFlowtext::get_frame(SPItem *after)
@@ -635,10 +664,10 @@ bool SPFlowtext::has_internal_frame()
 
 SPItem *create_flowtext_with_internal_frame (SPDesktop *desktop, NR::Point p0, NR::Point p1)
 {
-    SPDocument *doc = SP_DT_DOCUMENT (desktop);
+    SPDocument *doc = sp_desktop_document (desktop);
 
     Inkscape::XML::Node *root_repr = sp_repr_new("svg:flowRoot");
-    sp_repr_set_attr(root_repr, "xml:space", "preserve"); // we preserve spaces in the text objects we create
+    root_repr->setAttribute("xml:space", "preserve"); // we preserve spaces in the text objects we create
     SPItem *ft_item = SP_ITEM(desktop->currentLayer()->appendChildRepr(root_repr));
     SPObject *root_object = doc->getObjectByRepr(root_repr);
     g_assert(SP_IS_FLOWTEXT(root_object));
@@ -675,10 +704,10 @@ SPItem *create_flowtext_with_internal_frame (SPDesktop *desktop, NR::Point p0, N
     Inkscape::XML::Node *text = sp_repr_new_text("");
     para_repr->appendChild(text);
 
-    sp_repr_unref(root_repr);
-    sp_repr_unref(region_repr);
-    sp_repr_unref(para_repr);
-    sp_repr_unref(rect_repr);
+    Inkscape::GC::release(root_repr);
+    Inkscape::GC::release(region_repr);
+    Inkscape::GC::release(para_repr);
+    Inkscape::GC::release(rect_repr);
 
     return ft_item;
 }
