@@ -1,14 +1,9 @@
-#include "sp-item-update-cns.h"
-#include <vector>
 
-#include <approx-equal.h>
-#include <libnr/nr-point.h>
-#include <remove-last.h>
 #include "satisfied-guide-cns.h"
-#include "sp-guide-attachment.h"
 #include "sp-guide-constraint.h"
 #include <sp-guide.h>
 #include <sp-item.h>
+using std::find;
 using std::vector;
 
 void sp_item_update_cns(SPItem &item, SPDesktop const &desktop)
@@ -42,9 +37,9 @@ void sp_item_update_cns(SPItem &item, SPDesktop const &desktop)
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
