@@ -24,12 +24,8 @@
 # include "config.h"
 #endif
 
-#include <math.h>
-#include <string.h>
 
-#include <algorithm>
-
-#include "macros.h"
+#include "sp-item.h"
 #include "svg/svg.h"
 #include "print.h"
 #include "display/nr-arena.h"
@@ -37,38 +33,30 @@
 #include "attributes.h"
 #include "document.h"
 #include "uri.h"
-#include "uri-references.h"
 
-#include "selection.h"
 #include "style.h"
 #include <glibmm/i18n.h>
 #include "sp-root.h"
-#include "sp-anchor.h"
 #include "sp-clippath.h"
 #include "sp-mask.h"
 #include "sp-rect.h"
 #include "sp-use.h"
 #include "sp-text.h"
-#include "sp-item.h"
 #include "sp-item-rm-unsatisfied-cns.h"
 #include "sp-pattern.h"
-#include "sp-gradient.h"
+#include "sp-switch.h"
 #include "gradient-chemistry.h"
 #include "prefs-utils.h"
 #include "conn-avoid-ref.h"
+#include "conditions.h"
 
-#include "libnr/nr-matrix.h"
 #include "libnr/nr-matrix-div.h"
 #include "libnr/nr-matrix-fns.h"
-#include "libnr/nr-matrix-ops.h"
 #include "libnr/nr-matrix-scale-ops.h"
 #include "libnr/nr-matrix-translate-ops.h"
 #include "libnr/nr-scale-translate-ops.h"
-#include "libnr/nr-translate-matrix-ops.h"
 #include "libnr/nr-translate-scale-ops.h"
-#include "libnr/nr-rect.h"
 #include "algorithms/find-last-if.h"
-#include "util/list.h"
 #include "util/reverse-list.h"
 
 #include "xml/repr.h"
@@ -144,31 +132,39 @@ sp_item_class_init(SPItemClass *klass)
 static void
 sp_item_init(SPItem *item)
 {
-    SPObject *object = SP_OBJECT(item);
+    item->init();
+}
 
-    item->sensitive = TRUE;
+void SPItem::init() {
+    this->sensitive = TRUE;
 
-    item->transform = NR::identity();
+    this->transform_center_x = 0;
+    this->transform_center_y = 0;
 
-    item->display = NULL;
+    this->_is_evaluated = true;
+    this->_evaluated_status = StatusUnknown;
 
-    item->clip_ref = new SPClipPathReference(SP_OBJECT(item));
+    this->transform = NR::identity();
+
+    this->display = NULL;
+
+    this->clip_ref = new SPClipPathReference(this);
 		{
-			sigc::signal<void, SPObject *, SPObject *> cs1=item->clip_ref->changedSignal();
-			sigc::slot2<void,SPObject*, SPObject *> sl1=sigc::bind(sigc::ptr_fun(clip_ref_changed), item);
+			sigc::signal<void, SPObject *, SPObject *> cs1=this->clip_ref->changedSignal();
+			sigc::slot2<void,SPObject*, SPObject *> sl1=sigc::bind(sigc::ptr_fun(clip_ref_changed), this);
 			cs1.connect(sl1);
 		}
-		
-    item->mask_ref = new SPMaskReference(SP_OBJECT(item));
-		sigc::signal<void, SPObject *, SPObject *> cs2=item->mask_ref->changedSignal();
-		sigc::slot2<void,SPObject*, SPObject *> sl2=sigc::bind(sigc::ptr_fun(mask_ref_changed), item);
+
+    this->mask_ref = new SPMaskReference(this);
+		sigc::signal<void, SPObject *, SPObject *> cs2=this->mask_ref->changedSignal();
+		sigc::slot2<void,SPObject*, SPObject *> sl2=sigc::bind(sigc::ptr_fun(mask_ref_changed), this);
     cs2.connect(sl2);
 
-    if (!object->style) object->style = sp_style_new_from_object(SP_OBJECT(item));
+    if (!this->style) this->style = sp_style_new_from_object(this);
 
-    item->avoidRef = new SPAvoidRef(item);
-    
-    new (&item->_transformed_signal) sigc::signal<void, NR::Matrix const *, SPItem *>();
+    this->avoidRef = new SPAvoidRef(this);
+
+    new (&this->_transformed_signal) sigc::signal<void, NR::Matrix const *, SPItem *>();
 }
 
 bool SPItem::isVisibleAndUnlocked() const {
@@ -188,12 +184,14 @@ bool SPItem::isLocked() const {
 }
 
 void SPItem::setLocked(bool locked) {
-    sp_repr_set_attr(SP_OBJECT_REPR(this), "sodipodi:insensitive",
+    SP_OBJECT_REPR(this)->setAttribute("sodipodi:insensitive",
                      ( locked ? "1" : NULL ));
     updateRepr();
 }
 
 bool SPItem::isHidden() const {
+    if (!isEvaluated())
+        return true;
     return style->display.computed == SP_CSS_DISPLAY_NONE;
 }
 
@@ -206,6 +204,8 @@ void SPItem::setHidden(bool hide) {
 }
 
 bool SPItem::isHidden(unsigned display_key) const {
+    if (!isEvaluated())
+        return true;
     for ( SPItemView *view(display) ; view ; view = view->next ) {
         if ( view->key == display_key ) {
             g_assert(view->arenaitem != NULL);
@@ -222,7 +222,35 @@ bool SPItem::isHidden(unsigned display_key) const {
     return true;
 }
 
-/** 
+void SPItem::setEvaluated(bool evaluated) {
+    _is_evaluated = evaluated;
+    _evaluated_status = StatusSet;
+}
+
+void SPItem::resetEvaluated() {
+    if ( StatusCalculated == _evaluated_status ) {
+        _evaluated_status = StatusUnknown;
+        bool oldValue = _is_evaluated;
+        if ( oldValue != isEvaluated() ) {
+            requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_STYLE_MODIFIED_FLAG);
+        }
+    } if ( StatusSet == _evaluated_status ) {
+        SPObject const *const parent = SP_OBJECT_PARENT(this);
+        if (SP_IS_SWITCH(parent)) {
+            SP_SWITCH(parent)->resetChildEvaluated();
+        }
+    }
+}
+
+bool SPItem::isEvaluated() const {
+    if ( StatusUnknown == _evaluated_status ) {
+        _is_evaluated = sp_item_evaluate(this);
+        _evaluated_status = StatusCalculated;
+    }
+    return _is_evaluated;
+}
+
+/**
  * Returns something suitable for the `Hide' checkbox in the Object Properties dialog box.
  *  Corresponds to setExplicitlyHidden.
  */
@@ -233,9 +261,9 @@ SPItem::isExplicitlyHidden() const
 	    && this->style->display.value == SP_CSS_DISPLAY_NONE);
 }
 
-/** 
- * Sets the display CSS property to `hidden' if \a val is true, 
- * otherwise makes it unset 
+/**
+ * Sets the display CSS property to `hidden' if \a val is true,
+ * otherwise makes it unset
  */
 void
 SPItem::setExplicitlyHidden(bool const val) {
@@ -245,6 +273,42 @@ SPItem::setExplicitlyHidden(bool const val) {
     this->updateRepr();
 }
 
+/**
+ * Sets the transform_center_x and transform_center_y properties to retain the rotation centre
+ */
+void
+SPItem::setCenter(NR::Point object_centre) {
+    NR::Rect bbox = invokeBbox(sp_item_i2d_affine(this));
+    if (!bbox.isEmpty()) {
+        transform_center_x = object_centre[NR::X] - bbox.midpoint()[NR::X];
+        if (fabs(transform_center_x) < 1e-5) // rounding error
+            transform_center_x = 0;
+        transform_center_y = object_centre[NR::Y] - bbox.midpoint()[NR::Y];
+        if (fabs(transform_center_y) < 1e-5) // rounding error
+            transform_center_y = 0;
+    }
+}
+
+void
+SPItem::unsetCenter() {
+    transform_center_x = 0;
+    transform_center_y = 0;
+}
+
+bool SPItem::isCenterSet() {
+    return (transform_center_x != 0 || transform_center_y != 0);
+}
+
+NR::Point SPItem::getCenter() {
+    NR::Rect bbox = invokeBbox(sp_item_i2d_affine(this));
+    if (!bbox.isEmpty()) {
+        return bbox.midpoint() + NR::Point (this->transform_center_x, this->transform_center_y);
+    } else {
+        return NR::Point (0, 0); // something's wrong!
+    }
+}
+
+
 namespace {
 
 bool is_item(SPObject const &object) {
@@ -261,7 +325,7 @@ void SPItem::raiseToTop() {
     );
     if (topmost) {
         Inkscape::XML::Node *repr=SP_OBJECT_REPR(this);
-        sp_repr_change_order(sp_repr_parent(repr), repr, SP_OBJECT_REPR(topmost));
+        sp_repr_parent(repr)->changeOrder(repr, SP_OBJECT_REPR(topmost));
     }
 }
 
@@ -272,7 +336,7 @@ void SPItem::raiseOne() {
     if (next_higher) {
         Inkscape::XML::Node *repr=SP_OBJECT_REPR(this);
         Inkscape::XML::Node *ref=SP_OBJECT_REPR(next_higher);
-        sp_repr_change_order(sp_repr_parent(repr), repr, ref);
+        sp_repr_parent(repr)->changeOrder(repr, ref);
     }
 }
 
@@ -291,7 +355,7 @@ void SPItem::lowerOne() {
         ++next_lower;
         Inkscape::XML::Node *repr=SP_OBJECT_REPR(this);
         Inkscape::XML::Node *ref=( next_lower ? SP_OBJECT_REPR(&*next_lower) : NULL );
-        sp_repr_change_order(sp_repr_parent(repr), repr, ref);
+        sp_repr_parent(repr)->changeOrder(repr, ref);
     }
 }
 
@@ -311,7 +375,7 @@ void SPItem::lowerToBottom() {
         ++bottom;
         Inkscape::XML::Node *repr=SP_OBJECT_REPR(this);
         Inkscape::XML::Node *ref=( bottom ? SP_OBJECT_REPR(&*bottom) : NULL );
-        sp_repr_change_order(sp_repr_parent(repr), repr, ref);
+        sp_repr_parent(repr)->changeOrder(repr, ref);
     }
 }
 
@@ -324,6 +388,8 @@ sp_item_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr)
     sp_object_read_attr(object, "mask");
     sp_object_read_attr(object, "sodipodi:insensitive");
     sp_object_read_attr(object, "sodipodi:nonprintable");
+    sp_object_read_attr(object, "inkscape:transform-center-x");
+    sp_object_read_attr(object, "inkscape:transform-center-y");
     sp_object_read_attr(object, "inkscape:connector-avoid");
 
     if (((SPObjectClass *) (parent_class))->build) {
@@ -425,6 +491,27 @@ sp_item_set(SPObject *object, unsigned key, gchar const *value)
         case SP_ATTR_CONNECTOR_AVOID:
             item->avoidRef->setAvoid(value);
             break;
+        case SP_ATTR_TRANSFORM_CENTER_X:
+            if (value) {
+                item->transform_center_x = g_strtod(value, NULL);
+            } else {
+                item->transform_center_x = 0;
+            }
+            break;
+        case SP_ATTR_TRANSFORM_CENTER_Y:
+            if (value) {
+                item->transform_center_y = g_strtod(value, NULL);
+            } else {
+                item->transform_center_y = 0;
+            }
+            break;
+        case SP_PROP_SYSTEM_LANGUAGE:
+        case SP_PROP_REQUIRED_FEATURES:
+        case SP_PROP_REQUIRED_EXTENSIONS:
+            {
+                item->resetEvaluated();
+                // pass to default handler
+            }
         default:
             if (SP_ATTRIBUTE_IS_CSS(key)) {
                 sp_style_read_from_object(object->style, object);
@@ -462,6 +549,7 @@ clip_ref_changed(SPObject *old_clip, SPObject *clip, SPItem *item)
             nr_arena_item_set_clip(v->arenaitem, ai);
             nr_arena_item_unref(ai);
             sp_clippath_set_bbox(SP_CLIPPATH(clip), NR_ARENA_ITEM_GET_KEY(v->arenaitem), &bbox);
+            SP_OBJECT(clip)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
         }
     }
 }
@@ -489,6 +577,7 @@ mask_ref_changed(SPObject *old_mask, SPObject *mask, SPItem *item)
             nr_arena_item_set_mask(v->arenaitem, ai);
             nr_arena_item_unref(ai);
             sp_mask_set_bbox(SP_MASK(mask), NR_ARENA_ITEM_GET_KEY(v->arenaitem), &bbox);
+            SP_OBJECT(mask)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
         }
     }
 }
@@ -545,29 +634,29 @@ sp_item_write(SPObject *const object, Inkscape::XML::Node *repr, guint flags)
 
     gchar c[256];
     if (sp_svg_transform_write(c, 256, item->transform)) {
-        sp_repr_set_attr(repr, "transform", c);
+        repr->setAttribute("transform", c);
     } else {
-        sp_repr_set_attr(repr, "transform", NULL);
+        repr->setAttribute("transform", NULL);
     }
 
     SPObject const *const parent = SP_OBJECT_PARENT(object);
-    /** \todo Can someone please document why this is conditional on having 
-     * a parent? The only parentless thing I can think of is the top-level 
-     * <svg> element (SPRoot). SPRoot is derived from SPGroup, and can have 
-     * style.  I haven't looked at callers. 
+    /** \todo Can someone please document why this is conditional on having
+     * a parent? The only parentless thing I can think of is the top-level
+     * <svg> element (SPRoot). SPRoot is derived from SPGroup, and can have
+     * style.  I haven't looked at callers.
      */
     if (parent) {
         SPStyle const *const obj_style = SP_OBJECT_STYLE(object);
         if (obj_style) {
             gchar *s = sp_style_write_string(obj_style, SP_STYLE_FLAG_IFSET);
-            sp_repr_set_attr(repr, "style", ( *s ? s : NULL ));
+            repr->setAttribute("style", ( *s ? s : NULL ));
             g_free(s);
         } else {
-            /** \todo I'm not sure what to do in this case.  Bug #1165868 
-             * suggests that it can arise, but the submitter doesn't know 
+            /** \todo I'm not sure what to do in this case.  Bug #1165868
+             * suggests that it can arise, but the submitter doesn't know
              * how to do so reliably.  The main two options are either
-             * leave repr's style attribute unchanged, or explicitly clear it.  
-             * Must also consider what to do with property attributes for 
+             * leave repr's style attribute unchanged, or explicitly clear it.
+             * Must also consider what to do with property attributes for
              * the element; see below.
              */
             char const *style_str = repr->attribute("style");
@@ -577,21 +666,40 @@ sp_item_write(SPObject *const object, Inkscape::XML::Node *repr, guint flags)
             g_warning("Item's style is NULL; repr style attribute is %s", style_str);
         }
 
-        /** \note We treat object->style as authoritative.  Its effects have 
-         * been written to the style attribute above; any properties that are 
-         * unset we take to be deliberately unset (e.g. so that clones can 
+        /** \note We treat object->style as authoritative.  Its effects have
+         * been written to the style attribute above; any properties that are
+         * unset we take to be deliberately unset (e.g. so that clones can
          * override the property).
          *
-         * Note that the below has an undesirable consequence of changing the 
-         * appearance on renderers that lack CSS support (e.g. SVG tiny); 
-         * possibly we should write property attributes instead of a style 
+         * Note that the below has an undesirable consequence of changing the
+         * appearance on renderers that lack CSS support (e.g. SVG tiny);
+         * possibly we should write property attributes instead of a style
          * attribute.
          */
         sp_style_unset_property_attrs (object);
     }
 
     if (flags & SP_OBJECT_WRITE_EXT) {
-        sp_repr_set_attr(repr, "sodipodi:insensitive", ( item->sensitive ? NULL : "true" ));
+        repr->setAttribute("sodipodi:insensitive", ( item->sensitive ? NULL : "true" ));
+        if (item->transform_center_x != 0)
+            sp_repr_set_svg_double (repr, "inkscape:transform-center-x", item->transform_center_x);
+        else
+            repr->setAttribute ("inkscape:transform-center-x", NULL);
+        if (item->transform_center_y != 0)
+            sp_repr_set_svg_double (repr, "inkscape:transform-center-y", item->transform_center_y);
+        else
+            repr->setAttribute ("inkscape:transform-center-y", NULL);
+    }
+
+    if (item->clip_ref->getObject()) {
+        const gchar *value = g_strdup_printf ("url(%s)", item->clip_ref->getURI()->toString());
+        repr->setAttribute ("clip-path", value);
+        g_free ((void *) value);
+    }
+    if (item->mask_ref->getObject()) {
+        const gchar *value = g_strdup_printf ("url(%s)", item->mask_ref->getURI()->toString());
+        repr->setAttribute ("mask", value);
+        g_free ((void *) value);
     }
 
     if (((SPObjectClass *) (parent_class))->write) {
@@ -601,12 +709,23 @@ sp_item_write(SPObject *const object, Inkscape::XML::Node *repr, guint flags)
     return repr;
 }
 
+NR::Rect SPItem::invokeBbox(NR::Matrix const &transform) const
+{
+    NRRect r;
+    sp_item_invoke_bbox_full(this, &r, transform, 0, TRUE);
+    return NR::Rect(r);
+}
+
 void
 sp_item_invoke_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const clear)
 {
     sp_item_invoke_bbox_full(item, bbox, transform, 0, clear);
 }
 
+/** Calls \a item's subclass' bounding box method; clips it by the bbox of clippath, if any; and
+ * unions the resulting bbox with \a bbox. If \a clear is true, empties \a bbox first. Passes the
+ * transform and the flags to the actual bbox methods. Note that many of subclasses (e.g. groups,
+ * clones), in turn, call this function in their bbox methods. */
 void
 sp_item_invoke_bbox_full(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const flags, unsigned const clear)
 {
@@ -619,8 +738,25 @@ sp_item_invoke_bbox_full(SPItem const *item, NRRect *bbox, NR::Matrix const &tra
         bbox->x1 = bbox->y1 = -1e18;
     }
 
+    NRRect this_bbox;
+    this_bbox.x0 = this_bbox.y0 = 1e18;
+    this_bbox.x1 = this_bbox.y1 = -1e18;
+
+    // call the subclass method
     if (((SPItemClass *) G_OBJECT_GET_CLASS(item))->bbox) {
-        ((SPItemClass *) G_OBJECT_GET_CLASS(item))->bbox(item, bbox, transform, flags);
+        ((SPItemClass *) G_OBJECT_GET_CLASS(item))->bbox(item, &this_bbox, transform, flags);
+    }
+
+    // crop the bbox by clip path, if any
+    if (item->clip_ref->getObject()) {
+        NRRect b;
+        sp_clippath_get_bbox(SP_CLIPPATH(item->clip_ref->getObject()), &b, transform, flags);
+        nr_rect_d_intersect (&this_bbox, &this_bbox, &b);
+    }
+
+    // if non-empty (with some tolerance - ?) union this_bbox with the bbox we've got passed
+    if ( fabs(this_bbox.x1-this_bbox.x0) > -0.00001 && fabs(this_bbox.y1-this_bbox.y0) > -0.00001 ) {
+        nr_rect_d_union (bbox, bbox, &this_bbox);
     }
 }
 
@@ -662,20 +798,18 @@ sp_item_bbox_desktop(SPItem *item, NRRect *bbox)
 NR::Rect sp_item_bbox_desktop(SPItem *item)
 {
     NRRect ret;
-    sp_item_bbox_desktop(item, &ret);
+    sp_item_invoke_bbox(item, &ret, sp_item_i2d_affine(item), TRUE);
     return NR::Rect(ret);
 }
 
 static void sp_item_private_snappoints(SPItem const *item, SnapPointsIter p)
 {
-    NRRect bbox;
-    sp_item_invoke_bbox(item, &bbox, sp_item_i2d_affine(item), TRUE);
-    NR::Rect const bbox2(bbox);
+    NR::Rect const bbox = item->invokeBbox(sp_item_i2d_affine(item));
     /* Just a pair of opposite corners of the bounding box suffices given that we don't yet
        support angled guide lines. */
-    
-    *p = bbox2.min();
-    *p = bbox2.max();
+
+    *p = bbox.min();
+    *p = bbox.max();
 }
 
 void sp_item_snappoints(SPItem const *item, SnapPointsIter p)
@@ -725,7 +859,18 @@ sp_item_description(SPItem *item)
     g_assert(SP_IS_ITEM(item));
 
     if (((SPItemClass *) G_OBJECT_GET_CLASS(item))->description) {
-        return ((SPItemClass *) G_OBJECT_GET_CLASS(item))->description(item);
+        gchar *s = ((SPItemClass *) G_OBJECT_GET_CLASS(item))->description(item);
+        if (s && item->clip_ref->getObject()) {
+            gchar *snew = g_strdup_printf ("%s; <i>clipped</i>", s);
+            g_free (s);
+            s = snew;
+        }
+        if (s && item->mask_ref->getObject()) {
+            gchar *snew = g_strdup_printf ("%s; <i>masked</i>", s);
+            g_free (s);
+            s = snew;
+        }
+        return s;
     }
 
     g_assert_not_reached();
@@ -768,18 +913,42 @@ sp_item_invoke_show(SPItem *item, NRArena *arena, unsigned key, unsigned flags)
         nr_arena_item_set_visible(ai, !item->isHidden());
         nr_arena_item_set_sensitive(ai, item->sensitive);
         if (item->clip_ref->getObject()) {
-            NRArenaItem *ac;
-            if (!item->display->arenaitem->key) NR_ARENA_ITEM_SET_KEY(item->display->arenaitem, sp_item_display_key_new(3));
-            ac = sp_clippath_show(item->clip_ref->getObject(), arena, NR_ARENA_ITEM_GET_KEY(item->display->arenaitem));
+            SPClipPath *cp = item->clip_ref->getObject();
+
+            if (!item->display->arenaitem->key) {
+                NR_ARENA_ITEM_SET_KEY(item->display->arenaitem, sp_item_display_key_new(3));
+            }
+            int clip_key = NR_ARENA_ITEM_GET_KEY(item->display->arenaitem);
+
+            // Show and set clip
+            NRArenaItem *ac = sp_clippath_show(cp, arena, clip_key);
             nr_arena_item_set_clip(ai, ac);
             nr_arena_item_unref(ac);
+
+            // Update bbox, in case the clip uses bbox units
+            NRRect bbox;
+            sp_item_invoke_bbox(item, &bbox, NR::identity(), TRUE);
+            sp_clippath_set_bbox(SP_CLIPPATH(cp), clip_key, &bbox);
+            SP_OBJECT(cp)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
         }
         if (item->mask_ref->getObject()) {
-            NRArenaItem *ac;
-            if (!item->display->arenaitem->key) NR_ARENA_ITEM_SET_KEY(item->display->arenaitem, sp_item_display_key_new(3));
-            ac = sp_mask_show(item->mask_ref->getObject(), arena, NR_ARENA_ITEM_GET_KEY(item->display->arenaitem));
+            SPMask *mask = item->mask_ref->getObject();
+
+            if (!item->display->arenaitem->key) {
+                NR_ARENA_ITEM_SET_KEY(item->display->arenaitem, sp_item_display_key_new(3));
+            }
+            int mask_key = NR_ARENA_ITEM_GET_KEY(item->display->arenaitem);
+
+            // Show and set mask
+            NRArenaItem *ac = sp_mask_show(mask, arena, mask_key);
             nr_arena_item_set_mask(ai, ac);
             nr_arena_item_unref(ac);
+
+            // Update bbox, in case the mask uses bbox units
+            NRRect bbox;
+            sp_item_invoke_bbox(item, &bbox, NR::identity(), TRUE);
+            sp_mask_set_bbox(SP_MASK(mask), mask_key, &bbox);
+            SP_OBJECT(mask)->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
         }
         NR_ARENA_ITEM_SET_DATA(ai, item);
     }
@@ -832,7 +1001,7 @@ sp_item_adjust_pattern (SPItem *item, NR::Matrix const &postmul, bool set)
 {
     SPStyle *style = SP_OBJECT_STYLE (item);
 
-    if (style && (style->fill.type == SP_PAINT_TYPE_PAINTSERVER)) { 
+    if (style && (style->fill.type == SP_PAINT_TYPE_PAINTSERVER)) {
         SPObject *server = SP_OBJECT_STYLE_FILL_SERVER (item);
         if (SP_IS_PATTERN (server)) {
             SPPattern *pattern = sp_pattern_clone_if_necessary (item, SP_PATTERN (server), "fill");
@@ -840,7 +1009,7 @@ sp_item_adjust_pattern (SPItem *item, NR::Matrix const &postmul, bool set)
         }
     }
 
-    if (style && (style->stroke.type == SP_PAINT_TYPE_PAINTSERVER)) { 
+    if (style && (style->stroke.type == SP_PAINT_TYPE_PAINTSERVER)) {
         SPObject *server = SP_OBJECT_STYLE_STROKE_SERVER (item);
         if (SP_IS_PATTERN (server)) {
             SPPattern *pattern = sp_pattern_clone_if_necessary (item, SP_PATTERN (server), "stroke");
@@ -859,13 +1028,13 @@ sp_item_adjust_gradient (SPItem *item, NR::Matrix const &postmul, bool set)
         SPObject *server = SP_OBJECT_STYLE_FILL_SERVER(item);
         if (SP_IS_GRADIENT (server)) {
 
-            /** 
-             * \note Bbox units for a gradient are generally a bad idea because 
-             * with them, you cannot preserve the relative position of the 
-             * object and its gradient after rotation or skew. So now we 
-             * convert them to userspace units which are easy to keep in sync 
-             * just by adding the object's transform to gradientTransform.  
-             * \todo FIXME: convert back to bbox units after transforming with 
+            /**
+             * \note Bbox units for a gradient are generally a bad idea because
+             * with them, you cannot preserve the relative position of the
+             * object and its gradient after rotation or skew. So now we
+             * convert them to userspace units which are easy to keep in sync
+             * just by adding the object's transform to gradientTransform.
+             * \todo FIXME: convert back to bbox units after transforming with
              * the item, so as to preserve the original units.
              */
             SPGradient *gradient = sp_gradient_convert_to_userspace (SP_GRADIENT (server), item, "fill");
@@ -891,6 +1060,7 @@ sp_item_adjust_stroke (SPItem *item, gdouble ex)
     if (style && style->stroke.type != SP_PAINT_TYPE_NONE && !NR_DF_TEST_CLOSE (ex, 1.0, NR_EPSILON)) {
 
         style->stroke_width.computed *= ex;
+        style->stroke_width.set = TRUE;
 
         if (style->stroke_dash.n_dash != 0) {
             int i;
@@ -971,7 +1141,7 @@ sp_item_adjust_paint_recursive (SPItem *item, NR::Matrix advertized_transform, N
 
     if (is_pattern)
         sp_item_adjust_pattern (item, paint_delta);
-    else 
+    else
         sp_item_adjust_gradient (item, paint_delta);
 
 // Within text, we do not fork gradients, and so must not recurse to avoid double compensation
@@ -988,7 +1158,7 @@ sp_item_adjust_paint_recursive (SPItem *item, NR::Matrix advertized_transform, N
 }
 
 /**
- * A temporary wrapper for the next function accepting the NRMatrix 
+ * A temporary wrapper for the next function accepting the NRMatrix
  * instead of NR::Matrix
  */
 void
@@ -996,16 +1166,16 @@ sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NRMatrix const
 {
     if (transform == NULL)
         sp_item_write_transform(item, repr, NR::identity(), adv);
-    else 
+    else
         sp_item_write_transform(item, repr, NR::Matrix (transform), adv);
 }
 
 /**
- * Set a new transform on an object. 
+ * Set a new transform on an object.
  *
- * Compensate for stroke scaling and gradient/pattern fill transform, if 
+ * Compensate for stroke scaling and gradient/pattern fill transform, if
  * necessary. Call the object's set_transform method if transforms are
- * stored optimized. Send _transformed_signal. Invoke _write method so that 
+ * stored optimized. Send _transformed_signal. Invoke _write method so that
  * the repr is updated with the new transform.
  */
 void
@@ -1034,10 +1204,10 @@ sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NR::Matrix cons
         sp_item_adjust_rects_recursive(item, advertized_transform);
     }
 
-    // recursively compensate pattern fill if it's not to be transformed 
+    // recursively compensate pattern fill if it's not to be transformed
     if (prefs_get_int_attribute("options.transform", "pattern", 1) == 0) {
         sp_item_adjust_paint_recursive (item, advertized_transform.inverse(), NR::identity(), true);
-    } 
+    }
     /// \todo FIXME: add the same else branch as for gradients below, to convert patterns to userSpaceOnUse as well
     /// recursively compensate gradient fill if it's not to be transformed
     if (prefs_get_int_attribute("options.transform", "gradient", 1) == 0) {
@@ -1048,18 +1218,23 @@ sp_item_write_transform(SPItem *item, Inkscape::XML::Node *repr, NR::Matrix cons
         sp_item_adjust_paint_recursive (item, NR::identity(), NR::identity(), false);
     }
 
-    // run the object's set_transform if transforms are stored optimized
+    // run the object's set_transform if transforms are stored optimized and there's no clippath or mask
     gint preserve = prefs_get_int_attribute("options.preservetransform", "value", 0);
     NR::Matrix transform_attr (transform);
-    if (((SPItemClass *) G_OBJECT_GET_CLASS(item))->set_transform && !preserve) {
+    if (((SPItemClass *) G_OBJECT_GET_CLASS(item))->set_transform 
+                && !preserve && !item->clip_ref->getObject() && !item->mask_ref->getObject()) {
         transform_attr = ((SPItemClass *) G_OBJECT_GET_CLASS(item))->set_transform(item, transform);
     }
     sp_item_set_item_transform(item, transform_attr);
 
+    // Note: updateRepr comes before emitting the transformed signal since
+    // it causes clone SPUse's copy of the original object to brought up to
+    // date with the original.  Otherwise, sp_use_bbox returns incorrect
+    // values if called in code handling the transformed signal.
+    SP_OBJECT(item)->updateRepr();
+
     // send the relative transform with a _transformed_signal
     item->_transformed_signal.emit(&advertized_transform, item);
-
-    SP_OBJECT(item)->updateRepr();
 }
 
 gint
@@ -1207,23 +1382,6 @@ NR::Matrix matrix_from_desktop(NR::Matrix const m, SPItem const *item)
     return ret;
 }
 
-NRMatrix *sp_item_i2d_affine(SPItem const *item, NRMatrix *affine)
-{
-    g_return_val_if_fail(item != NULL, NULL);
-    g_return_val_if_fail(SP_IS_ITEM(item), NULL);
-    g_return_val_if_fail(affine != NULL, NULL);
-
-    NRMatrix const i2doc(sp_item_i2doc_affine(item));
-
-    NRMatrix doc2dt;
-    nr_matrix_set_scale(&doc2dt, 1, -1);
-    doc2dt.c[5] = sp_document_height(SP_OBJECT_DOCUMENT(item));
-
-    nr_matrix_multiply(affine, &i2doc, &doc2dt);
-
-    return affine;
-}
-
 void sp_item_set_i2d_affine(SPItem *item, NR::Matrix const &i2dt)
 {
     g_return_if_fail( item != NULL );
@@ -1243,22 +1401,12 @@ void sp_item_set_i2d_affine(SPItem *item, NR::Matrix const &i2dt)
 
 
 NR::Matrix
-sp_item_dt2i_affine(SPItem const *item, SPDesktop *)
+sp_item_dt2i_affine(SPItem const *item)
 {
     /* fixme: Implement the right way (Lauris) */
     return sp_item_i2d_affine(item).inverse();
 }
 
-NRMatrix *
-sp_item_dt2i_affine(SPItem const *item, SPDesktop *dt, NRMatrix *affine)
-{
-    /* fixme: Implement the right way (Lauris) */
-    NRMatrix i2dt;
-    sp_item_i2d_affine(item, &i2dt);
-    nr_matrix_invert(affine, &i2dt);
-    return affine;
-}
-
 /* Item views */
 
 static SPItemView *
@@ -1300,7 +1448,7 @@ sp_item_view_list_remove(SPItemView *list, SPItemView *view)
 }
 
 /**
- * Return the arenaitem corresponding to the given item in the display 
+ * Return the arenaitem corresponding to the given item in the display
  * with the given key
  */
 NRArenaItem *
