@@ -21,59 +21,35 @@
  *
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
-#include <algorithm>
-#include <string.h>
-
-#include <libnr/nr-rect.h>
-#include <libnr/nr-matrix.h>
 #include <libnr/nr-matrix-fns.h>
-#include <libnr/nr-point-matrix-ops.h>
-#include <libnr/nr-rotate.h>
-//#include <libnrtype/nr-typeface.h>
 #include <libnrtype/FontFactory.h>
 #include <libnrtype/font-instance.h>
 #include <libnrtype/font-style-to-pos.h>
 
-#include <livarot/LivarotDefs.h>
-#include <livarot/Shape.h>
-#include <livarot/Path.h>
-
-#include <glib.h>
-//#include <gtk/gtk.h>
-
 #include <glibmm/i18n.h>
 #include "svg/svg.h"
 #include "svg/stringstream.h"
-#include "display/curve.h"
-#include "display/nr-arena-group.h"
 #include "display/nr-arena-glyphs.h"
 #include "attributes.h"
 #include "document.h"
 #include "desktop-handles.h"
 #include "sp-namedview.h"
-#include "fontsize-expansion.h"
 #include "style.h"
-#include "version.h"
 #include "inkscape.h"
-#include "print.h"
 #include "sp-metrics.h"
 #include "xml/quote.h"
 #include "xml/repr.h"
 #include "mod360.h"
 
-#include "sp-item.h"
-#include "sp-text.h"
+#include "sp-textpath.h"
 #include "sp-tspan.h"
-#include "sp-string.h"
 
 #include "text-editing.h"
 
-//#include "sp-use-reference.h"
-//#include "prefs-utils.h"
-
-
 /*#####################################################
 #  SPTEXT
 #####################################################*/
@@ -287,6 +263,20 @@ sp_text_modified (SPObject *object, guint flags)
     guint cflags = (flags & SP_OBJECT_MODIFIED_CASCADE);
     if (flags & SP_OBJECT_MODIFIED_FLAG) cflags |= SP_OBJECT_PARENT_MODIFIED_FLAG;
 
+    // FIXME: all that we need to do here is nr_arena_glyphs_[group_]set_style, to set the changed
+    // style, but there's no easy way to access the arena glyphs or glyph groups corresponding to a
+    // text object. Therefore we do here the same as in _update, that is, destroy all arena items
+    // and create new ones. This is probably quite wasteful.
+    if (flags & ( SP_OBJECT_STYLE_MODIFIED_FLAG )) {
+        SPText *text = SP_TEXT (object);
+        NRRect paintbox;
+        sp_item_invoke_bbox(text, &paintbox, NR::identity(), TRUE);
+        for (SPItemView* v = text->display; v != NULL; v = v->next) {
+            text->_clearFlow(NR_ARENA_GROUP(v->arenaitem));
+            text->layout.show(NR_ARENA_GROUP(v->arenaitem), &paintbox);
+        }
+    }
+
     /* Create temporary list of children */
     GSList *l = NULL;
     SPObject *child;
@@ -324,8 +314,8 @@ sp_text_write (SPObject *object, Inkscape::XML::Node *repr, guint flags)
             if (crepr) l = g_slist_prepend (l, crepr);
         }
         while (l) {
-            sp_repr_add_child (repr, (Inkscape::XML::Node *) l->data, NULL);
-            sp_repr_unref ((Inkscape::XML::Node *) l->data);
+            repr->addChild((Inkscape::XML::Node *) l->data, NULL);
+            Inkscape::GC::release((Inkscape::XML::Node *) l->data);
             l = g_slist_remove (l, l->data);
         }
     } else {
@@ -359,6 +349,21 @@ static void
 sp_text_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsigned const /*flags*/)
 {
     SP_TEXT(item)->layout.getBoundingBox(bbox, transform);
+
+    // Add stroke width
+    SPStyle* style=SP_OBJECT_STYLE (item);
+    if (style->stroke.type != SP_PAINT_TYPE_NONE) {
+        double const scale = expansion(transform);
+        if ( fabs(style->stroke_width.computed * scale) > 0.01 ) { // sinon c'est 0=oon veut pas de bord
+            double const width = MAX(0.125, style->stroke_width.computed * scale);
+            if ( fabs(bbox->x1 - bbox->x0) > -0.00001 && fabs(bbox->y1 - bbox->y0) > -0.00001 ) {
+                bbox->x0-=0.5*width;
+                bbox->x1+=0.5*width;
+                bbox->y0-=0.5*width;
+                bbox->y1+=0.5*width;
+            }
+        }
+    }
 }
 
 
@@ -404,7 +409,7 @@ sp_text_description(SPItem *item)
         n = g_strdup(_("&lt;no name found&gt;"));
     }
 
-    GString *xs = SP_PX_TO_METRIC_STRING(style->font_size.computed, SP_DT_NAMEDVIEW(SP_ACTIVE_DESKTOP)->getDefaultMetric());
+    GString *xs = SP_PX_TO_METRIC_STRING(style->font_size.computed, sp_desktop_namedview(SP_ACTIVE_DESKTOP)->getDefaultMetric());
 
     char *ret = ( SP_IS_TEXT_TEXTPATH(item)
                   ? g_strdup_printf(_("<b>Text on path</b> (%s, %s)"), n, xs->str)
@@ -483,8 +488,7 @@ sp_text_print (SPItem *item, SPPrintContext *ctx)
     dbox.y0 = 0.0;
     dbox.x1 = sp_document_width (SP_OBJECT_DOCUMENT (item));
     dbox.y1 = sp_document_height (SP_OBJECT_DOCUMENT (item));
-    NRMatrix ctm;
-    sp_item_i2d_affine (item, &ctm);
+    NR::Matrix const ctm = sp_item_i2d_affine(item);
 
     group->layout.print(ctx,&pbox,&dbox,&bbox,ctm);
 }
@@ -580,22 +584,24 @@ void SPText::rebuildLayout()
 }
 
 
-void SPText::_adjustFontsizeRecursive(SPItem *item, double ex)
+void SPText::_adjustFontsizeRecursive(SPItem *item, double ex, bool is_root)
 {
     SPStyle *style = SP_OBJECT_STYLE (item);
 
     if (style && !NR_DF_TEST_CLOSE (ex, 1.0, NR_EPSILON)) {
-        style->font_size.computed *= ex;
-        if (style->text) {
-            style->letter_spacing.computed *= ex;
-            style->word_spacing.computed *= ex;
+        if (!style->font_size.set && is_root) {
+            style->font_size.set = 1;
+            style->font_size.type = SP_FONT_SIZE_LENGTH;
         }
+        style->font_size.computed *= ex;
+        style->letter_spacing.computed *= ex;
+        style->word_spacing.computed *= ex;
         item->updateRepr();
     }
 
     for (SPObject *o = item->children; o != NULL; o = o->next) {
         if (SP_IS_ITEM(o))
-            _adjustFontsizeRecursive(SP_ITEM(o), ex);
+            _adjustFontsizeRecursive(SP_ITEM(o), ex, false);
     }
 }
 
@@ -645,7 +651,7 @@ void TextTagAttributes::readFrom(Inkscape::XML::Node const *node)
 
 bool TextTagAttributes::readSingleAttribute(unsigned key, gchar const *value)
 {
-    std::vector<SPSVGLength> *attr_vector;
+    std::vector<SVGLength> *attr_vector;
     switch (key) {
         case SP_ATTR_X:      attr_vector = &attributes.x; break;
         case SP_ATTR_Y:      attr_vector = &attributes.y; break;
@@ -655,15 +661,8 @@ bool TextTagAttributes::readSingleAttribute(unsigned key, gchar const *value)
         default: return false;
     }
 
-    GList *list_base = sp_svg_length_list_read(value);     // FIXME: sp_svg_length_list_read() amalgamates repeated separators. This prevents unset values.
-    // simple GList to std::vector<> converter:
-    attr_vector->clear();
-    attr_vector->reserve(g_list_length(list_base));
-    for (GList *list = list_base ; list ; list = list->next) {
-        attr_vector->push_back(*reinterpret_cast<SPSVGLength*>(list->data));
-        g_free(list->data);
-    }
-    g_list_free(list_base);
+    // FIXME: sp_svg_length_list_read() amalgamates repeated separators. This prevents unset values.
+    *attr_vector = sp_svg_length_list_read(value);
     return true;
 }
 
@@ -676,7 +675,7 @@ void TextTagAttributes::writeTo(Inkscape::XML::Node *node) const
     writeSingleAttribute(node, "rotate", attributes.rotate);
 }
 
-void TextTagAttributes::writeSingleAttribute(Inkscape::XML::Node *node, gchar const *key, std::vector<SPSVGLength> const &attr_vector)
+void TextTagAttributes::writeSingleAttribute(Inkscape::XML::Node *node, gchar const *key, std::vector<SVGLength> const &attr_vector)
 {
     if (attr_vector.empty())
         node->setAttribute(key, NULL);
@@ -685,7 +684,7 @@ void TextTagAttributes::writeSingleAttribute(Inkscape::XML::Node *node, gchar co
         gchar single_value_string[32];
 
         // FIXME: this has no concept of unset values because sp_svg_length_list_read() can't read them back in
-        for (std::vector<SPSVGLength>::const_iterator it = attr_vector.begin() ; it != attr_vector.end() ; it++) {
+        for (std::vector<SVGLength>::const_iterator it = attr_vector.begin() ; it != attr_vector.end() ; it++) {
             g_ascii_formatd(single_value_string, sizeof (single_value_string), "%.8g", it->computed);
             if (!string.empty()) string += ' ';
             string += single_value_string;
@@ -723,7 +722,7 @@ void TextTagAttributes::mergeInto(Inkscape::Text::Layout::OptionalTextTagAttrs *
     mergeSingleAttribute(&output->rotate, parent_attrs.rotate, parent_attrs_offset, copy_dxdyrotate ? &attributes.rotate : NULL);
 }
 
-void TextTagAttributes::mergeSingleAttribute(std::vector<SPSVGLength> *output_list, std::vector<SPSVGLength> const &parent_list, unsigned parent_offset, std::vector<SPSVGLength> const *overlay_list)
+void TextTagAttributes::mergeSingleAttribute(std::vector<SVGLength> *output_list, std::vector<SVGLength> const &parent_list, unsigned parent_offset, std::vector<SVGLength> const *overlay_list)
 {
     if (overlay_list == NULL) {
         output_list->resize(std::max(0, (int)parent_list.size() - (int)parent_offset));
@@ -733,7 +732,7 @@ void TextTagAttributes::mergeSingleAttribute(std::vector<SPSVGLength> *output_li
         output_list->reserve(std::max((int)parent_list.size() - (int)parent_offset, (int)overlay_list->size()));
         unsigned overlay_offset = 0;
         while (parent_offset < parent_list.size() || overlay_offset < overlay_list->size()) {
-            SPSVGLength const *this_item;
+            SVGLength const *this_item;
             if (overlay_offset < overlay_list->size()) {
                 this_item = &(*overlay_list)[overlay_offset];
                 overlay_offset++;
@@ -759,7 +758,7 @@ void TextTagAttributes::erase(unsigned start_index, unsigned n)
     eraseSingleAttribute(&attributes.rotate, start_index, n);
 }
 
-void TextTagAttributes::eraseSingleAttribute(std::vector<SPSVGLength> *attr_vector, unsigned start_index, unsigned n)
+void TextTagAttributes::eraseSingleAttribute(std::vector<SVGLength> *attr_vector, unsigned start_index, unsigned n)
 {
     if (attr_vector->size() <= start_index) return;
     if (attr_vector->size() <= start_index + n)
@@ -780,10 +779,10 @@ void TextTagAttributes::insert(unsigned start_index, unsigned n)
     insertSingleAttribute(&attributes.rotate, start_index, n, false);
 }
 
-void TextTagAttributes::insertSingleAttribute(std::vector<SPSVGLength> *attr_vector, unsigned start_index, unsigned n, bool is_xy)
+void TextTagAttributes::insertSingleAttribute(std::vector<SVGLength> *attr_vector, unsigned start_index, unsigned n, bool is_xy)
 {
     if (attr_vector->size() <= start_index) return;
-    SPSVGLength zero_length;
+    SVGLength zero_length;
     zero_length = 0.0;
     attr_vector->insert(attr_vector->begin() + start_index, n, zero_length);
     if (is_xy) {
@@ -805,7 +804,7 @@ void TextTagAttributes::split(unsigned index, TextTagAttributes *second)
     splitSingleAttribute(&attributes.rotate, index, &second->attributes.rotate, true);
 }
 
-void TextTagAttributes::splitSingleAttribute(std::vector<SPSVGLength> *first_vector, unsigned index, std::vector<SPSVGLength> *second_vector, bool trimZeros)
+void TextTagAttributes::splitSingleAttribute(std::vector<SVGLength> *first_vector, unsigned index, std::vector<SVGLength> *second_vector, bool trimZeros)
 {
     second_vector->clear();
     if (first_vector->size() <= index) return;
@@ -813,7 +812,7 @@ void TextTagAttributes::splitSingleAttribute(std::vector<SPSVGLength> *first_vec
     std::copy(first_vector->begin() + index, first_vector->end(), second_vector->begin());
     first_vector->resize(index);
     if (trimZeros)
-        while (!first_vector->empty() && (!first_vector->back().set || first_vector->back().value == 0.0))
+        while (!first_vector->empty() && (!first_vector->back()._set || first_vector->back().value == 0.0))
             first_vector->resize(first_vector->size() - 1);
 }
 
@@ -831,7 +830,7 @@ void TextTagAttributes::join(TextTagAttributes const &first, TextTagAttributes c
     joinSingleAttribute(&attributes.rotate, first.attributes.rotate, second.attributes.rotate, second_index);
 }
 
-void TextTagAttributes::joinSingleAttribute(std::vector<SPSVGLength> *dest_vector, std::vector<SPSVGLength> const &first_vector, std::vector<SPSVGLength> const &second_vector, unsigned second_index)
+void TextTagAttributes::joinSingleAttribute(std::vector<SVGLength> *dest_vector, std::vector<SVGLength> const &first_vector, std::vector<SVGLength> const &second_vector, unsigned second_index)
 {
     if (second_vector.empty())
         *dest_vector = first_vector;
@@ -839,7 +838,7 @@ void TextTagAttributes::joinSingleAttribute(std::vector<SPSVGLength> *dest_vecto
         dest_vector->resize(second_index + second_vector.size());
         if (first_vector.size() < second_index) {
             std::copy(first_vector.begin(), first_vector.end(), dest_vector->begin());
-            SPSVGLength zero_length;
+            SVGLength zero_length;
             zero_length = 0.0;
             std::fill(dest_vector->begin() + first_vector.size(), dest_vector->begin() + second_index, zero_length);
         } else
@@ -850,7 +849,7 @@ void TextTagAttributes::joinSingleAttribute(std::vector<SPSVGLength> *dest_vecto
 
 void TextTagAttributes::transform(NR::Matrix const &matrix, double scale_x, double scale_y, bool extend_zero_length)
 {
-    SPSVGLength zero_length;
+    SVGLength zero_length;
     zero_length = 0.0;
 
     /* edge testcases for this code:
@@ -880,15 +879,15 @@ void TextTagAttributes::transform(NR::Matrix const &matrix, double scale_x, doub
             attributes.y[i] = point[NR::Y];
         }
     }
-    for (std::vector<SPSVGLength>::iterator it = attributes.dx.begin() ; it != attributes.dx.end() ; it++)
+    for (std::vector<SVGLength>::iterator it = attributes.dx.begin() ; it != attributes.dx.end() ; it++)
         *it = it->computed * scale_x;
-    for (std::vector<SPSVGLength>::iterator it = attributes.dy.begin() ; it != attributes.dy.end() ; it++)
+    for (std::vector<SVGLength>::iterator it = attributes.dy.begin() ; it != attributes.dy.end() ; it++)
         *it = it->computed * scale_y;
 }
 
 void TextTagAttributes::addToDxDy(unsigned index, NR::Point const &adjust)
 {
-    SPSVGLength zero_length;
+    SVGLength zero_length;
     zero_length = 0.0;
 
     if (adjust[NR::X] != 0.0) {
@@ -903,11 +902,15 @@ void TextTagAttributes::addToDxDy(unsigned index, NR::Point const &adjust)
 
 void TextTagAttributes::addToRotate(unsigned index, double delta)
 {
-    SPSVGLength zero_length;
+    SVGLength zero_length;
     zero_length = 0.0;
 
-    if (attributes.rotate.size() < index + 1)
-        attributes.rotate.resize(index + 1, zero_length);
+    if (attributes.rotate.size() < index + 1) {
+        if (attributes.rotate.empty())
+            attributes.rotate.resize(index + 1, zero_length);
+        else
+            attributes.rotate.resize(index + 1, attributes.rotate.back());
+    }
     attributes.rotate[index] = mod360(attributes.rotate[index].computed + delta);
 }
 
