@@ -1,7 +1,7 @@
 #define __SP_USE_C__
 
 /*
- * SVG <svg> implementation
+ * SVG <use> implementation
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
@@ -16,25 +16,18 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
-#include <string.h>
-#include <glib-object.h>
-#include <libnr/nr-matrix.h>
 #include <libnr/nr-matrix-ops.h>
 #include <libnr/nr-matrix-fns.h>
 #include "libnr/nr-matrix-translate-ops.h"
 #include <glibmm/i18n.h>
-#include "svg/svg.h"
 #include "display/nr-arena-group.h"
 #include "attributes.h"
 #include "document.h"
 #include "sp-object-repr.h"
 #include "sp-flowregion.h"
 #include "uri.h"
-#include "uri-references.h"
-#include "macros.h"
+#include "print.h"
 #include "xml/repr.h"
-#include "sp-item.h"
-#include "enums.h"
 #include "prefs-utils.h"
 #include "style.h"
 #include "sp-symbol.h"
@@ -42,7 +35,6 @@
 #include "sp-use-reference.h"
 
 /* fixme: */
-#include "desktop-events.h"
 
 static void sp_use_class_init(SPUseClass *classname);
 static void sp_use_init(SPUse *use);
@@ -120,10 +112,10 @@ sp_use_class_init(SPUseClass *classname)
 static void
 sp_use_init(SPUse *use)
 {
-    sp_svg_length_unset(&use->x, SP_SVG_UNIT_NONE, 0.0, 0.0);
-    sp_svg_length_unset(&use->y, SP_SVG_UNIT_NONE, 0.0, 0.0);
-    sp_svg_length_unset(&use->width, SP_SVG_UNIT_PERCENT, 1.0, 1.0);
-    sp_svg_length_unset(&use->height, SP_SVG_UNIT_PERCENT, 1.0, 1.0);
+    use->x.unset();
+    use->y.unset();
+    use->width.unset(SVGLength::PERCENT, 1.0, 1.0);
+    use->height.unset(SVGLength::PERCENT, 1.0, 1.0);
     use->href = NULL;
 
     new (&use->_delete_connection) sigc::connection();
@@ -195,27 +187,19 @@ sp_use_set(SPObject *object, unsigned key, gchar const *value)
 
     switch (key) {
         case SP_ATTR_X:
-            if (!sp_svg_length_read(value, &use->x)) {
-                sp_svg_length_unset(&use->x, SP_SVG_UNIT_NONE, 0.0, 0.0);
-            }
+            use->x.readOrUnset(value);
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_Y:
-            if (!sp_svg_length_read(value, &use->y)) {
-                sp_svg_length_unset(&use->y, SP_SVG_UNIT_NONE, 0.0, 0.0);
-            }
+            use->y.readOrUnset(value);
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_WIDTH:
-            if (!sp_svg_length_read(value, &use->width)) {
-                sp_svg_length_unset(&use->width, SP_SVG_UNIT_PERCENT, 1.0, 1.0);
-            }
+            use->width.readOrUnset(value, SVGLength::PERCENT, 1.0, 1.0);
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
         case SP_ATTR_HEIGHT:
-            if (!sp_svg_length_read(value, &use->height)) {
-                sp_svg_length_unset(&use->height, SP_SVG_UNIT_PERCENT, 1.0, 1.0);
-            }
+            use->height.readOrUnset(value, SVGLength::PERCENT, 1.0, 1.0);
             object->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG);
             break;
 
@@ -271,7 +255,7 @@ sp_use_write(SPObject *object, Inkscape::XML::Node *repr, guint flags)
 
     if (use->ref->getURI()) {
         gchar *uri_string = use->ref->getURI()->toString();
-        sp_repr_set_attr(repr, "xlink:href", uri_string);
+        repr->setAttribute("xlink:href", uri_string);
         g_free(uri_string);
     }
 
@@ -296,11 +280,23 @@ sp_use_bbox(SPItem const *item, NRRect *bbox, NR::Matrix const &transform, unsig
 static void
 sp_use_print(SPItem *item, SPPrintContext *ctx)
 {
+    bool translated = false;
+    NRMatrix tp;
     SPUse *use = SP_USE(item);
 
+    if ((use->x._set && use->x.computed != 0) || (use->y._set && use->y.computed != 0)) {
+        nr_matrix_set_translate(&tp, use->x.computed, use->y.computed);
+        sp_print_bind(ctx, &tp, 1.0);
+        translated = true;
+    }
+
     if (use->child && SP_IS_ITEM(use->child)) {
         sp_item_invoke_print(SP_ITEM(use->child), ctx);
     }
+
+    if (translated) {
+        sp_print_release(ctx);
+    }
 }
 
 static gchar *
@@ -311,7 +307,7 @@ sp_use_description(SPItem *item)
     char *ret;
     if (use->child) {
         static unsigned recursion_depth = 0;
-        if (recursion_depth >= 2) {
+        if (recursion_depth >= 4) {
             /* TRANSLATORS: Used for statusbar description for long <use> chains:
              * "Clone of: Clone of: ... in Layer 1". */
             return g_strdup(_("..."));
@@ -412,8 +408,8 @@ sp_use_get_root_transform(SPUse *use)
         // represent the values of the x and y attributes on the 'use' element." - http://www.w3.org/TR/SVG11/struct.html#UseElement
         if (SP_IS_USE(i_tem)) {
             SPUse *i_use = SP_USE(i_tem);
-            if ((i_use->x.set && i_use->x.computed != 0) || (i_use->y.set && i_use->y.computed != 0)) {
-                t = t * NR::translate(i_use->x.set ? i_use->x.computed : 0, i_use->y.set ? i_use->y.computed : 0);
+            if ((i_use->x._set && i_use->x.computed != 0) || (i_use->y._set && i_use->y.computed != 0)) {
+                t = t * NR::translate(i_use->x._set ? i_use->x.computed : 0, i_use->y._set ? i_use->y.computed : 0);
             }
         }
 
@@ -432,9 +428,9 @@ NR::Matrix
 sp_use_get_parent_transform(SPUse *use)
 {
     NR::Matrix t(NR::identity());
-    if ((use->x.set && use->x.computed != 0) || (use->y.set && use->y.computed != 0)) {
-        t *= NR::translate(use->x.set ? use->x.computed : 0,
-                           use->y.set ? use->y.computed : 0);
+    if ((use->x._set && use->x.computed != 0) || (use->y._set && use->y.computed != 0)) {
+        t *= NR::translate(use->x._set ? use->x.computed : 0,
+                           use->y._set ? use->y.computed : 0);
     }
 
     t *= SP_ITEM(use)->transform;
@@ -571,16 +567,16 @@ sp_use_update(SPObject *object, SPCtx *ctx, unsigned flags)
     flags &= SP_OBJECT_MODIFIED_CASCADE;
 
     /* Set up child viewport */
-    if (use->x.unit == SP_SVG_UNIT_PERCENT) {
+    if (use->x.unit == SVGLength::PERCENT) {
         use->x.computed = use->x.value * (ictx->vp.x1 - ictx->vp.x0);
     }
-    if (use->y.unit == SP_SVG_UNIT_PERCENT) {
+    if (use->y.unit == SVGLength::PERCENT) {
         use->y.computed = use->y.value * (ictx->vp.y1 - ictx->vp.y0);
     }
-    if (use->width.unit == SP_SVG_UNIT_PERCENT) {
+    if (use->width.unit == SVGLength::PERCENT) {
         use->width.computed = use->width.value * (ictx->vp.x1 - ictx->vp.x0);
     }
-    if (use->height.unit == SP_SVG_UNIT_PERCENT) {
+    if (use->height.unit == SVGLength::PERCENT) {
         use->height.computed = use->height.value * (ictx->vp.y1 - ictx->vp.y0);
     }
     cctx.vp.x0 = 0.0;
@@ -662,7 +658,7 @@ sp_use_unlink(SPUse *use)
     }
 
     // Add the duplicate repr just after the existing one.
-    sp_repr_add_child(parent, copy, repr);
+    parent->addChild(copy, repr);
 
     // Retrieve the SPItem of the resulting repr.
     SPObject *unlinked = document->getObjectByRepr(copy);
@@ -677,22 +673,22 @@ sp_use_unlink(SPUse *use)
 
     // Hold onto our SPObject and repr for now.
     sp_object_ref(SP_OBJECT(use), NULL);
-    sp_repr_ref(repr);
+    Inkscape::GC::anchor(repr);
 
     // Remove ourselves, not propagating delete events to avoid a
     // chain-reaction with other elements that might reference us.
     SP_OBJECT(use)->deleteObject(false);
 
     // Give the copy our old id and let go of our old repr.
-    sp_repr_set_attr(copy, "id", repr->attribute("id"));
-    sp_repr_unref(repr);
+    copy->setAttribute("id", repr->attribute("id"));
+    Inkscape::GC::release(repr);
 
     // Remove tiled clone attrs.
-    sp_repr_set_attr(copy, "inkscape:tiled-clone-of", NULL);
-    sp_repr_set_attr(copy, "inkscape:tile-w", NULL);
-    sp_repr_set_attr(copy, "inkscape:tile-h", NULL);
-    sp_repr_set_attr(copy, "inkscape:tile-cx", NULL);
-    sp_repr_set_attr(copy, "inkscape:tile-cy", NULL);
+    copy->setAttribute("inkscape:tiled-clone-of", NULL);
+    copy->setAttribute("inkscape:tile-w", NULL);
+    copy->setAttribute("inkscape:tile-h", NULL);
+    copy->setAttribute("inkscape:tile-cx", NULL);
+    copy->setAttribute("inkscape:tile-cy", NULL);
 
     // Establish the succession and let go of our object.
     SP_OBJECT(use)->setSuccessor(unlinked);
