@@ -11,11 +11,15 @@
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
-#include <glib.h>
-#include <libnr/nr-forward.h>
-#include <svg/svg-types.h>
-#include <string>
+#include <glib/gtypes.h>
+#include "svg/svg-length.h"
+#include <vector>
 
+struct NArtBpath;
+struct NRMatrix;
+namespace NR {
+    class Matrix;
+}
 
 /* Generic */
 
@@ -32,7 +36,7 @@ unsigned int sp_svg_number_read_d (const gchar *str, double *val);
 /*
  * No buffer overflow checking is done, so better wrap them if needed
  */
-unsigned int sp_svg_number_write_de (gchar *buf, double val, unsigned int tprec, unsigned int padf);
+unsigned int sp_svg_number_write_de (gchar *buf, double val, unsigned int tprec, int min_exp, unsigned int padf);
 
 /* Length */
 
@@ -45,17 +49,11 @@ unsigned int sp_svg_number_write_de (gchar *buf, double val, unsigned int tprec,
  * Any return value pointer can be NULL
  */
 
-unsigned int sp_svg_length_read (const gchar *str, SPSVGLength *length);
-unsigned int sp_svg_length_read_absolute (const gchar *str, SPSVGLength *length);
 unsigned int sp_svg_length_read_computed_absolute (const gchar *str, float *length);
-GList *sp_svg_length_list_read (const gchar *str);
-unsigned int sp_svg_length_read_ldd (const gchar *str, SPSVGLengthUnit *unit, double *value, double *computed);
+std::vector<SVGLength> sp_svg_length_list_read (const gchar *str);
+unsigned int sp_svg_length_read_ldd (const gchar *str, SVGLength::Unit *unit, double *value, double *computed);
 
-void sp_svg_length_set (SPSVGLength *length, SPSVGLengthUnit unit, float value, float computed);
-void sp_svg_length_unset (SPSVGLength *length, SPSVGLengthUnit unit, float value, float computed);
-void sp_svg_length_update (SPSVGLength *length, double em, double ex, double scale);
-
-std::string sp_svg_length_write_with_units(SPSVGLength const &length);
+std::string sp_svg_length_write_with_units(SVGLength const &length);
 
 bool sp_svg_transform_read(gchar const *str, NR::Matrix *transform);
 
@@ -64,13 +62,8 @@ unsigned sp_svg_transform_write(gchar str[], unsigned size, NRMatrix const *tran
 
 double sp_svg_read_percentage (const char * str, double def);
 
-unsigned int sp_svg_read_color (const gchar * str, unsigned int def);
-int sp_svg_write_color (char * buf, int buflen, unsigned int color);
-
 /* NB! As paths can be long, we use here dynamic string */
 
-#include <libnr/nr-path.h>
-
 NArtBpath * sp_svg_read_path (const char * str);
 char * sp_svg_write_path (const NArtBpath * bpath);
 
@@ -81,9 +74,9 @@ char * sp_svg_write_path (const NArtBpath * bpath);
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
