@@ -27,6 +27,10 @@ GrayMap *gdkPixbufToGrayMap(GdkPixbuf *buf);
 
 GdkPixbuf *grayMapToGdkPixbuf(GrayMap *grayMap);
 
+PackedPixelMap *gdkPixbufToPackedPixelMap(GdkPixbuf *buf);
+
+GdkPixbuf *packedPixelMapToGdkPixbuf(PackedPixelMap *ppMap);
+
 RgbMap *gdkPixbufToRgbMap(GdkPixbuf *buf);
 
 GdkPixbuf *rgbMapToGdkPixbuf(RgbMap *rgbMap);
