@@ -15,29 +15,22 @@
 
 #include "inkscape-potrace.h"
 
+#include <glibmm/i18n.h>
+#include <gtkmm.h>
 
 #include "trace/filterset.h"
 #include "trace/imagemap-gdk.h"
 
 #include <inkscape.h>
 #include <desktop-handles.h>
-#include <document.h>
 #include "message-stack.h"
-#include <selection.h>
-#include <sp-image.h>
 #include <sp-path.h>
 #include <svg/stringstream.h>
-#include <xml/repr.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <glibmm/i18n.h>
-
-#include <gtkmm.h>
-
-#include "greymap.h"
 #include "curve.h"
-#include "potracelib.h"
 #include "bitmap.h"
 
+
+
 static void updateGui()
 {
    //## Allow the GUI to update
@@ -292,12 +285,12 @@ filterIndexed(PotraceTracingEngine &engine, GdkPixbuf * pixbuf)
         if (engine.getMultiScanSmooth())
             {
             RgbMap *gaussMap = rgbMapGaussian(gm);
-            newGm = rgbMapQuantize(gaussMap, 4, engine.getMultiScanNrColors());
+            newGm = rgbMapQuantize(gaussMap,  (int)log2(engine.getMultiScanNrColors())+2, engine.getMultiScanNrColors());
             gaussMap->destroy(gaussMap);
             }
         else
             {
-            newGm = rgbMapQuantize(gm, 4, engine.getMultiScanNrColors());
+            newGm = rgbMapQuantize(gm,  (int)log2(engine.getMultiScanNrColors())+2, engine.getMultiScanNrColors());
             }
         gm->destroy(gm);
         }
@@ -309,12 +302,12 @@ filterIndexed(PotraceTracingEngine &engine, GdkPixbuf * pixbuf)
         if (engine.getMultiScanSmooth())
             {
             RgbMap *gaussMap = rgbMapGaussian(gm);
-            newGm = rgbMapQuantize(gaussMap, 4, engine.getMultiScanNrColors());
+            newGm = rgbMapQuantize(gaussMap, (int)log2(engine.getMultiScanNrColors())+2, engine.getMultiScanNrColors());
             gaussMap->destroy(gaussMap);
             }
         else
             {
-            newGm = rgbMapQuantize(gm, 4, engine.getMultiScanNrColors());
+           newGm = rgbMapQuantize(gm, (int)log2(engine.getMultiScanNrColors())+2, engine.getMultiScanNrColors());
             }
         gm->destroy(gm);
 
@@ -334,17 +327,20 @@ filterIndexed(PotraceTracingEngine &engine, GdkPixbuf * pixbuf)
 
 
 
-GdkPixbuf *
-PotraceTracingEngine::preview(GdkPixbuf * pixbuf)
+Glib::RefPtr<Gdk::Pixbuf>
+PotraceTracingEngine::preview(Glib::RefPtr<Gdk::Pixbuf> thePixbuf)
 {
+    GdkPixbuf *pixbuf = thePixbuf->gobj();
+
     if ( traceType == TRACE_QUANT_COLOR ||
          traceType == TRACE_QUANT_MONO   )
         {
         IndexedMap *gm = filterIndexed(*this, pixbuf);
         if (!gm)
-            return NULL;
+            return Glib::RefPtr<Gdk::Pixbuf>(NULL);
 
-        GdkPixbuf *newBuf = indexedMapToGdkPixbuf(gm);
+        Glib::RefPtr<Gdk::Pixbuf> newBuf =
+             Glib::wrap(indexedMapToGdkPixbuf(gm), false);
 
         gm->destroy(gm);
 
@@ -354,9 +350,10 @@ PotraceTracingEngine::preview(GdkPixbuf * pixbuf)
         {
         GrayMap *gm = filter(*this, pixbuf);
         if (!gm)
-            return NULL;
+            return Glib::RefPtr<Gdk::Pixbuf>(NULL);
 
-        GdkPixbuf *newBuf = grayMapToGdkPixbuf(gm);
+        Glib::RefPtr<Gdk::Pixbuf> newBuf =
+            Glib::wrap(grayMapToGdkPixbuf(gm), false);
 
         gm->destroy(gm);
 
@@ -383,7 +380,7 @@ char *PotraceTracingEngine::grayMapToPath(GrayMap *grayMap, long *nodeCount)
         {
         for (int x=0 ; x<grayMap->width ; x++)
             {
-            BM_UPUT(potraceBitmap, x, y, 
+            BM_UPUT(potraceBitmap, x, y,
                   grayMap->getPixel(grayMap, x, y) ? 0 : 1);
             }
         }
@@ -402,7 +399,7 @@ char *PotraceTracingEngine::grayMapToPath(GrayMap *grayMap, long *nodeCount)
         }
 
     /* trace a bitmap*/
-    potrace_state_t *potraceState = potrace_trace(potraceParams, 
+    potrace_state_t *potraceState = potrace_trace(potraceParams,
 			                          potraceBitmap);
 
     //## Free the Potrace bitmap
@@ -435,7 +432,7 @@ char *PotraceTracingEngine::grayMapToPath(GrayMap *grayMap, long *nodeCount)
     if ( nodeCount)
         *nodeCount = thisNodeCount;
 
-    
+
     return d;
 
 }
@@ -551,7 +548,7 @@ PotraceTracingEngine::traceBrightnessMulti(GdkPixbuf * thePixbuf, int *nrPaths)
         if (desktop)
             {
             gchar *msg = g_strdup_printf(_("Trace: %d.  %ld nodes"), traceCount++, nodeCount);
-            SP_DT_MSGSTACK(desktop)->flash(Inkscape::NORMAL_MESSAGE, msg);
+            sp_desktop_message_stack(desktop)->flash(Inkscape::NORMAL_MESSAGE, msg);
             g_free(msg);
             }
         }
@@ -652,7 +649,7 @@ PotraceTracingEngine::traceQuant(GdkPixbuf * thePixbuf, int *nrPaths)
         if (desktop)
             {
             gchar *msg = g_strdup_printf(_("Trace: %d.  %ld nodes"), colorIndex, nodeCount);
-            SP_DT_MSGSTACK(desktop)->flash(Inkscape::NORMAL_MESSAGE, msg);
+            sp_desktop_message_stack(desktop)->flash(Inkscape::NORMAL_MESSAGE, msg);
             g_free(msg);
             }
 
@@ -676,9 +673,12 @@ PotraceTracingEngine::traceQuant(GdkPixbuf * thePixbuf, int *nrPaths)
  *  of an SVG <path> element.
  */
 TracingEngineResult *
-PotraceTracingEngine::trace(GdkPixbuf * thePixbuf, int *nrPaths)
+PotraceTracingEngine::trace(Glib::RefPtr<Gdk::Pixbuf> pixbuf,
+                            int *nrPaths)
 {
 
+    GdkPixbuf *thePixbuf = pixbuf->gobj();
+
     //Set up for messages
     keepGoing             = 1;
 
