@@ -22,7 +22,7 @@ namespace Dialog {
 
 Find::Find() 
     : Dialog ("dialogs.find", SP_VERB_DIALOG_FIND),
-      _page_find("Find", 1, 1)
+      _page_find(1, 1)
 {
     // Top level vbox
     Gtk::VBox *vbox = get_vbox();
@@ -50,9 +50,9 @@ Find::~Find()
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
