@@ -22,8 +22,8 @@ namespace Dialog {
 
 TextProperties::TextProperties() 
     : Dialog ("dialogs.textandfont", SP_VERB_DIALOG_TEXT),
-      _page_font("Font", 1, 1),
-      _page_text("Text", 1, 1)
+      _page_font(1, 1),
+      _page_text(1, 1)
 {
     // Top level vbox
     Gtk::VBox *vbox = get_vbox();
@@ -51,9 +51,9 @@ TextProperties::~TextProperties()
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
