@@ -15,10 +15,7 @@
 
 #include <gtkmm/iconfactory.h>
 #include <gtkmm/stock.h>
-#include <glibmm/miscutils.h>
-#include <vector>
 
-#include "icons.h"
 #include "stock.h"
 #include "path-prefix.h"
 
@@ -691,9 +688,9 @@ get_icon_path(char const *const utf8_basename)
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
