@@ -14,65 +14,35 @@
 # include "config.h"
 #endif
 
-#include <string.h>
-#include <glib.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkaccelgroup.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkselection.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtktooltips.h>
-#include <gtk/gtkdnd.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkhandlebox.h>
-#include <gdk/gdkkeysyms.h>
 
-#include "macros.h"
-#include "helper/window.h"
-#include "widgets/icon.h"
 #include "widgets/button.h"
 #include "widgets/spw-utilities.h"
 #include "widgets/widget-sizes.h"
 #include "widgets/spinbutton-events.h"
+#include "widgets/icon.h"
+#include "widgets/sp-widget.h"
 
 #include "prefs-utils.h"
-#include "inkscape-stock.h"
-#include "file.h"
 #include "selection-chemistry.h"
-#include "path-chemistry.h"
-#include "inkscape-private.h"
 #include "document.h"
 #include "inkscape.h"
-#include "sp-object.h"
 #include "desktop-style.h"
 #include "desktop.h"
 #include "desktop-handles.h"
 #include "sp-namedview.h"
-#include "interface.h"
 #include "toolbox.h"
-#include "helper/gnome-utils.h"
 #include <glibmm/i18n.h>
-#include "widgets/sp-widget.h"
 #include "helper/unit-menu.h"
 #include "helper/units.h"
-#include "helper/window.h"
-#include "widgets/sp-widget.h"
-#include "macros.h"
 #include "inkscape.h"
 #include "verbs.h"
-#include "interface.h"
-#include "inkscape-stock.h"
 #include "prefs-utils.h"
 #include "selection.h"
 #include "selection-chemistry.h"
-#include "widgets/desktop-widget.h"
 #include "sp-item-transform.h"
-#include "libnr/nr-matrix.h"
-#include "libnr/nr-matrix-ops.h"
-#include "libnr/nr-translate-scale-ops.h"
-#include "libnr/nr-matrix-translate-ops.h"
+#include "message-stack.h"
 
-#include "select-toolbar.h"
 
 static void
 sp_selection_layout_widget_update(SPWidget *spw, Inkscape::Selection *sel)
@@ -130,7 +100,7 @@ static void
 sp_selection_layout_widget_modify_selection(SPWidget *spw, Inkscape::Selection *selection, guint flags, gpointer data)
 {
     SPDesktop *desktop = (SPDesktop *) data;
-    if ((SP_DT_SELECTION(desktop) == selection) // only respond to changes in our desktop
+    if ((sp_desktop_selection(desktop) == selection) // only respond to changes in our desktop
         && (flags & (SP_OBJECT_MODIFIED_FLAG        |
                      SP_OBJECT_PARENT_MODIFIED_FLAG |
                      SP_OBJECT_CHILD_MODIFIED_FLAG   )))
@@ -143,7 +113,7 @@ static void
 sp_selection_layout_widget_change_selection(SPWidget *spw, Inkscape::Selection *selection, gpointer data)
 {
     SPDesktop *desktop = (SPDesktop *) data;
-    if (SP_DT_SELECTION(desktop) == selection) // only respond to changes in our desktop
+    if (sp_desktop_selection(desktop) == selection) // only respond to changes in our desktop
         sp_selection_layout_widget_update(spw, selection);
 }
 
@@ -166,8 +136,10 @@ sp_object_layout_any_value_changed(GtkAdjustment *adj, SPWidget *spw)
     gtk_object_set_data(GTK_OBJECT(spw), "update", GINT_TO_POINTER(TRUE));
 
     SPDesktop *desktop = SP_ACTIVE_DESKTOP;
-    Inkscape::Selection *selection = SP_DT_SELECTION(desktop);
-    SPDocument *document = SP_DT_DOCUMENT(desktop);
+    Inkscape::Selection *selection = sp_desktop_selection(desktop);
+    SPDocument *document = sp_desktop_document(desktop);
+
+    sp_document_ensure_up_to_date (document);
     NR::Rect bbox = selection->bounds();
 
     if (!((bbox.max()[NR::X] - bbox.min()[NR::X] > 1e-6) || (bbox.max()[NR::Y] - bbox.min()[NR::Y] > 1e-6))) {
@@ -175,6 +147,8 @@ sp_object_layout_any_value_changed(GtkAdjustment *adj, SPWidget *spw)
     }
 
     gdouble x0, y0, x1, y1, xrel, yrel;
+    GtkAdjustment *a_w;
+    GtkAdjustment *a_h;
 
     if (unit.base == SP_UNIT_ABSOLUTE || unit.base == SP_UNIT_DEVICE) {
         GtkAdjustment *a;
@@ -182,12 +156,12 @@ sp_object_layout_any_value_changed(GtkAdjustment *adj, SPWidget *spw)
         x0 = sp_units_get_pixels (a->value, unit);
         a = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "Y");
         y0 = sp_units_get_pixels (a->value, unit);
-        a = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "width");
-        x1 = x0 + sp_units_get_pixels (a->value, unit);
-        xrel = sp_units_get_pixels (a->value, unit) / bbox.extent(NR::X);
-        a = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "height");
-        y1 = y0 + sp_units_get_pixels (a->value, unit);
-        yrel = sp_units_get_pixels (a->value, unit) / bbox.extent(NR::Y);
+        a_w = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "width");
+        x1 = x0 + sp_units_get_pixels (a_w->value, unit);
+        xrel = sp_units_get_pixels (a_w->value, unit) / bbox.extent(NR::X);
+        a_h = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "height");
+        y1 = y0 + sp_units_get_pixels (a_h->value, unit);
+        yrel = sp_units_get_pixels (a_h->value, unit) / bbox.extent(NR::Y);
     } else {
         GtkAdjustment *a;
         a = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "X");
@@ -196,20 +170,20 @@ sp_object_layout_any_value_changed(GtkAdjustment *adj, SPWidget *spw)
         a = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "Y");
         double const y0_propn = a->value * unit.unittobase;
         y0 = y0_propn * bbox.min()[NR::Y];
-        a = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "width");
-        xrel = a->value * unit.unittobase;
+        a_w = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "width");
+        xrel = a_w->value * unit.unittobase;
         x1 = x0 + xrel * bbox.extent(NR::X);
-        a = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "height");
-        yrel = a->value * unit.unittobase;
+        a_h = (GtkAdjustment *) gtk_object_get_data(GTK_OBJECT(spw), "height");
+        yrel = a_h->value * unit.unittobase;
         y1 = y0 + yrel * bbox.extent(NR::Y);
     }
 
     // Keep proportions if lock is on
     GtkWidget *lock = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(spw), "lock"));
     if (SP_BUTTON_IS_DOWN(lock)) {
-        if (fabs(xrel - 1.0) < 1e-6) {
+        if (adj == a_h) {
             x1 = x0 + yrel * bbox.extent(NR::X);
-        } else if (fabs(yrel - 1.0) < 1e-6) {
+        } else if (adj == a_w) {
             y1 = y0 + xrel * bbox.extent(NR::Y);
         }
     }
@@ -239,73 +213,12 @@ sp_object_layout_any_value_changed(GtkAdjustment *adj, SPWidget *spw)
                                      sv > 5e-4 ? "selector:toolbar:scale:vertical" : NULL );
 
     if (actionkey != NULL) {
-        NR::Matrix p2o = NR::Matrix (NR::translate (-bbox.min()));
-        NR::Matrix o2n = NR::Matrix (NR::translate (x0, y0));
-
-        NR::Matrix scale = NR::Matrix (NR::scale (1, 1));
-        NR::Matrix unbudge = NR::Matrix (NR::translate (0, 0));
-
-        int transform_stroke = prefs_get_int_attribute ("options.transform", "stroke", 1);
         gdouble strokewidth = stroke_average_width (selection->itemList());
+        int transform_stroke = prefs_get_int_attribute ("options.transform", "stroke", 1);
 
-        if ( fabs( bbox.extent(NR::X) ) > 1e-06 ) {
-            if (transform_stroke || strokewidth == NR_HUGE || 
-                fabs(x1 - x0 - strokewidth) < 1e-3 || fabs(bbox.extent(NR::X) - strokewidth) < 1e-3) {
-                NR::Matrix direct = NR::Matrix (NR::scale(( x1 - x0 ) / ( bbox.extent(NR::X) ), 1));
-                if (strokewidth == NR_HUGE || strokewidth == 0 || 
-                    fabs(bbox.extent(NR::X) - strokewidth) < 1e-3) {
-                    scale *= direct;
-                } else {
-                    // To find out the matrix to scale an object to get the given target dimension if the
-                    // stroke is scaled by the expansion of that matrix, we need to solve a
-                    // quadratic equation
-                    gdouble A = (bbox.extent(NR::X) - strokewidth)*(bbox.extent(NR::X) - strokewidth);
-                    gdouble B = -2 * (bbox.extent(NR::X) - strokewidth) * (x1 - x0) - strokewidth*strokewidth;
-                    gdouble C = (x1 - x0)*(x1 - x0);
-                    if (B*B - 4*A*C > 0) {
-                        gdouble sol2 = (-B - sqrt (B*B - 4*A*C))/(2*A);
-                        scale *= NR::scale(sol2, 1);
-                        unbudge *= NR::translate (0.5 * strokewidth * (sqrt(sol2) - sol2), 0);
-                    } else {
-                        scale *= direct;
-                    }
-                }
-
-            } else {
-                gdouble ratio = ( x1 - x0 - strokewidth) / ( bbox.extent(NR::X) - strokewidth);
-                scale *= NR::scale(ratio, 1);
-                unbudge *= NR::translate (0.5 * strokewidth * (1 - ratio), 0);
-            }
-        }
-
-        if ( fabs( bbox.extent(NR::Y) ) > 1e-06 ) {
-            if (transform_stroke || strokewidth == NR_HUGE || 
-                fabs(y1 - y0 - strokewidth) < 1e-3 || fabs(bbox.extent(NR::Y) - strokewidth) < 1e-3) {
-                NR::Matrix direct = NR::Matrix (NR::scale(1, ( y1 - y0 ) / ( bbox.extent(NR::Y) )));
-                if (strokewidth == NR_HUGE || strokewidth == 0 || 
-                    fabs(bbox.extent(NR::Y) - strokewidth) < 1e-3) {
-                    scale *= direct;
-                } else {
-                    gdouble A = (bbox.extent(NR::Y) - strokewidth)*(bbox.extent(NR::Y) - strokewidth);
-                    gdouble B = -2 * (bbox.extent(NR::Y) - strokewidth) * (y1 - y0) - strokewidth*strokewidth;
-                    gdouble C = (y1 - y0)*(y1 - y0);
-                    if (B*B - 4*A*C > 0) {
-                        gdouble sol2 = (-B - sqrt (B*B - 4*A*C))/(2*A);
-                        scale *= NR::scale(1, sol2);
-                        unbudge *= NR::translate (0, 0.5 * strokewidth * (sqrt(sol2) - sol2));
-                    } else {
-                        scale *= direct;
-                    }
-                }
-            } else {
-                gdouble ratio = ( y1 - y0 - strokewidth ) / ( bbox.extent(NR::Y) - strokewidth );
-                scale *= NR::scale(1, ratio);
-                unbudge *= NR::translate (0, 0.5 * strokewidth * (1 - ratio));
-            }
-        }
-
-        sp_selection_apply_affine(selection, p2o * scale * unbudge * o2n);
+        NR::Matrix scaler = get_scale_transform_with_stroke (bbox, strokewidth, transform_stroke, x0, y0, x1, y1);
 
+        sp_selection_apply_affine(selection, scaler);
         sp_document_maybe_done (document, actionkey);
 
         // defocus spinbuttons by moving focus to the canvas, unless "stay" is on
@@ -359,7 +272,7 @@ static gboolean aux_set_unit(SPUnitSelector *,
         return FALSE;
     }
 
-    Inkscape::Selection *selection = SP_DT_SELECTION(desktop);
+    Inkscape::Selection *selection = sp_desktop_selection(desktop);
 
     if (selection->isEmpty())
         return FALSE;
@@ -419,18 +332,58 @@ static gboolean aux_set_unit(SPUnitSelector *,
 
 static void toggle_stroke (GtkWidget *button, gpointer data) {
     prefs_set_int_attribute ("options.transform", "stroke", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) ? 1 : 0);
+    SPDesktop *desktop = (SPDesktop *)data;
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
+        desktop->messageStack()->flash(Inkscape::INFORMATION_MESSAGE, _("Now <b>stroke width</b> is <b>scaled</b> when objects are scaled."));
+    } else {
+        desktop->messageStack()->flash(Inkscape::INFORMATION_MESSAGE, _("Now <b>stroke width</b> is <b>not scaled</b> when objects are scaled."));
+    }
 }
 
 static void toggle_corners (GtkWidget *button, gpointer data) {
     prefs_set_int_attribute ("options.transform", "rectcorners", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) ? 1 : 0);
+    SPDesktop *desktop = (SPDesktop *)data;
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
+        desktop->messageStack()->flash(Inkscape::INFORMATION_MESSAGE, _("Now <b>rounded rectangle corners</b> are <b>scaled</b> when rectangles are scaled."));
+    } else {
+        desktop->messageStack()->flash(Inkscape::INFORMATION_MESSAGE, _("Now <b>rounded rectangle corners</b> are <b>not scaled</b> when rectangles are scaled."));
+    }
 }
 
 static void toggle_gradient (GtkWidget *button, gpointer data) {
     prefs_set_int_attribute ("options.transform", "gradient", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) ? 1 : 0);
+    SPDesktop *desktop = (SPDesktop *)data;
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
+        desktop->messageStack()->flash(Inkscape::INFORMATION_MESSAGE, _("Now <b>gradients</b> are <b>transformed</b> along with their objects when those are transformed (moved, scaled, rotated, or skewed)."));
+    } else {
+        desktop->messageStack()->flash(Inkscape::INFORMATION_MESSAGE, _("Now <b>gradients</b> remain <b>fixed</b> when objects are transformed (moved, scaled, rotated, or skewed)."));
+    }
 }
 
 static void toggle_pattern (GtkWidget *button, gpointer data) {
     prefs_set_int_attribute ("options.transform", "pattern", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) ? 1 : 0);
+    SPDesktop *desktop = (SPDesktop *)data;
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
+        desktop->messageStack()->flash(Inkscape::INFORMATION_MESSAGE, _("Now <b>patterns</b> are <b>transformed</b> along with their objects when those are transformed (moved, scaled, rotated, or skewed)."));
+    } else {
+        desktop->messageStack()->flash(Inkscape::INFORMATION_MESSAGE, _("Now <b>patterns</b> remain <b>fixed</b> when objects are transformed (moved, scaled, rotated, or skewed)."));
+    }
+}
+
+static void toggle_lock (GtkWidget *button, gpointer data) {
+
+    GtkWidget *old_child = gtk_bin_get_child(GTK_BIN(button));
+    gtk_container_remove (GTK_CONTAINER(button), old_child);
+
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
+        GtkWidget *child = sp_icon_new (Inkscape::ICON_SIZE_DECORATION, "width_height_lock");
+        gtk_widget_show (child);
+        gtk_container_add (GTK_CONTAINER (button), child);
+    } else {
+        GtkWidget *child = sp_icon_new (Inkscape::ICON_SIZE_DECORATION, "lock_unlocked");
+        gtk_widget_show (child);
+        gtk_container_add (GTK_CONTAINER (button), child);
+    }
 }
 
 GtkWidget *
@@ -441,23 +394,23 @@ sp_select_toolbox_new(SPDesktop *desktop)
     GtkTooltips *tt = gtk_tooltips_new();
     GtkWidget *tb = gtk_hbox_new(FALSE, 0);
 
-    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_ROTATE_90_CCW), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_ROTATE_90_CW), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_FLIP_HORIZONTAL), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_FLIP_VERTICAL), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, Inkscape::ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_ROTATE_90_CCW), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, Inkscape::ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_ROTATE_90_CW), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, Inkscape::ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_FLIP_HORIZONTAL), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, Inkscape::ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_OBJECT_FLIP_VERTICAL), view, tt);
 
     aux_toolbox_space(tb, AUX_BETWEEN_BUTTON_GROUPS);
 
-    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_TO_BACK), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_LOWER), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_RAISE), view, tt);
-    sp_toolbox_button_normal_new_from_verb(tb, GTK_ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_TO_FRONT), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, Inkscape::ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_TO_BACK), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, Inkscape::ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_LOWER), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, Inkscape::ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_RAISE), view, tt);
+    sp_toolbox_button_normal_new_from_verb(tb, Inkscape::ICON_SIZE_SMALL_TOOLBAR, Inkscape::Verb::get(SP_VERB_SELECTION_TO_FRONT), view, tt);
 
     // Create the parent widget for x y w h tracker.
     GtkWidget *spw = sp_widget_new_global(INKSCAPE);
 
     // Remember the desktop's canvas widget, to be used for defocusing.
-    gtk_object_set_data(GTK_OBJECT(spw), "dtw", SP_DT_CANVAS(desktop));
+    gtk_object_set_data(GTK_OBJECT(spw), "dtw", sp_desktop_canvas(desktop));
 
     // The vb frame holds all other widgets and is used to set sensitivity depending on selection state.
     GtkWidget *vb = gtk_hbox_new(FALSE, 0);
@@ -469,7 +422,7 @@ sp_select_toolbox_new(SPDesktop *desktop)
     GtkWidget *us = sp_unit_selector_new(SP_UNIT_ABSOLUTE | SP_UNIT_DEVICE);
     sp_unit_selector_setsize(us, AUX_OPTION_MENU_WIDTH, AUX_OPTION_MENU_HEIGHT);
     sp_unit_selector_add_unit(SP_UNIT_SELECTOR(us), &sp_unit_get_by_id(SP_UNIT_PERCENT), 0);
-    sp_unit_selector_set_unit (SP_UNIT_SELECTOR(us), SP_DT_NAMEDVIEW(desktop)->doc_units);
+    sp_unit_selector_set_unit (SP_UNIT_SELECTOR(us), sp_desktop_namedview(desktop)->doc_units);
     g_signal_connect(G_OBJECT(us), "set_unit", G_CALLBACK(aux_set_unit), spw);
 
     // four spinbuttons
@@ -492,15 +445,16 @@ sp_select_toolbox_new(SPDesktop *desktop)
 
     // lock toggle
     GtkWidget *lockbox = gtk_vbox_new(TRUE, 0);
-    GtkWidget *lock = sp_button_new_from_data( GTK_ICON_SIZE_MENU,
+    GtkWidget *lock = sp_button_new_from_data( Inkscape::ICON_SIZE_DECORATION,
                                               SP_BUTTON_TYPE_TOGGLE,
                                               NULL,
-                                              "width_height_lock",
-                                              _("Change both width and height by the same proportion"),
+                                              "lock_unlocked",
+                                              _("When locked, change both width and height by the same proportion"),
                                               tt);
     gtk_box_pack_start(GTK_BOX(lockbox), lock, TRUE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vb), lockbox, FALSE, FALSE, 0);
     gtk_object_set_data(GTK_OBJECT(spw), "lock", lock);
+    g_signal_connect_after (G_OBJECT (lock), "clicked", G_CALLBACK (toggle_lock), desktop);
 
     gtk_container_add(GTK_CONTAINER(vb),
                       //TRANSLATORS: only translate "string" in "context|string". 
@@ -522,7 +476,7 @@ sp_select_toolbox_new(SPDesktop *desktop)
     gtk_signal_connect(GTK_OBJECT(spw), "change_selection", GTK_SIGNAL_FUNC(sp_selection_layout_widget_change_selection), desktop);
 
     // Update now.
-    sp_selection_layout_widget_update(SP_WIDGET(spw), SP_ACTIVE_DESKTOP ? SP_DT_SELECTION(SP_ACTIVE_DESKTOP) : NULL);
+    sp_selection_layout_widget_update(SP_WIDGET(spw), SP_ACTIVE_DESKTOP ? sp_desktop_selection(SP_ACTIVE_DESKTOP) : NULL);
 
     // Insert spw into the toolbar.
     gtk_box_pack_start(GTK_BOX(tb), spw, FALSE, FALSE, AUX_BETWEEN_BUTTON_GROUPS);
@@ -535,50 +489,50 @@ sp_select_toolbox_new(SPDesktop *desktop)
     GtkWidget *cbox = gtk_hbox_new (FALSE, 0);
 
     {
-    GtkWidget *button = sp_button_new_from_data( GTK_ICON_SIZE_SMALL_TOOLBAR,
+    GtkWidget *button = sp_button_new_from_data( Inkscape::ICON_SIZE_DECORATION,
                                               SP_BUTTON_TYPE_TOGGLE,
                                               NULL,
                                               "transform_stroke",
                                               _("When scaling objects, scale the stroke width by the same proportion"),
                                               tt);
-    g_signal_connect_after (G_OBJECT (button), "clicked", G_CALLBACK (toggle_stroke), NULL);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), prefs_get_int_attribute ("options.transform", "stroke", 1));
+    g_signal_connect_after (G_OBJECT (button), "clicked", G_CALLBACK (toggle_stroke), desktop);
     gtk_box_pack_start(GTK_BOX(cbox), button, FALSE, FALSE, 0);
     }
 
     {
-    GtkWidget *button = sp_button_new_from_data( GTK_ICON_SIZE_SMALL_TOOLBAR,
+    GtkWidget *button = sp_button_new_from_data( Inkscape::ICON_SIZE_DECORATION,
                                               SP_BUTTON_TYPE_TOGGLE,
                                               NULL,
                                               "transform_corners",
                                               _("When scaling rectangles, scale the radii of rounded corners"),
                                               tt);
-    g_signal_connect_after (G_OBJECT (button), "clicked", G_CALLBACK (toggle_corners), NULL);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), prefs_get_int_attribute ("options.transform", "rectcorners", 1));
+    g_signal_connect_after (G_OBJECT (button), "clicked", G_CALLBACK (toggle_corners), desktop);
     gtk_box_pack_start(GTK_BOX(cbox), button, FALSE, FALSE, 0);
     }
 
     {
-    GtkWidget *button = sp_button_new_from_data( GTK_ICON_SIZE_SMALL_TOOLBAR,
+    GtkWidget *button = sp_button_new_from_data( Inkscape::ICON_SIZE_DECORATION,
                                               SP_BUTTON_TYPE_TOGGLE,
                                               NULL,
                                               "transform_gradient",
                                               _("Transform gradients (in fill or stroke) along with the objects"),
                                               tt);
-    g_signal_connect_after (G_OBJECT (button), "clicked", G_CALLBACK (toggle_gradient), NULL);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), prefs_get_int_attribute ("options.transform", "gradient", 1));
+    g_signal_connect_after (G_OBJECT (button), "clicked", G_CALLBACK (toggle_gradient), desktop);
     gtk_box_pack_start(GTK_BOX(cbox), button, FALSE, FALSE, 0);
     }
 
     {
-    GtkWidget *button = sp_button_new_from_data( GTK_ICON_SIZE_SMALL_TOOLBAR,
+    GtkWidget *button = sp_button_new_from_data( Inkscape::ICON_SIZE_DECORATION,
                                               SP_BUTTON_TYPE_TOGGLE,
                                               NULL,
                                               "transform_pattern",
                                               _("Transform patterns (in fill or stroke) along with the objects"),
                                               tt);
-    g_signal_connect_after (G_OBJECT (button), "clicked", G_CALLBACK (toggle_pattern), NULL);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), prefs_get_int_attribute ("options.transform", "pattern", 1));
+    g_signal_connect_after (G_OBJECT (button), "clicked", G_CALLBACK (toggle_pattern), desktop);
     gtk_box_pack_start(GTK_BOX(cbox), button, FALSE, FALSE, 0);
     }
 
