@@ -2,20 +2,12 @@
 # include "config.h"
 #endif
 #include <math.h>
-#include <string.h>
-#include <stdlib.h>
 #include <gtk/gtksignal.h>
-#include <gtk/gtkhbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkspinbutton.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkcolorsel.h>
-#include <gtk/gtktooltips.h>
-#include "../color.h"
 #include <glibmm/i18n.h>
 #include "../dialogs/dialog-events.h"
-#include "sp-color-preview.h"
 #include "sp-color-wheel-selector.h"
 #include "sp-color-scales.h"
 
@@ -295,9 +287,9 @@ void ColorWheelSelector::_wheelChanged( SPColorWheel *wheel, SPColorWheelSelecto
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
