@@ -13,17 +13,9 @@
  * This code is in public domain
  */
 
-#include <gdk/gdk.h>
-#include <gdk/gdkrgb.h>
 #include <gtk/gtksignal.h>
-#include <gtk/gtkmain.h>
 #include "sp-color-wheel.h"
-#include "math.h"
-#include "color.h"
-#include "string.h" // for memcpy
-#include "gtk/gtkwindow.h"
 
-#include "libnr/nr-matrix.h"
 #include "libnr/nr-rotate-ops.h"
 
 #define WHEEL_SIZE 96
@@ -1163,9 +1155,9 @@ static void sp_color_wheel_process_in_triangle( SPColorWheel *wheel, gdouble x,
   Local Variables:
   mode:c++
   c-file-style:"stroustrup"
-  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
   indent-tabs-mode:nil
   fill-column:99
   End:
 */
-// vim: filetype=c++:expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
