@@ -22,6 +22,7 @@
 #include "xml/sp-css-attr.h"
 
 #define SP_SODIPODI_NS_URI "http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
+#define SP_BROKEN_SODIPODI_NS_URI "http://inkscape.sourceforge.net/DTD/sodipodi-0.dtd"
 #define SP_INKSCAPE_NS_URI "http://www.inkscape.org/namespaces/inkscape"
 #define SP_XLINK_NS_URI "http://www.w3.org/1999/xlink"
 #define SP_SVG_NS_URI "http://www.w3.org/2000/svg"
@@ -134,14 +135,6 @@ Inkscape::XML::Node *sp_repr_new(gchar const *name);
 Inkscape::XML::Node *sp_repr_new_text(gchar const *content);
 Inkscape::XML::Node *sp_repr_new_comment(gchar const *comment);
 
-/// Make \a repr an anchored object. See Inkscape::GC::Anchored.
-inline Inkscape::XML::Node *sp_repr_ref(Inkscape::XML::Node *repr) {
-    return Inkscape::GC::anchor(repr);
-}
-inline Inkscape::XML::Node *sp_repr_unref(Inkscape::XML::Node *repr) {
-    Inkscape::GC::release(repr);
-    return NULL;
-}
 /*inline Inkscape::XML::Node *sp_repr_duplicate(Inkscape::XML::Node const *repr) {
     return repr->duplicate();
 }*/
@@ -182,19 +175,6 @@ inline Inkscape::XML::Node *sp_repr_next(Inkscape::XML::Node *repr) {
     return ( repr ? repr->next() : NULL );
 }
 
-/// Add \a child node to children of \a repr before \a ref.
-inline unsigned sp_repr_add_child(Inkscape::XML::Node *repr, Inkscape::XML::Node *child, Inkscape::XML::Node *ref)
-{
-    repr->addChild(child, ref);
-    return true;
-}
-
-/// Remove \a child from children of \a repr.
-inline unsigned sp_repr_remove_child(Inkscape::XML::Node *repr, Inkscape::XML::Node *child) {
-    repr->removeChild(child);
-    return true;
-}
-
 /* IO */
 
 Inkscape::XML::Document *sp_repr_read_file(gchar const *filename, gchar const *default_ns);
@@ -214,8 +194,10 @@ SPCSSAttr *sp_repr_css_attr_inherited(Inkscape::XML::Node *repr, gchar const *at
 gchar const *sp_repr_css_property(SPCSSAttr *css, gchar const *name, gchar const *defval);
 void sp_repr_css_set_property(SPCSSAttr *css, gchar const *name, gchar const *value);
 void sp_repr_css_unset_property(SPCSSAttr *css, gchar const *name);
+bool sp_repr_css_property_is_unset(SPCSSAttr *css, gchar const *name);
 double sp_repr_css_double_property(SPCSSAttr *css, gchar const *name, double defval);
 
+gchar *sp_repr_css_write_string(SPCSSAttr *css);
 void sp_repr_css_set(Inkscape::XML::Node *repr, SPCSSAttr *css, gchar const *key);
 void sp_repr_css_merge(SPCSSAttr *dst, SPCSSAttr *src);
 void sp_repr_css_attr_add_from_string(SPCSSAttr *css, const gchar *data);
@@ -258,13 +240,6 @@ Inkscape::XML::Node *sp_repr_lookup_child(Inkscape::XML::Node *repr,
                                           gchar const *key,
                                           gchar const *value);
 
-/// Put \a child after \a ref in list of \a repr's children.
-inline unsigned sp_repr_change_order(Inkscape::XML::Node *repr,
-                                     Inkscape::XML::Node *child,
-                                     Inkscape::XML::Node *ref) {
-    repr->changeOrder(child, ref);
-    return true;
-}
 
 Inkscape::XML::Document *sp_repr_document_new_list(GSList *reprs);
 
